/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @version 0.83
 * @list of changes:
 * 2008-10-00: Hartmut Schorrig creation
 *
 * @author  Hartmut Schorrig 
 * @content Some routines to convert several data to string representation
 *          at low C-level, able to use without any other complex library concepts
 * @dependencies: libOSAL concept.
 * @note:   The macros STACKTRC_... may be defined in a simple way, 
 *          if the stacktrace-exception handling won't be use.
 *
 ****************************************************************************/

#include "fw_Formatter.h"
//#include "fw_Exception.h"       //STACKTRC_...
#include "fw_SimpleC.h"         //ARRAYLEN
//#include "fw_ThreadContext.h"   //os_getCurrentStacktraceThreadContext()
#include <string.h>             //strchr(), strlen(), memset()


char const typedVaArg_VaArgBuffer[] = "typedVariableArgumentList_ObjectJc";


int toStringFormat_Fw(char* buffer, int sizeBuffer, OS_TimeStamp const* time, char const* sFormat, int timeZoneAdjustHours)
{
  int32 seconds = time->time_sec; 
  int32 nanos = time->time_nsec;
  int lenFormat = strlen(sFormat);
  int32 nHour, nMinute, nSec;
  int32 nYear4,nYear, nDays;  //NOTE: nDays is max 0xC22E, it should be either uint16 or int32.
  int8 nMonth;
  int nBaseYear = 1970;
  
  int posBuffer = 0;
  int maxPosBuffer = sizeBuffer;
  STACKTRC_ENTRY("toStringFormat_DateFw");
  //NO, do not intialize! init0p_MemC(buffer, sizeBuffer); 

  //convert the  continuously counting seconds in year .. second:
  //

  { static const uint16 aMonth[]    ={31,28,31,30,31,30,31,31,30,31,30,31};
    static const uint16 aMonthSkip[]={31,29,31,30,31,30,31,31,30,31,30,31};
    const uint16* pDayOfMonth;
    
    //Millesekunden des Tages
    nDays  =(uint16)(seconds/(24L*3600L));  //volle Tage
    nSec=seconds - nDays*(24L*3600L);

    nDays-=((nBaseYear%4)+1)*365+1;  //Bezug auf 1.1. nach 1. Schaltjahr Bezug auf 1.1.1973, wenn 1970 das Baseyear ist. 
                                    //Hinweis: vor 1973 erfolgt falsche Darstellung
    nYear4=(int16)(nDays/(4*365+1));         //Anzahl volle 4 Jahre
    nDays-=nYear4*(4*365+1);        //verbleibt Restzeit nach Schaltjahr
    nYear=(int16)(nDays/365);                //volle Jahre nach Schaltjahr
    if(nYear == 4)
    { //at last day, dec 31 of the last year, what is a sleep year, the nrof days = 1460 = 3*365 + 366 -1;  
      nYear = 3;  //That are 3 years of a period with 3 non-sleep year and the sleep year, not 4! 
    }
    nDays-=nYear*365;               //verbleibende Tage im Jahr

    nMonth=0;                       //zun"chst Index auf aMonth
    pDayOfMonth=aMonth;
    if(nYear==3) pDayOfMonth=aMonthSkip; //Schaltjahrestabelle
    while(nDays>=pDayOfMonth[nMonth])    //wenn zuviele Tage f?r Monat (1. Tag ab 0)
    { nDays-=pDayOfMonth[nMonth];        //dann diese subtr.
      nMonth++;                          //und index incr.
    }
    nMonth+=(uint8)(1);                           //+1 da Index ab 0 war
    nDays+=(uint8)(1);                            //+1 da Anzahl Tage im Monat ab 0 war
    nYear+=(short)
           ( ((nBaseYear+3)/4)*4 +1      //1. Jahr nach Schaltjahr nach baseyear
             +4*nYear4);                 //Jahre absolut nach  Christi Geburt


    { nHour=nSec/3600L; nSec-=3600L*nHour;
      nMinute=nSec/60L;  nSec-=60L*nMinute;
    }
  }

  { static const char* sMonthsFull[12] = {"January", "February", "March", "April", "May", "June"
                       , "July", "August", "September", "October", "November", "December"};
    static const char* sMonthsShort[12] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun"
                       , "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
    char sprintfBuffer[8];  //no more as 4 digits, but respect 4-byte-boundary
    while(lenFormat > 0)
    { //read and increment, decrement lenFormat
      char cFormat = *sFormat;
      int lenFormatToken = 1;
      sFormat +=1;
      lenFormat -=1;

      //detect number of equal chars one after another.
      while(lenFormat > 0 && *sFormat == cFormat)
      { lenFormatToken +=1;
        sFormat +=1;
        lenFormat -=1;
      }

      switch(cFormat)
      {
        case 'Y':
        { char const* format;
          if(lenFormatToken >=4)
          { //append_i_StringBufferJc(&buffer, nYear);
            format = "%4.4i";
          }
          else if(lenFormatToken >=2)
          { nYear -= 2000;
            if(nYear < 0) { nYear += 100; }
            if(nYear >= 100) { nYear -= 100; }
            //if(nYear < 10)
            { //append_c_StringBufferJc(&buffer, '0');
            }
            format = "%2.2i";
          }
          else
          { format = null;
          }
          if(format != null)
          { 
            int nrofChars = sprintf(sprintfBuffer, format, nYear);
            if(nrofChars > (maxPosBuffer - posBuffer))
            { nrofChars = maxPosBuffer - posBuffer; //truncate it.
            }
            memcpy(buffer+posBuffer, sprintfBuffer, nrofChars);
            posBuffer += nrofChars;
          }  
        } break;
        case 'M':
        { if(lenFormatToken >3)
          { int nrofChars = strlen(sMonthsFull[nMonth-1]);
            if(nrofChars > (maxPosBuffer - posBuffer))
            { nrofChars = maxPosBuffer - posBuffer; //truncate it.
            }
            memcpy(buffer+posBuffer, sMonthsFull[nMonth-1], nrofChars);
            posBuffer += nrofChars;
          }
          else if(lenFormatToken ==3)
          { int nrofChars = strlen(sMonthsShort[nMonth-1]);
            if(nrofChars > (maxPosBuffer - posBuffer))
            { nrofChars = maxPosBuffer - posBuffer; //truncate it.
            }
            memcpy(buffer+posBuffer, sMonthsShort[nMonth-1], nrofChars);
            posBuffer += nrofChars;
          }
          else
          {
            int nrofChars = sprintf(sprintfBuffer, "%2.2i", nMonth);
            if(nrofChars > (maxPosBuffer - posBuffer))
            { nrofChars = maxPosBuffer - posBuffer; //truncate it.
            }
            memcpy(buffer+posBuffer, sprintfBuffer, nrofChars);
            posBuffer += nrofChars;
          }
        } break;
        case 'd':
        { int nrofChars = sprintf(sprintfBuffer, "%2.2i", (int)nDays);
          if(nrofChars > (maxPosBuffer - posBuffer))
          { nrofChars = maxPosBuffer - posBuffer; //truncate it.
          }
          memcpy(buffer+posBuffer, sprintfBuffer, nrofChars);
          posBuffer += nrofChars;
        } break;
        case 'H': 
        case 'h':  //temporary: convert h like H because application errors exists.
        { int nrofChars = sprintf(sprintfBuffer, "%2.2i", (int)nHour);
          if(nrofChars > (maxPosBuffer - posBuffer))
          { nrofChars = maxPosBuffer - posBuffer; //truncate it.
          }
          memcpy(buffer+posBuffer, sprintfBuffer, nrofChars);
          posBuffer += nrofChars;
        } break;
        case 'a':  //am/pm-Marker
        { const char* sAmPm = nHour >=12 ? "PM" : "AM";
          int nrofChars = 2;
          if(nrofChars > (maxPosBuffer - posBuffer))
          { nrofChars = maxPosBuffer - posBuffer; //truncate it.
          }
          memcpy(buffer+posBuffer, sAmPm, nrofChars);
          posBuffer += nrofChars;
        } break;
        case 'K':  //hour from 1 to 12
        { int nrofChars = sprintf(sprintfBuffer, "%2.2i", (int)(nHour >=12 ? nHour-11 : nHour+1));
          if(nrofChars > (maxPosBuffer - posBuffer))
          { nrofChars = maxPosBuffer - posBuffer; //truncate it.
          }
          memcpy(buffer+posBuffer, sprintfBuffer, nrofChars);
          posBuffer += nrofChars;
        } break;
        case 'm':
        { int nrofChars = sprintf(sprintfBuffer, "%2.2i", (int)nMinute);
          if(nrofChars > (maxPosBuffer - posBuffer))
          { nrofChars = maxPosBuffer - posBuffer; //truncate it.
          }
          memcpy(buffer+posBuffer, sprintfBuffer, nrofChars);
          posBuffer += nrofChars;
        } break;
        case 's':
        { int nrofChars = sprintf(sprintfBuffer, "%2.2i", (int)nSec);
          if(nrofChars > (maxPosBuffer - posBuffer))
          { nrofChars = maxPosBuffer - posBuffer; //truncate it.
          }
          memcpy(buffer+posBuffer, sprintfBuffer, nrofChars);
          posBuffer += nrofChars;
        } break;
        case 'S':  //millisecond
        { const char* format;
          int millisec = (int)(nanos/1000000L); 
          if(millisec < 0){ millisec +=1000; }
          switch(lenFormatToken)
          { case 1: format = "%1.1u"; millisec /= 100; break;
            case 2: format = "%2.2u"; millisec /= 10; break;
            default: format = "%3.3u"; break;
          }
          { int nrofChars = sprintf(sprintfBuffer, format, millisec);
            if(nrofChars > (maxPosBuffer - posBuffer))
            { nrofChars = maxPosBuffer - posBuffer; //truncate it.
            }
            memcpy(buffer+posBuffer, sprintfBuffer, nrofChars);
            posBuffer += nrofChars;
          }
        } break;
        default:
        { //transform the character itself.
          while(--lenFormatToken >=0 && posBuffer < maxPosBuffer)
          { buffer[posBuffer++] = cFormat;
          }
        }
      }
      ASSERT(posBuffer <= maxPosBuffer);  //detect errors.
    }
    //NO! The buffer may be a part of a string! buffer[posBuffer] = 0;    
  }
  STACKTRC_LEAVE; return posBuffer;
}




int strlen_Fwc(char const* text, int maxnrofChars)
{ char const* text1 = text;
  char const* text9 = text + maxnrofChars;
  //optimization: test only one pointer register, which is incremented too
  while(text1 < text9 && *text1 != 0){ text1+=1;}
  return (text1 - text);
}



int parsePrinfStyleString_fwFormatter(ParseResultPrintfStyle_fwFormatter* dst, int zDst, char const* src, int zSrc)
{ //int iDst = 0;
  int retError = 0;
  char const* sTextEnd = src + zSrc;  //exclusive end
  ParseResultPrintfStyle_fwFormatter* dstAct = dst;
  ParseResultPrintfStyle_fwFormatter* dstMax = dst + zDst; //exclusive end address
  char const* sText1 = src;
  STACKTRC_ENTRY("parsePrinfStyleString_fw_Formatter");
  memset(dst,0, zDst * sizeof(*dst));  //clear all content
  while(sText1 < sTextEnd)
  { if(dstAct >= dstMax)
    { //THROW_s0(IndexOutOfBoundsException, "to many %-chars (arguments) in text, 2*max=", zDst); 
    }
    ASSERT(dstAct < dstMax);
    { const char* sText2;
      sText2 = strchr(sText1, '%');
      if(sText2 == null)
      { sText2 = sTextEnd; //use string to its end.
      }
      if(dstAct < dstMax && sText2 > sText1)
      { //any text from actual position to found '%' or to end.
        dstAct->text.text = sText1;
        dstAct->text.nrofChars = sText2 - sText1;
        dstAct->text.specifier = 't'; 
        dstAct +=1;       //next array element.
        sText1 = sText2;  //continue after it.
      }
    }
    if(sText1 < sTextEnd)
    { //'%' found
      int nrofChars = 0;
      int nrofChars2 = 0;
      char const* pos;
      char flag = 0;
      char lengthModifier = 0;
      char cFormat;
      
      if(dstAct >= dstMax)
      { //THROW_s0(IndexOutOfBoundsException, "to many %-chars (arguments) in text, 2*max=", zDst); 
      }
      ASSERT(dstAct < dstMax);
      sText1 +=1;   //skip over '%'
      cFormat = *(sText1++);
      pos = strchr("-+ #0", cFormat);   //test of flags
      if(pos != null)
      { flag = cFormat;
        cFormat = *sText1++;
      }
      while(cFormat >= '0' && cFormat <= '9')
      { nrofChars = 10 * nrofChars + (cFormat - '0');
        cFormat = *(sText1++);
      }
      if(cFormat == '.')
      { //precision
        cFormat = *sText1++;
        while(cFormat >= '0' && cFormat <= '9')
        { nrofChars2 = 10 * nrofChars2 + (cFormat - '0');
          cFormat = *sText1++;
        }
      }
      pos = strchr("hlL", cFormat);
      if(pos != null)
      { lengthModifier = cFormat;
        cFormat = *sText1++;
      }
      dstAct->value.width = nrofChars;
      dstAct->value.precision = nrofChars2;
      dstAct->value.specifier = cFormat;
      switch(cFormat)
      { case 'c': dstAct->value.argument = 'C'; break;
        case 's': dstAct->value.argument = 'z'; break;
        case 'd':
        case 'x':
        case 'X':
        case 'o':
        case 'u':
        case 'i': 
        { dstAct->value.argument = lengthModifier == 'h' ? 'S' : lengthModifier == 'l' ? 'J' : 'I'; 
          if(cFormat != 'u')
          { //the precision is designated negativ, if a negativ value should be displayed, see toString_Integer_FW(...)
            dstAct->value.precision = - dstAct->value.precision;
            if( dstAct->value.precision == 0)
            { dstAct->value.precision = -1;  //the minimum of characters for number.
            }
          }
        } break;
        case 'e':
        case 'E':
        case 'f':
        case 'g':
        case 'G': 
        { //NOTE: it is always an double, no possiblility of float.
          //The variables in the argument list are converted to double automatically.
          //long double is not supported yet.
          dstAct->value.argument = lengthModifier == 'L' ? 'D' : 'D'; 
        } break;
        case 'p': dstAct->value.argument = 'P'; dstAct->value.width = 8; dstAct->value.precision = 8; break;
        case 'n': dstAct->value.argument = 'N'; break;
        case '%': dstAct->value.argument = '%'; break;
        default: dstAct->value.argument = '?';
      }
      dstAct +=1;  //next array element.
    }

  }
  STACKTRC_LEAVE; return retError < 0 ? retError : dstAct - dst;
}



/**Returns a string of argument types from given printf-style-string. 
 * This method is necessary especially for routines which processed variable arguments in a special way,
 * but don't use the printf-style format string.
 * @param typeArgs a char[..] to affiliate the type chars.
 * @param zTypeArgs lenght of the char[...]
 * @param text the text containing the type characters in the printf-style.  
 * @return number of type args chars.
 */ 
int detectTypeArgs_fwFormatter(char* typeArgs, int zTypeArgs, StringJc text)
{
  /**The types are not given in the argument list, than parse the text to get that informations,
   * write first in the same text form as possible given in typeArgs, to handle in same way: */
  ParseResultPrintfStyle_fwFormatter parseResult[22]; //NOTE: regard parse Results as text between the values.
  int zText;
  char const* sText = getCharsAndLength_StringJc(&text, &zText);
  int nrofArgs = parsePrinfStyleString_fwFormatter(parseResult, ARRAYLEN(parseResult), sText, zText);
  int ii =0; int jj =0; 
  while( ii < nrofArgs && jj < zTypeArgs)
  { if(parseResult[ii].value.specifier != 't')
    {
      typeArgs[jj++] = parseResult[ii++].value.argument; 
    }
    else{ ii+=1; }
  }
  if(jj < zTypeArgs){ typeArgs[jj]= 0; } //terminate with 0. 
  return jj; 
}




/** found an algorithm unusing division, because some divisions may be expensive in time
  * at some processors.
*/
int toString_Integer_FW(char* buffer, int zBuffer, int32 value, int radix, int minNrofCharsAndNegative, ThCxt* _thCxt)
{ char cc;
  /**Array of values to test the position in digit. Fill it with 10, 100 etc if radix = 10;*/
  uint32 testValues[32]; //max 32
  uint32 uvalue; 

  int idxTestValues = 0;
  int nChars = 0;
  int minNrofChars;
  STACKTRC_TENTRY("insert_i_StringBufferJc");
  if(minNrofCharsAndNegative < 0 && value < 0)
  { uvalue = -value; //may be -0x80000000
    buffer[nChars++] = '-';
  }
  else
  { uvalue = value;
  }
  minNrofChars = minNrofCharsAndNegative < 0 ?  -minNrofCharsAndNegative : minNrofCharsAndNegative;
  //
  { //fill in digigts to detect positions in digit
    uint32 powerRadix = radix;
    uint32 powerRadixLast = 0;
    while(  uvalue >= powerRadix 
         && powerRadix > powerRadixLast  //prevent overflow.
         )
    { testValues[idxTestValues++] = powerRadix;
      powerRadixLast = powerRadix;
      powerRadix *= radix;
    }
  }
  //
  //idxTestValues is the number of digits -1, because testValues[0] contains 10.  
  { int nrofZero = minNrofChars - idxTestValues -1;
    while(--nrofZero >=0 && nChars < zBuffer) buffer[nChars++] = '0';
  }
  //test input digit
  { uint32 test;
    while(--idxTestValues >=0  && nChars < zBuffer)
    { cc = '0';
      test =testValues[idxTestValues];
      while(uvalue >= test)
      { cc += 1;
        uvalue -= test;
      }
      if(cc > '9')
      { cc += ('a' - '9' -1);
      }
      buffer[nChars++] = cc;
    }
  }
  cc = (char)('0' + uvalue); //last digit
  if(cc > '9')
  { cc += ('a' - '9' -1);
  }
  if(nChars < zBuffer)
  { buffer[nChars++] = cc;
  }
  STACKTRC_LEAVE; return nChars;
}



int toString_Float_FW(char* buffer, int zBuffer, float value, char cFormat, int width, int precision, ThCxt* _thCxt)
{ int nrofChars;
  if(strchr("eEfGg", cFormat)!=null)
  { char format[6] = "%9.9f";
    format[1] = '0' + width;
    format[3] = '0' + precision;
    format[4] = cFormat;
    nrofChars = sprintf(buffer, format, value);
    
  }
  else
  { nrofChars = -1;
  }  
  return nrofChars;
}

int toString_Double_FW(char* buffer, int zBuffer, double value, char cFormat, int width, int precision, ThCxt* _thCxt)
{ int nrofChars;
  if(strchr("eEfGg", cFormat)!=null)
  { char format[20] = {0};
    sprintf(format, "%%%d.%d%c", width, precision, cFormat);
    nrofChars = sprintf(buffer, format, value);
  }
  else
  { nrofChars = -1;
  }  
  return nrofChars;
}



typedef struct char8_t { char s[ 8]; } char8;
typedef struct char16_t{ char s[16]; } char16;
typedef struct char24_t{ char s[24]; } char24;
typedef struct char32_t{ char s[32]; } char32;
typedef struct char40_t{ char s[40]; } char40;
typedef struct char48_t{ char s[48]; } char48;
typedef struct char56_t{ char s[56]; } char56;
typedef struct char64_t{ char s[64]; } char64;

//NOTE: The following type is a non-left-value-array by GNU Compiler.
//typedef char char8[8];
;

int format_va_arg_Formatter_FW(ThCxt* _thCxt, const char* sText, int zText, char* buffer, int zBuffer, Va_listFW typedArgs)
{
  char const* sTypeArgs = typedArgs.typeArgs;
  va_list vargList = typedArgs.args;
  ParseResultPrintfStyle_fwFormatter parseResult[20]; 
  int nrofParseResult;
  int idxParseResult;
  int iBuffer = 0;
  STACKTRC_TENTRY("format_BV_StringJc");
  //va_start(vargList, args->v[0]);
  //parse the output and control string:
  nrofParseResult = parsePrinfStyleString_fwFormatter(parseResult, ARRAYLEN(parseResult), sText, zText);
  
  for(idxParseResult = 0; idxParseResult < nrofParseResult; idxParseResult++)
  { ParseResultPrintfStyle_fwFormatter* actParseResult = &parseResult[idxParseResult]; 
    int nrofCharsToCopy = 0;
    char const* srcToCopy = null;
    int32 arg32ToCopy = -1;
    float argFloat;
    double argDouble; 
    int radixIntConversion = 0;
    char typeConversion = '?';
    switch(actParseResult->value.specifier)
    { case 't':
      { //normal text
        nrofCharsToCopy = actParseResult->text.nrofChars;
        srcToCopy = actParseResult->text.text;
        typeConversion = 't';
      } break;
      case 's':
      { //text from argument
        if(sTypeArgs != null)
        { char cType = *(sTypeArgs++);
          switch(cType)
          { case 's': 
            { StringJc arg = va_arg(vargList, StringJc); 
              srcToCopy = getCharsAndLength_StringJc(&arg, &nrofCharsToCopy); 
            } break;
            case 'z':
            { srcToCopy = va_arg(vargList, char const*); 
              if(srcToCopy == null)
              { srcToCopy = "(null)";   //a null pointer may be acceptable.
              }
              nrofCharsToCopy = strlen(srcToCopy);
            } break;
            case '1':
            { //NOTE: There is a problem with GNU-Compiler. 
              //Solution: vargList pointers the current argument, take this pointer,
              //but increment with size of given type.
              srcToCopy = (char const*)vargList; 
              va_arg(vargList, char8); 
              nrofCharsToCopy = strlen_Fwc(srcToCopy, 8);
            } break;
            case '2':
            { srcToCopy = (char const*)vargList; 
              va_arg(vargList, char16); 
              nrofCharsToCopy = 16;
            } break;
            case '3':
            { srcToCopy = (char const*)vargList; 
              va_arg(vargList, char24); 
              nrofCharsToCopy = 24;
            } break;
            case '4':
            { srcToCopy = (char const*)vargList; 
              va_arg(vargList, char32); 
              nrofCharsToCopy = 32;
            } break;
            case '5':
            { srcToCopy = (char const*)vargList; 
              va_arg(vargList, char40); 
              nrofCharsToCopy = 40;
            } break;
            case '6':
            { srcToCopy = (char const*)vargList; 
              va_arg(vargList, char48); 
              nrofCharsToCopy = 48;
            } break;
            case '7':
            { srcToCopy = (char const*)vargList; 
              va_arg(vargList, char56); 
              nrofCharsToCopy = 56;
            } break;
            case '8':
            { srcToCopy = (char const*)vargList; 
              va_arg(vargList, char64); 
              nrofCharsToCopy = strlen_Fwc(srcToCopy, 64);
            } break;
            case '$':
            { //special designation: the vargList contains a directly 0-terminated string.
              //it is the last member of vargList, therefore this vargList isn't incremented.
              //The direct is the simple and opportune way to get the pointer.
              srcToCopy = (char const*)vargList; 
              nrofCharsToCopy = strlen_Fwc(srcToCopy, 100);
            } break;
            default:
            { srcToCopy="ERROR unknwon Type";
              nrofCharsToCopy = strlen(srcToCopy);
            }
          }
          typeConversion = 't';
        }
        else      
        { srcToCopy = va_arg(vargList, char const*); 
          if(srcToCopy == null)
          { srcToCopy = "(null)";   //a null pointer may be acceptable.
          }
          nrofCharsToCopy = strlen_Fwc(srcToCopy, 100);  //here an access error is possible if the argument list is wrong.
          typeConversion = 't';
        }
      } break;
      case 'p':  //pointer, here only int-like pointer (not segmented)
      case 'i':
      case 'd':
      case 'x':
      case 'X':
      case 'c':
      { //integer or character types.
        char cType;
        if(sTypeArgs != null)
        { cType = *(sTypeArgs++);
        }
        else      
        { cType = actParseResult->value.argument;
        }
        switch(cType)
        { case 'C': arg32ToCopy = va_arg(vargList, char_va_list); break; 
          case 'Z': arg32ToCopy = va_arg(vargList, bool_va_list); break; 
          case 'B': arg32ToCopy = va_arg(vargList, int8_va_list); break; 
          case 'S': arg32ToCopy = va_arg(vargList, int16_va_list); break; 
          case 'I': case 'K': case 'M': case 'N': arg32ToCopy = va_arg(vargList, int32); break; 
          case 'J': arg32ToCopy = (int32)va_arg(vargList, int64); break; 
          case 'F': arg32ToCopy = (int32)va_arg(vargList, float_va_list); break; 
          case 'D': arg32ToCopy = (int32)va_arg(vargList, double); break; 
          case 'P': arg32ToCopy = (int32)va_arg(vargList, void*); break; 
          default: arg32ToCopy= va_arg(vargList, int32); break;
        }
        typeConversion = actParseResult->value.specifier == 'c' ? 'c' :'I';
      } break;
      
      case 'e':
      case 'E':
      case 'f':
      case 'g':
      case 'G':
      { //integer
        char cType;
        if(sTypeArgs != null)
        { cType = *(sTypeArgs++);
        }
        else      
        { cType = actParseResult->value.argument;
        }
        switch(cType)
        { case 'D': 
          { argDouble = va_arg(vargList, double);
            typeConversion = 'D';     
          } break; 
          case 'F':
          { argFloat = (float)va_arg(vargList, float_va_list); 
            typeConversion = 'F';     
          } break; 
          default: 
          { argFloat = (float)va_arg(vargList, int32);  //get int, convert float. 
            typeConversion = 'F';     
          } break; 
        }
      } break;
      default:
      { char arg[] = "?%???";
        arg[2] = actParseResult->value.specifier;
        srcToCopy = "???";   //do not use the stack instance, it is a mistake!
        nrofCharsToCopy = 3;
      }
    }//switch
    switch(actParseResult->value.specifier)
    { case 'i': case 'd': radixIntConversion = 10; break;
      case 'p': radixIntConversion = 16; break;
      case 'x': radixIntConversion = 16; break;
      case 'X': radixIntConversion = 16; break;
      case 'o': radixIntConversion = 8; break;
      default: radixIntConversion = 0;
    }
    //convert a given number:
    switch(typeConversion)
    { case 't':
      { if( (zBuffer - iBuffer) < nrofCharsToCopy)
        { nrofCharsToCopy = zBuffer - iBuffer -1;  //shorten it, but copy so much as possible, the user will see it. 
        }
        memcpy(buffer + iBuffer, srcToCopy, nrofCharsToCopy);
        iBuffer += nrofCharsToCopy;
        srcToCopy = null;
      } break;
      case 'c':
      { //A char is requested in representation, take the arg32.
        char cc = (char)arg32ToCopy;
        if(cc < 0x20){ cc = '?'; }   //ctrl chars are not printable!
        if(iBuffer < (zBuffer -1))
        { buffer[iBuffer++] = cc;
        }
      } break;
      case 'I':
      { nrofCharsToCopy = toString_Integer_FW(buffer + iBuffer, zBuffer - iBuffer -1, arg32ToCopy, radixIntConversion, actParseResult->value.precision, _thCxt);
        iBuffer += nrofCharsToCopy;
        radixIntConversion = 0;
      } break;

      case 'D': 
      { nrofCharsToCopy = toString_Double_FW(buffer + iBuffer, zBuffer - iBuffer -1, argDouble, actParseResult->value.specifier, actParseResult->value.width, actParseResult->value.precision, _thCxt);
        if(nrofCharsToCopy >0)
        { iBuffer += nrofCharsToCopy;
        }
      } break; 
      case 'F':
      { nrofCharsToCopy = toString_Float_FW(buffer + iBuffer, zBuffer - iBuffer -1, argFloat, actParseResult->value.specifier, actParseResult->value.width, actParseResult->value.precision, _thCxt);
        if(nrofCharsToCopy >0)
        { iBuffer += nrofCharsToCopy;
        }
      } break; 
    }
  } 
  if(iBuffer < zBuffer)
  { buffer[iBuffer] = 0;  //should be always!
  }
  STACKTRC_LEAVE; return iBuffer;
}
