/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 *
 * @author JcHartmut www.vishia.org
 *
 ****************************************************************************/
#ifndef __MemC_h__
#define __MemC_h__
/**This file contains some usefull definitions for simpe C programming.

*/
#include <os_types_def.h>


/*@CLASS_C MemAreaC @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


/**returns a pointer to MemAreaC with OFFSET in memory.
 * @param BASE a pointer from type struct MemAreaC_t*. The type will be tested by compiling.
 * @param OFFSET a address-offset (sizeof-Type)
 * @return pointer from type struct MemAreaC_t* at memory location BASE + OFFSET
 */
#define addOffset_MemAreaC(BASE, OFFSET) ( (struct MemAreaC_t*) (((MemUnit*)(BASE)) + (OFFSET)) )

/**offset from BASE to BASE2.
 * @param BASE a pointer from type struct MemAreaC_t*. The type will be tested by compiling.
 * @param BASE2 a pointer from type struct MemAreaC_t*. The type will be tested by compiling.
 * @return number of memory address locations from BASE to BASE2
 */
#define offset_MemAreaC(BASE, BASE2) ((MemUnit*)(BASE2) - (MemUnit*)(BASE))




/*@CLASS_C MemC @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


#define MemC OS_ValuePtr

/**A const definition of a MemC using as initializer for static or stack variables.
 * In standard-C only useable for global static variables, if SIZE or PTR are not constants itself.
 */
#define CONST_MemC(PTR, SIZE) CONST_OS_ValuePtr(SIZE, PTR)


/**gets the size. 
 * It's an uint value because the operator sizeof() returns an uint also, and typically it is compared one another.
 */
#define size_MemC(THIS) (uint)(value_OS_ValuePtr(THIS))

//#define getPtr_MemC(MEMC, TYPE) PTR_OS_ValuePtr(MEMC, TYPE)

#define PTR_MemC(MEMC, TYPE) PTR_OS_ValuePtr(MEMC, TYPE)

//@deprecated, NOTE: better to use PTR_MemC
#define start_MemC(THIS) PTR_OS_ValuePtr(THIS, struct MemAreaC)

#define end_MemC(THIS) (addOffset_MemAreaC(value_OS_ValuePtr(THIS), size_MemC(THIS)))

#define set_MemC(THIS, INT, PTR) set_OS_ValuePtr(THIS, INT, PTR)


/**Sets the size and offset.
 * The offset should be either 0 (if the MemC contains not an BlockHeapBlockJc-location) or it should be a negativ number.
 * If the offset is 0, the size is only limited to 31 or 15 bits (15 bit for simple 16-bit-controller).
 * If the offset is negativ, the SIZE must not exceed the number of associated bits.
 * But it is not checked here, because this macro is not used in user space,
 * but typicall used only at implementation layer of the os system adapter.
 */
//#define setSizeAndOffset_MemC(THIS, SIZE, OFFSET) ((THIS).size_ = (SIZE) + ((OFFSET)<<16))


/**allocates memory in a stanard way. This method substitures the using of malloc() from standard-C-libraries.
 * It is possible to implement this method in a platform/framework specific way different to the given C-libraries. 
 */
METHOD_C MemC alloc_MemC(int size);

/**frees a allocated memory. It is the opposite method to alloc_MemC(). */
METHOD_C int free_MemC(MemC mem);

/**Initializes the given mem area with 0. This method substitutes the using of memset(ptr, 0, size) of standard-C. */
METHOD_C MemC init0_MemC(MemC mem);

/**Initializes the given mem area with 0. This method substitutes directly the  memset(ptr, 0, size) of standard-C.*/
METHOD_C void init0p_MemC(void* ptr, int size);

/**A macro to prevent writing errors. The pointered mem area occupied from the given type is set to 0, */
#define INIT0p_MemC(PTR) init0p_MemC(PTR, sizeof(*PTR));

/**Builds a MemC struct and returns it per value.
 * This is a helper method to supply a MemC argument in a method call.
 */
METHOD_C MemC build_MemC(void* address, int size);

/**Checks wether the size info in MemC is greater or equal given size, throws an IllegalArgumentException if there isn't so.
 */
METHOD_C void checkSize_MemC(MemC mem, int size, struct ThreadContextFW_t* _thCxt);


/**Builds a MemC struct and returns it per value.
 */
METHOD_C MemC fromArea_MemC(void* address, void* endAddress);

/**Build a subset from current.
 * @param offsetStart 0... adress offset from start
 * @param offsetEnd 0 = to end, negativ = backward from end, positiv = from start. It should be greaterequal offsetStart.
 */
METHOD_C MemC subset_MemC(MemC parent, int offsetStart, int offsetEnd);


/**Build a address inside mem. 
 * @param offset 0... adress offset from start
 * @param nrofBytes It is checked, whether the area contains enaugh bytes
 * @throws IndexOutOfBoundsException if the param are failed.
 */
METHOD_C struct MemAreaC_t* address_MemC(MemC mem, int offset, int nrofBytes);

/**Inserts a hole in the memory area at byte index idx. 
 * It means the content after insertAddress will be shifted to back.
 * The free moved content at insertAddress don't be changed. It means, this content is invalid in most of cases.
 * It is a copy of the content after insertAddr+ nrofBytes.
 *
 * Using this method no pointer arithmetic and memmove, memset is necessary. It prevents errors.
 * @param mem A area of memory, address and size.
 * @param insertAddr address inside the area of memory, on which the nrofBytes should inserted.
 * @param nrofBytes number of bytes to insert.
 */ 
METHOD_C void insert_MemC(MemC mem, void const* insertAddr, int nrofBytes);

/**Deletes some bytes in the memory area at byte index idx. It means the content after idx will be shifted to front.
 * The rest to end is filled with 0-bytes.
 */ 
METHOD_C void delete_MemC(MemC mem, int idx, int nrofBytes);

/**Allocates the amount of bytes in heap,but do not initialize it. */
#define MALLOC(TYPE) ((TYPE*)malloc(sizeof(TYPE)))


/**allocates and initializes with 0.
 */
MemC alloc0_MemC(int size);


/**Kompatibilitt: */
#define os_init0MemC init0_MemC




#endif  //__MemC_h__
