/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content
 *
 * @author Hartmut Schorrig, Pinzberg, Germany
 *
 * @version 0.83
 */
#ifndef __Jc_FileIoJc_h__
#define __Jc_FileIoJc_h__

#include "Jc/ObjectJc.h"
#include "OSAL/inc/os_file.h"

struct ThreadContextFW_t;

/*@CLASS_C FileJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


typedef struct FileJc_t
{ ObjectJc object;
  OS_FileDescription fileDescription;
} FileJc_s;


FileJc_s* ctorO_FileJc(ObjectJc* othis, StringJc name, ThCxt* _thCxt);

void init_FileJc(FileJc_s* ythis, StringJc name, ThCxt* _thCxt);

int32 lastModified_FileJc(FileJc_s* ythis);

bool exists_FileJc(FileJc_s* ythis);


/*@CLASS_C FileWriterJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/**This struct is the implementation of org.vishia.FileWriter. That class is a wrapper arround java.io.FileWriter.
 * The originally java.lang.FileWriter isn't useable well, because no static instance can be made.
 */
typedef struct FileWriterJc_t
{ union { ObjectJc object;} base;
  
  /**The C-standard FILE* handle. Don't use FILE* because prevent of include <stdio.h>. 
   * That header should only included in implementation.
   */
  void* file;
}FileWriterJc_s;

#define kFileNotFound_FileWriterJc -1

#define kFileOpenError_FileWriterJc -2


METHOD_C FileWriterJc_s* ctorO_FileWriterJc(ObjectJc* othis, struct ThreadContextFW_t* _thCxt);

METHOD_C int open_FileWriterJc(FileWriterJc_s* ythis, StringJc fileName, bool append, struct ThreadContextFW_t* _thCxt);

METHOD_C bool isOpen_FileWriterJc(FileWriterJc_s* ythis);

METHOD_C void write_FileWriterJc(FileWriterJc_s* ythis, StringJc text, struct ThreadContextFW_t* _thCxt);

METHOD_C void flush_FileWriterJc(FileWriterJc_s* ythis, struct ThreadContextFW_t* _thCxt);

METHOD_C void close_FileWriterJc(FileWriterJc_s* ythis, struct ThreadContextFW_t* _thCxt);



/*@CLASS_C FileReaderJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


typedef struct FileReaderJc_t
{ ObjectJc object;

  FILE* file_;
  int32 lenFile_;
  int32 bEof_;
  /**Incremented on any open/close action. */
  int modCount_;
  
}FileReaderJc_s;


FileReaderJc_s* ctorO_FileReaderJc(ObjectJc* othis, FILE* file, ThCxt* _thCxt);

int open_FileReaderJcF(FileReaderJc_s* ythis, StringJc sFileName, ThCxt* _thCxt);

bool isOpen_FileReaderJcF(FileReaderJc_s* ythis);

bool ready_FileReaderJcF(FileReaderJc_s* ythis);

int close_FileReaderJcF(FileReaderJc_s* ythis, ThCxt* _thCxt);





/*@CLASS_C BufferedReaderJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

typedef struct BufferedReaderJc_t
{ ObjectJc object;

  int32 lenBuffer;
  int32 lenLine;
  int32 bFinish;
  MemC buffer;

  /**The aggregated input reader. TODO use ReaderJc with C-interfacing. */
  FileReaderJc_s* input;

  /**To compare with input->modCount, */
  int32 modCountInput;

}BufferedReaderJc_s;


BufferedReaderJc_s* ctorO_BufferedReaderJc(ObjectJc* othis, FileReaderJc_s* input, MemC buffer, ThCxt* _thCxt);

StringJc readLine_BufferedReaderJc(BufferedReaderJc_s* ythis, ThCxt* _thCxt);


#endif //__Jc_FileIoJc_h__
