/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content
 *
 * @author Hartmut Schorrig, Pinzberg, Germany
 *
 * @version 0.83
 * @content Implementation of String processing
 *
 * @author Jchartmut www.vishia.org
 * @version 0.81
 * list of changes:
 * 2006-05-00: www.vishia.de creation
 *
 ****************************************************************************/

#include "Jc/StringJc.h"
#include "Fwc/fw_Exception.h"

//#include "fw_ThreadContext.h"

#include <string.h>

#ifdef mBackRef_ObjectJc
  /** The RuntimeHeapJc.h are only needed, if back references are to be stored.*/
//xxxTODO  #include "RuntimeHeapJc.h"
#else
  #define setBackref_BheapJc(REF, SRC_P) /*it is unused*/
#endif


/*@CLASS_C_IMPLEMENT StringJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


/*
const struct{ ObjectArrayJc head; FieldJc data[2]; } fields_StringBufferJc
=
{ CONST_ObjectArrayJc(FieldJc, 2, 0, null, &fields_StringBufferJc)
, { CONST_FieldJc(StringBufferJc, "count", count, int, 0)
  , CONST_FieldJc(StringBufferJc, "count", count, int, 0)
} };

const ReflectionStringBufferJc reflectionStringBufferJc
=
{ { CONST_ObjectJc(0, &reflectionStringBufferJc, null)
  , "StringBufferJc"
  , 0 //posObjBase
  , sizeof(StringBufferJc)
  , (FieldJcArray const*) &fields_StringBufferJc
  , (MethodJcARRAY const*) null
  ,  null  //superClasses
  ,  null  //interfaces
  ,0 //modifiers
  }
, { null
  }
};
*/

/** Some methods from StringJc are defined in ObjectJc.h, because there are simple methods
  * not using RuntimeHeap, Garbage Collection and others. This simple methods are implemented here.
  * That are all methods which do not copy strings, only compare, convert to char* etc.
  */



/**reads the nrofChars and the reference into local variable. Static subroutine for inner organisation:*/
char const* getCharsAndCount_StringJc(const StringJc* ythis, int* count)
{ char const* chars = _STRING_R2_StringJc(*ythis);
  int nChars = _LENGTH_R2_StringJc(*ythis);
  if(nChars == mLength__StringJc) { nChars = strlen(chars); }

  *count = nChars;
  return( chars);
}


void setFromBuffer_StringJc(StringJc* ythis, struct StringBufferJc_t* buffer)
{ STACKTRC_ENTRY("setFromBuffer_StringJc");
  clear_StringJc(ythis);
  _SETSTRING_R2_StringJc(*ythis, (buffer->size < 0 ? buffer->value.buffer : buffer->value.direct));
  setBackRefJc(&ythis->value__, buffer);
  _SETLENGTH_R2_StringJc(*ythis, buffer->_count & mLength__StringJc);
  _SETBUFFERED_R2_StringJc(*ythis);
  STACKTRC_LEAVE;
}







void set_StringJc(StringJc* ythis, StringJc src)
{ STACKTRC_ENTRY("set_StringJc");
  clear_StringJc(ythis);

  #ifdef mBackRef_ObjectJc
    //if( (src.s.refbase & mBackRef_ObjectJc) >0)
    { _SETSTRING_R2_StringJc(*ythis, _STRING_R2_StringJc(src));
      setBackRefJc(&ythis->value__, _STRING_R2_StringJc(src));
      _SETLENGTH_R2_StringJc(*ythis, _LENGTH_R2_StringJc(src));
      //ythis->s.refbase |= src.s.refbase & ~mBackRef_ObjectJc;  //copy nrof Chars and mBuffered__StringJc
    }
    //else{ *ythis = src; } //copy the content
  #else
    *ythis = src; //copy the content
  #endif
  STACKTRC_LEAVE;
}





void setRef_StringJc(StringJc* ythis, StringJc src)
{ int srcLength;
  char const* srcString;
  STACKTRC_ENTRY("setRef_StringJc");
  srcString = getCharsAndLength_StringJc(&src, &srcLength);
  if(src.value__ & mNonPersists__StringJc){
    /**if the String is not persistent, it should be copied! */
    StringBufferJc* buffer = new_StringBufferJc(-1, _thCxt); //allocate in heap
    replace_zi_StringBufferJc(buffer, 0,0, srcString, srcLength, _thCxt); //copy
    srcString = _getCharsAndCount_StringBufferJc(buffer, &srcLength); 
  }
  if(srcString == null)
  { /**handle null-pointer. */
    clearBackRefJc(&ythis->value__);  
    clear_StringJc(ythis);  
  }
  else 
  {
    set_OS_ValuePtr(*ythis, srcLength, srcString);  //yet without backref
    #ifdef mBackRef_ObjectJc
      setBackRefJc(&ythis->value__, srcString);  //clears also an existing backref.
    #endif
  }
}






void clear_StringJc(StringJc* ythis)
{ STACKTRC_ENTRY("clear_StringJc");

  #ifdef mBackRef_ObjectJc
    if( IDXBACKREFJc(*ythis) >0)
    { clearBackRefJc(&ythis->value__);
    }
    else{ ythis->value__ = 0; _SETSTRING_R2_StringJc(*ythis, null); } //copy the content
  #else
    init_s0i_StringJc(*ythis, null, 0);
  #endif
  STACKTRC_LEAVE;
}

#if 0
void clear_StringJc(StringJc* ythis)
{ STACKTRC_NAME("setref_String")
  ObjectRefValuesJc refbase = ythis->s.refbase;
  if(  (refbase & mIsConstant_StringJc) != mIsConstant_StringJc
    && ythis->s.ref.buffer != null
    )
  {
    clearBackRef_ObjectJc(&ythis->s.refbase);
  }
  ythis->s.refbase = 0;
  ythis->s.ref = null;
}
#endif




/** Gets the nr of chars and the pointer to the chars from a StringJcRef.
 * A StringJcRef may be a reference directly to a const char*, in this case
 * the offsetBackPointer contains a negative value, it is the count in negative image.<br/>
 * A StringJcRef may also be a reference directly to a StringBufferJc. This is the typical case
 * by concated strings. In this case
 */

#if 0
const char* getCharsAndCount_StringJc(const StringJcRef* ythis, int* count)
{ STACKTRC_ENTRY("toChar_String")
  ObjectRefValuesJc refbase = ythis->s.refbase;
  const char* chars;
  if((refbase & mZeroTerminated_StringJc) == mZeroTerminated_StringJc)
  { chars = ythis->s.ref.chars;
    *count = strlen(chars);
  }
  else if( (refbase & mIsConstant_StringJc) == mIsConstant_StringJc)
  { //it is a constant string
    *count = refbase & mLength__StringJc;
    chars = ythis->s.ref.chars;
  }
  else
  { //it is either a directly referenced StringBufferJc or a SubstringJc
    int offset;
    SubstringJc* substring = ythis->s.ref.substring;
    *count = substring->count;
    offset = - substring->negOffset;
    if(offset >= 0)
    { chars = substring->stringBuffer->(buffer->size < 0 ? buffer->value.buffer : buffer->value.direct) + offset;
    }
    else
    { //it is a referenced StringBufferJc itself:
      StringBufferJc* buffer = (StringBufferJc*)(substring);
      //TRICKY the lengthAdd is at same position.
      chars = buffer->(buffer->size < 0 ? buffer->value.buffer : buffer->value.direct);
    }
  }
  STACKTRC_LEAVE; return( chars);
}
#endif





bool equals_s0_StringJc(const StringJc ythis, const char* sCmp )
{ int countThis = _LENGTH_R2_StringJc(ythis);
  const char* strythis = _STRING_R2_StringJc(ythis);
  if(countThis == mLength__StringJc){ countThis = strlen(strythis); }
  if((int)(strlen(sCmp)) != countThis)
  { return(false);
  }
  else
  { bool bRet = memcmp(strythis, sCmp, countThis) == 0;
    return(bRet);
  }
}


bool startsWith_s0i_StringJc(const StringJc ythis, const char* sCmp, int zCmp)
{ int countThis = _LENGTH_R2_StringJc(ythis);
  const char* strythis = _STRING_R2_StringJc(ythis);
  if(countThis == mLength__StringJc){ countThis = strlen(strythis); }
  if(zCmp > countThis)
  { return(false);
  }
  else
  { bool bRet = memcmp(strythis, sCmp, zCmp) == 0;
    return(bRet);
  }
}

bool startsWith_s0_StringJc(const StringJc ythis, const char* sCmp)
{ return startsWith_s0i_StringJc(ythis, sCmp, strlen(sCmp)); 
}

bool startsWith_StringJc(const StringJc ythis, const StringJc cmp)
{ int zCmp;
  const char* sCmp = getCharsAndCount_StringJc(&cmp, &zCmp);
  return startsWith_s0i_StringJc(ythis, sCmp, zCmp); 
}

/** Realisation of substring: If the parent string is a constant string,
  * the substring will be also a simple constant string directly pointered a part of parent.
  * If the String is inside a StringBuffer, may be a SubstringJc of StringBufferJc or StringBufferJc itself,
  * the problem is more complexly. In this case inside the parent string it should be registered
  * the using of the Buffer. It may be a RuntimeHeapBlockJc with garbage collection.
  */
METHOD_C StringJc substring_StringJc(StringJc ythis, int beginIndex, int endIndex, ThCxt* _thCxt)
{ StringJc ret = {0};
  const char* chars = _STRING_R2_StringJc(ythis);
  int count = _LENGTH_R2_StringJc(ythis);
  STACKTRC_TENTRY("substring_StringJc");

  if(count == mLength__StringJc) { count = strlen(chars);}

  if(endIndex < 0) {endIndex = count; }

  if(endIndex > count){ THROW_s0(IndexOutOfBoundsException, "endIndex", endIndex); }
  else if(beginIndex <0 || beginIndex > endIndex){ THROW_s0(IndexOutOfBoundsException, "beginIndex", beginIndex); }
  else
  {
    _SETSTRING_R2_StringJc(ret, chars + beginIndex);
    _SETLENGTH_R2_StringJc(ret, endIndex - beginIndex);
  }

  STACKTRC_LEAVE; return( ret);
}


#if 0
METHOD_C StringJc substring_StringJc(StringJc* ythis, int beginIndex, int endIndex)
{ STACKTRC_ENTRY("substring_StringJc")
  StringJc ret;
  int count;
  if( (ythis->s.refbase & mIsConstant_StringJc) == mIsConstant_StringJc)
  { //it is a constant string
    const char* chars = ythis->s.ref;
    if((ythis->s.refbase & mZeroTerminated_StringJc) == mZeroTerminated_StringJc)
    {     count = strlen(chars);}
    else{ count = ythis->s.refbase & mLength__StringJc; }

    if(endIndex < 0) {endIndex = count; }

    if(endIndex > count){ THROWJc(IndexOutOfBoundsExceptionJc, "endIndex", endIndex); }
    else if(beginIndex <0 || beginIndex > endIndex){ THROWJc(IndexOutOfBoundsExceptionJc, "beginIndex", beginIndex); }
    else
    { ret.ref.chars = chars + beginIndex;
      ret.s.refbase = (endIndex - beginIndex) | mIsConstant_StringJc;
    }
  }
  else
  { THROWJc(RuntimeExceptionJc, "substring of not constant Strings not ready yet.",0);
  }
  STACKTRC_LEAVE; return( ret;
}
#endif



METHOD_C int indexOf_i_StringJc(StringJc ythis, int ch, int fromIndex)
{ const char* chars = _STRING_R2_StringJc(ythis);
  int nChars = _LENGTH_R2_StringJc(ythis);
  //STACKTRC_TENTRY("StringJc.indexOf(int,int)");
  if(nChars == mLength__StringJc) { nChars = strlen(chars); }
  //const char* chars = getCharsAndCount_StringJc(ythis, &nChars);
  if(fromIndex <0)
  { //STACKTRC_LEAVE; 
    return( -1);
  }
  else
  { int ii;
    for(ii = fromIndex; ii < nChars; ii++)
    { if(chars[ii] == ch) 
      { //STACKTRC_LEAVE; 
        return( ii); 
      }
    }
    //STACKTRC_LEAVE; 
    return( -1);  //not found STACKTRC_LEAVE; return(s in for loop if found!
  }
}



int indexOf_Si_StringJc(StringJc ythis, StringJc str, int fromIndex)
{ int nChars;
  //STACKTRC_TENTRY("StringJc._Si_StringJc");
  { 
    const char* chars = getCharsAndCount_StringJc(&ythis, &nChars);
    int nCharsSearch;
    const char* charsSearch = getCharsAndCount_StringJc(&str, &nCharsSearch);
    if(fromIndex <0)
    { //STACKTRC_LEAVE; 
      return( -1);
    }
    else
    { int ii;
      char cc = charsSearch[0];
      for(ii = fromIndex; ii < (nChars - nCharsSearch); ii++)
      { if(chars[ii] == cc)
        { //first char found, test the rest chars
          if(strncmp(charsSearch, chars+ii, nCharsSearch) == 0)
          { //STACKTRC_LEAVE; 
            return( ii);
          }
        }
      }
    }
    //STACKTRC_LEAVE; 
    return( -1);  //not found, note: return ii in for-loop if found.
  }
}


METHOD_C int indexOf_s0i_StringJc(StringJc ythis, const char* src, int fromIndex)
{ StringJc str1;
  init_s0_StringJc(str1, src);
  return indexOf_Si_StringJc(ythis, str1, fromIndex);

}


int lastIndexOf_i_StringJc(StringJc ythis, int ch, int fromIndex )
{ const char* chars = _STRING_R2_StringJc(ythis);
  int nChars = _LENGTH_R2_StringJc(ythis);
  //STACKTRC_TENTRY("StringJc.indexOf(int,int)");
  if(nChars == mLength__StringJc) { nChars = strlen(chars); }
  //const char* chars = getCharsAndCount_StringJc(ythis, &nChars);
  if(fromIndex <0)
  { //STACKTRC_LEAVE; 
    return( -1);
  }
  else
  { int ii;
    if(fromIndex >= nChars)
    { fromIndex = nChars -1; }
    for(ii = fromIndex; ii >=0; ii--)
    { if(chars[ii] == ch) 
      { //STACKTRC_LEAVE; 
        return( ii); 
      }
    }
    //STACKTRC_LEAVE; 
    return( -1);  //not found STACKTRC_LEAVE; return(s in for loop if found!
  }
}





void getChars_StringJc(StringJc ythis, int srcBegin, int srcEnd, char_Y* dst, int dstBegin, ThCxt* _thCxt)
{
  int sizeBuffer = dst->head.length;
  int nrofChars = srcEnd - srcBegin;
  int sizeString;
  char const* sString = getCharsAndLength_StringJc(&ythis, &sizeString);
  STACKTRC_TENTRY("getChars_StringJc");
  //if(sizeBuffer < size_StringJc(ythis,_thCxt)) THROW_s0(IndexOutOfBoundsException, "size to small", sizeBuffer);
  if(nrofChars > sizeBuffer) THROW_s0(IndexOutOfBoundsException, "buffer to small", sizeBuffer);
  if(nrofChars < 0) THROW_s0(IndexOutOfBoundsException, "srcEnd less than srcBegin", nrofChars);
  if(srcBegin < 0) THROW_s0(IndexOutOfBoundsException, "srcBegin <0", srcBegin);
  if(dstBegin < 0) THROW_s0(IndexOutOfBoundsException, "dstBegin <0", srcBegin);
  if((dstBegin + nrofChars) > sizeBuffer) THROW_s0(IndexOutOfBoundsException, "dstBegin fault", dstBegin);

  memcpy(dst->data + dstBegin, sString + srcBegin, nrofChars);  
  STACKTRC_LEAVE;
}











METHOD_C StringJc toString_StringBufferJc(StringBufferJc* src, ThCxt* _thCxt)
{
  bool bPersistents = !isOptimizeString_ThCxt(_thCxt);  //optimize:non persistents
  StringJc ret = NULL_StringJc;
  const char* s0 = src->size < 0 ? src->value.buffer : src->value.direct;
  int count = src->_count & mLength__StringJc;
  int nonPersistent = src->_mode & _mTemporary_StringBufferJc ? 0 : mNonPersists__StringJc;
  STACKTRC_TENTRY("toString_StringBufferJc");
  /**If the StringBuffer is a temporary, the String is persistant.
   * Elsewhere the String is not persistant. That is okay mostly. 
   * If the String will be stored persistent, it would be copied than. 
   */
  if(bPersistents && nonPersistent == mNonPersists__StringJc){
    /**It needs to save the String persistents: */
    StringBufferJc* buffer = new_StringBufferJc(-1, _thCxt); //allocate in heap
    setTemporary_StringBufferJc(buffer);  //only referenced by this String.
    replace_zi_StringBufferJc(buffer, 0,0, s0, src->_count & mLength__StringJc, _thCxt); //copy
    activateGarbageCollectorAccess_BlockHeapJc(&buffer->base.object);
    s0 = _getCharsAndCount_StringBufferJc(buffer, &count);
    nonPersistent = 0; 
  }
  set_OS_ValuePtr(ret, count | mBuffered__StringJc | nonPersistent, s0);

  //false, because it isn't a reference in data: setFromBuffer_StringJc(&ret, src);  //the String is contained in the Buffer.
  STACKTRC_LEAVE; return ret;
}


void ctorc_StringBufferJc(StringBufferJc* ythis, int size)
{ ctorc_ObjectJc(&ythis->base.object);
  ythis->_count = 0;
  if(size < 0)
  { size = -size;
  }
  ythis->size = size;
  memset(ythis->value.direct, 0, ythis->size);
}



/**************************************************************************************************/
/* some conversions numeric from/to String */

static int stringToInt__(const char* src, int size)
{ int val = 0;
  bool bNegativ;
  STACKTRC_ENTRY("stringToInt__");
  if(*src == '-') { src+=1; size -=1; bNegativ = true; }
  else { bNegativ = false; }
  while(size > 0 && *src >='0' && *src <='9')
  { val = 10* val + (*src - '0');
    src+=1;
    size -=1;
  }
  if(bNegativ){ val = -val; }
  STACKTRC_LEAVE; return( val);
}



int parseInt_IntegerJc(StringJc src)
{ return stringToInt__(_STRING_R2_StringJc(src), _LENGTH_R2_StringJc(src));
}



