/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content
 *
 * @author Hartmut Schorrig, Pinzberg, Germany
 *
 * @version 0.83
 * @content Definition of String processing Java-like
 *   Note: The included file Fwc/fw_String.h contains the definition of the type StringJc
 *   and basicly access macros (without implementation). If any routines from the Fwc-Level (Framework C)
 *   uses this type to reference a character field with given length, it needn't include 
 *   the full definitions given here, but only the basicly things.
 *
 *   The Type StringJc is the same as OS_ValuePtr
 *   from the OSAL level (Operation System Adaption Layer). This type is the designed and should be adapted
 *   to a compiler and its cpu to support return by value using only registers. 
 *   Therefore a return per value and typically also call per value for the type StringJc is
 *
 * @author Hartmut Schorrig
 * @version 0.83
 * list of changes:
 * 2004-05-00: Hartmut creation
 * 2008-12-00: Hartmut a lot of changes, not documented yet.
 * 2009-02-05: Hartmut new: capacity_StringBufferJc
 *
 ****************************************************************************/
#ifndef __StringJc_h__
#define __StringJc_h__

#include "Fwc/fw_String.h"    //based on the there defined type StringJc and the access macros.
#include "Fwc/fw_LogMessage.h" //only because Va_listFW, TODO
#include "Jc/ObjectJc.h"      //It is a concept of CRuntimeJavalike
#include "Jc/ReflectionJc.h"
#include <string.h>
#include <stdarg.h>
/*@CLASS_C StringJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

struct StringBufferJc_t;

/** Clears the reference to a string. If the string is a StringBuffer or Substring,
  * and RuntimeHeap with garbage collection is used, the back reference to ythis
  * will be released.
  */
METHOD_C void clear_StringJc(StringJc* ythis);


/** Set this as reference to a String.
  * The string src may be a constant string, a substring or a StringBuffer.
  * If Runtimeheap with garbage collection is used, and src is a StringBuffer,
  * the back referenced to ythis will be registered in buffer.
  * If ythis was a reference before, the reference will be
  * released. If the reference is already a reference to the same buffer,
  * nothing will be done.
  * @param src The source StringJc. Note: It is given per value, not per reference,
           so direct output from StringJc-returning methods are useable as argument.
           The value consist of 2 processor registers.
  */
METHOD_C void set_StringJc(StringJc* ythis, StringJc src);


typedef struct StringJcARRAY_t{ ObjectJc object; int32 length; int16 sizeElement; int16 mode; StringJc data[10]; } *StringJcARRAY;

/**Constructs a new String by decoding the specified subarray of bytes using the specified charset. The length of the new String is a function of the charset, and hence may not be equal to the length of the subarray. 
 * The behavior of this constructor when the given bytes are not valid in the given charset is unspecified. The CharsetDecoder class should be used when more control over the decoding process is required. 
 *
 * @param bytes - The bytes to be decoded into characters
 * @param offset - The index of the first byte to decode
 * @param length - The number of bytes to decode
 * @param charsetName - The name of a supported charset 
 * @throws UnsupportedEncodingException - If the named charset is not supported 
 * @throws IndexOutOfBoundsException - If the offset and length arguments index characters outside the bounds of the bytes array
 * @since JDK1.1 
 * @java java.lang.String(byte[] bytes, int offset, int length, String charsetName) 
 */
METHOD_C StringJc new_bYiiCharset_StringJc(int8_Y* volatile bytes, int offset, int length, StringJc charsetName, ThCxt* _thCxt);


METHOD_C StringJc new_cY_StringJc(char_Y* chars, ThCxt* _thCxt);


METHOD_C StringJc new_cYi_StringJc(char_Y* chars, int offset, int count, ThCxt* _thCxt);

  /** Assigns a StringBufferJc to the String, usefull to work without heap
   */

  METHOD_C void assignBuffer_StringJc(StringJc* ythis, struct StringBufferJc_t* buffer);







  /* Gets the nr of chars and the pointer to the chars from a StringJc.
   * A StringJc may be a reference directly to a const char*, in this case
   * the offsetBackPointer contains a negative value, it is the count in negative image.<br/>
   * A StringJc may also be a reference directly to a StringBufferJc. This is the typical case
   * by concated strings. In this case
   */
  METHOD_C const char* getCharsAndCount_StringJc(const StringJc* ythis, int* count);



  METHOD_C int size_StringJc(StringJc const ythis, ThCxt* _thCxt);



  /** Compares this string to the specified object. The result is true if
    * and only if the argument is not null and is a String object that represents
    * the same sequence of characters as this object.
    * Modified from Java, see String.equals(Object)
    */
  METHOD_C bool equals_s0_StringJc(const StringJc ythis, const char* sCmp);

  /** Compares this string to the specified object. The result is true if
    * and only if the argument is not null and is a String object that represents
    * the same sequence of characters as this object.
    * Modified from Java, see String.equals(Object)
    */
  METHOD_C bool equals_StringJc(const StringJc ythis, const StringJc sCmp);


  /** Tests if this string starts with the specified prefix.
    * Modified from Java, see String.startsWith(String)
    */
  METHOD_C bool startsWith_s0_StringJc(const StringJc ythis, const char* sCmp);


  /** Tests if this string starts with the specified prefix.
    * Modified from Java, see String.startsWith(String)
    */
  METHOD_C bool startsWith_s0i_StringJc(const StringJc ythis, const char* sCmp, int zCmp);


  /** Tests if this string starts with the specified prefix.
    * Original Java, see String.startsWith(String)
    */
  METHOD_C bool startsWith_StringJc(const StringJc ythis, const StringJc sCmp);







  /** Set this as reference to a Stringbuffer.
    * If Runtimeheap with garbage collection is used, the back referenced to ythis will be
    * registered in buffer. If ythis was a reference before, the reference will be
    * released. If the reference is already a reference to the same buffer,
    * nothing will be done.
    * @param ythis the String to be set.
    * @param buffer the buffer to be referenced.
    */
  METHOD_C void setFromBuffer_StringJc(StringJc* ythis, struct StringBufferJc_t* buffer);


  /**sets the reference, java: String str = src;
  */
  METHOD_C void setRef_StringJc(StringJc* ref, StringJc src);

  /**sets the reference, java: String str = src;
  */
  METHOD_C void setref_b_String(StringJc* ref, struct StringBufferJc_t* src);




  /** Returns a new string that is a substring of this string. The substring begins at the specified beginIndex and extends to the character at index endIndex - 1.
    * Thus the length of the substring is endIndex-beginIndex.
    * Same as in Java, see String.substring(int, int)
    * @param startIndex the beginning index, inclusive.
    * @param endIndex the ending index, exclusive. If -1, than the endIndex is the length of string, like call in Java without endIndex.
    * @throws IndexOutOfBoundsException - if the beginIndex is negative,
    * or endIndex is larger than the length of this String object, or beginIndex is larger than endIndex.
    */
  METHOD_C StringJc substring_StringJc(StringJc ythis, int beginIndex, int endIndex, ThCxt* _thCxt);
  
  /**Same, with only begin index. */
  #define substring_i_StringJc(STR, BEGIN, THC) substring_StringJc(STR, BEGIN, -1, THC)

  /** Returns the index within this string of the first occurrence
      of the specified character. If a character with value ch occurs in
      the character sequence represented by this String object,
      then the index of the first such occurrence is returned --
      that is, the smallest value k such that:

      , this.charAt(k) == ch

      is true. If no such character occurs in this string, then -1 is returned.

      Same as in Java.

      @param ch - a character.
      @return the index of the first occurrence of the character in the character sequence represented by this object, or -1 if the character does not occur.
    */
  #define indexOf_StringJc(YTHIS, CH)  indexOf_i_StringJc(YTHIS, CH, 0)

  /** Returns the index within this string of the first occurrence
      of the specified character, starting the search at the specified index.

      If a character with value ch occurs in the character sequence
      represented by this String object at an index no smaller than fromIndex,
      then the index of the first such occurrence is returned
      --that is, the smallest value k such that:

      , (this.charAt(k) == ch) && (k >= fromIndex)

      is true. If no such character occurs in this string at or after position fromIndex, then -1 is returned.

      There is no restriction on the value of fromIndex.
      If it is negative, it has the same effect as if it were zero:
      this entire string may be searched. If it is greater than the length of this string,
      it has the same effect as if it were equal to the length of this string: -1 is returned.
      Same as in Java.

      @param ch - a character.
      @param fromIndex the index to start the search from.
      @return the index of the first occurrence of the character in the character sequence represented by this object, or -1 if the character does not occur.
    */
  METHOD_C int indexOf_i_StringJc(StringJc ythis, int ch, int fromIndex);



  METHOD_C int indexOf_Si_StringJc(StringJc ythis, StringJc str, int fromIndex);

  #define indexOf_S_StringJc(YTHIS, STR)  indexOf_Si_StringJc(YTHIS, STR, 0)


  METHOD_C int indexOf_s0i_StringJc(StringJc ythis, const char* str , int fromIndex);


  #define indexOf_s0_StringJc(YTHIS, CH)  indexOf_s0i_StringJc(YTHIS, CH, 0)


  //#define indexOf_s_StringJc(YTHIS, CH)  indexOf_s_i_StringJc(YTHIS, CH, 0)




#define lastIndexOf_StringJc(YTHIS, CH)  indexOf_i_StringJc(YTHIS, CH, 0)

METHOD_C int lastIndexOf_i_StringJc(StringJc ythis, int ch, int fromIndex );



METHOD_C int lastIndexOf_Si_StringJc(StringJc ythis, StringJc str, int fromIndex);

#define lastIndexOf_S_StringJc(YTHIS, STR)  indexOf_Si_StringJc(YTHIS, STR, 0)


METHOD_C int lastIndexOf_s0i_StringJc(StringJc* ythis, const char* str , int fromIndex);


#define lastIndexOf_s0_StringJc(YTHIS, CH)  indexOf_s0i_StringJc(YTHIS, CH, 0)





METHOD_C void add_s0_StringJc(StringJc* ythis, const char*);


/**Concatenated the given String with a new String. 
 * The location of the String referenced with ythis will be changed, if it isn't a StringBufferJc yet.
 * @param add the String to add.
 */
METHOD_C void add_s_StringJc(StringJc* ythis, StringJc add);

METHOD_C void add_i_StringJc(StringJc* ythis, int);

METHOD_C const char* toChar_StringJc(const StringJc* ythis);

METHOD_C StringJc format_BV_StringJc(StringJc text, struct StringBufferJc_t* sbuffer, Va_listFW args); 
//METHOD_C StringJc format_BV_StringJc(StringJc text, struct StringBufferJc_t* sbuffer, struct VaArgBuffer_t const* args); 
    

METHOD_C StringJc toString_DoubleJc(double value, ThCxt* _thCxt);


METHOD_C StringJc replace_StringJc(StringJc ythis, char oldChar, char newChar, ThCxt* _thCxt);

/**copies the content of the src into a char*-buffer, javalike java.util.String.getChar(...).
 * Copies characters from this string into the destination character array. 
 * The first character to be copied is at index srcBegin; 
 * the last character to be copied is at index srcEnd-1 
 * (thus the total number of characters to be copied is srcEnd-srcBegin). 
 * The characters are copied into the subarray of dst starting at index dstBegin and ending at index:
 * ,  dstbegin + (srcEnd-srcBegin) - 1
 *
 * @param srcBegin index of the first character in the string to copy.
 * @param srcEnd index after the last character in the string to copy.
 * @param dst the destination array.
 * @param dstBegin the start offset in the destination array. 
 * @throws IndexOutOfBoundsException - If any of the following is true: 
 * * srcBegin is negative. 
 * * srcBegin is greater than srcEnd 
 * * srcEnd is greater than the length of this string 
 * * dstBegin is negative 
 * * dstBegin+(srcEnd-srcBegin) is larger than dst.length
 */
METHOD_C void getChars_StringJc(StringJc ythis, int srcBegin, int srcEnd, char_Y* dst, int dstBegin, ThCxt* _thCxt);



/**This method tests whether a StringJc is contained in a StringBuffer-instance already. 
 * If it is so, it will be tested whether the instance is referenced from another source. 
 * If the instance has not a reference, the instance of StringBufferJc will used.
 * If the String isn't contained in a StringBufferJc, or the instance is used from any other side,
 * a new StringBuffer is created and the content of src is filled there.
 *
 * This method is typically used for concatenations of Strings. The concatenation is done using a StringBuffer.
 * A unnecessarry copy of data should be prevent. Mostly the situation is so, that the String
 * is contained in a StringBuffer, from a concatenation statement before, and it isn't referenced from another source. 
 */
METHOD_C struct StringBufferJc_t* transsubstantiateToStringBuffer_StringJc(StringJc ythis, ThCxt* _thCxt);


//void String_Integer(int value, int radix, StringBufferJc* dst);

/*@CLASS_CPP StringJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)

/**The C++-Variante. It is comfortable to use C++ because some methods and operators are argument-sensitive (overloaded),
 * and the code is better readable.
 * This class defines only methods, and the methods are not virutal. It followes the java convention: The class java.lang.String is final.
 * Thats why a applicable side action exists: The memory layout of StringJcpp is identical with the memory layout of StringJc.
 * So conversions may be able without any problems. Using static_cast<StringJc*>
 */
class  StringJcpp: public StringJc
{
  public: StringJcpp(){ set_OS_ValuePtr(*(static_cast<StringJc*>(this)),0, null); }

  public: StringJcpp(const char* src)
  { init_s0i_StringJc(*this, src, strlen(src));
  }

  public: StringJcpp(const char* src, int size)
  { ctor_s0i_StringJc(*this, src, size);
  }

  public: StringJcpp(const StringJc src)
  {
    init_s_StringJc(*this, src);
  }

  public: StringJcpp&  operator=(class StringBufferJcpp& src);

  public: StringJcpp(class StringBufferJcpp& src);

  public: ~StringJcpp(){ } //  CLEARREFJc(*static_cast<StringJc*>(this)); }

  public: operator StringJc(){ return *static_cast<StringJc*>(this); }

  public: char charAt(int idx){ return charAt_StringJc(*static_cast<StringJc*>(this), idx); }


  public: StringJcpp& operator=(StringJcpp src)
  {
    set_StringJc(this, src); return *this;
  }

  public: StringJcpp& operator=(StringJc src)
  {
    set_StringJc(this, src); return *this;
  }

  public: StringJcpp& operator=(const char* src)
  {
    clear_StringJc(this); init_s0_StringJc(*this, src); return *this;
  }

  public: StringJcpp& operator=(int src)  //src may be only 0
  {
    clear_StringJc(this); return *this;
  }

  public: StringJcpp& operator+(const char* add)
  { add_s0_StringJc(this, add); return *this;
  }

  public: StringJcpp& operator+(const int add)
  { add_i_StringJc(this, add); return *this;
  }

  public: StringJcpp& operator+=(const char* add)
  { add_s0_StringJc(this, add); return *this;
  }

  public: StringJcpp& operator+=(const StringJcpp& add)
  { add_s_StringJc(this, add); return *this;
  }

  public: StringJcpp& operator+=(int add)
  { add_i_StringJc(this, add); return *this;
  }


  public: bool equals(const char* sCmp)
  { return equals_s0_StringJc(*static_cast<StringJc*>(this), sCmp);
  }

  public: bool equals(const StringJc* sCmp)
  { return equals_StringJc(*static_cast<StringJc*>(this), *sCmp);
  }

  public: int length()
  { return length_StringJc(*static_cast<StringJc*>(this));
  }

  public: bool startsWith(const char* sCmp)
  { return startsWith_s0_StringJc(*static_cast<StringJc*>(this), sCmp);
  }

  public: bool startsWith(const StringJc* sCmp)
  { return startsWith_StringJc(*static_cast<StringJc*>(this), *sCmp);
  }

  public: int indexOf(char cc)
  { return indexOf_StringJc(*static_cast<StringJc*>(this), cc);
  }

  public: int indexOf(char cc, int fromIndex)
  { return indexOf_i_StringJc(*static_cast<StringJc*>(this), cc, fromIndex);
  }

  public: int indexOf(const char* str)
  { return indexOf_s0i_StringJc(*static_cast<StringJc*>(this), str, strlen(str));
  }


  public: StringJc substring(int beginIndex, int endIndex)
  { StringJc ret = substring_StringJc(*static_cast<StringJc*>(this), beginIndex, endIndex, null);
    return ret;
  }

  public: StringJc substring(int beginIndex)
  { StringJc ret = substring_StringJc(*static_cast<StringJc*>(this), beginIndex, -1, null);
    return ret;
  }
};

#endif //__CPLUSPLUSJcpp




/*@CLASS_C StringBufferJc @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/


//#define staticSize_StringBufferJc 80

/**A StringBufferJc is a structure to manage char[]-arrays. 
 * It is derivated from the contains the java.lang.StringBuffer rsp. java.lang.StringBuilder.
 * Some Java-like methods are supplied, but not all, but some methods are additional 
 * and specifical for usages in C or C++.
 *
 * There are 2 forms of memory layout of this struct:
 * * a) The struct StringBufferJc contains only the count, size and the pointer of the char[...]-buffer.
 * * b) The struct contains the char-buffer itself.
 *
 * The last variant is possible if enaugh raw memory is allocated outside 
 * and the [[method_ctorM_StringBufferJc(MemC)]] is called. 
 * That variant needs only 1 memory block for the buffer management and the buffer.
 *
 */
typedef struct  StringBufferJc_t
{
  /** It is an Object.*/
  union{ ObjectJc object;} base;

  /** actual nr of characters. 0 or positive. */
  int16 _count;

  /**The size of the Buffer and marker to use value. 
   * If this number is negative, a buffer is referenced, use value.buffer. 
   * If this number is positive, the buffer follows immediately, use value.direct. 
   */
  int16 size;

  int32 _mode;
  /**The count word is designated with this bits, if the instance is only created temporary while concatenating Strings. 
   */
  #define _mTemporary_StringBufferJc mNonPersists__StringJc 

  union value_t
  {
    /** The buffer may be disposed directly in this structure. 
    */
    char direct[4];
    #define offsetChars_StringBufferJc (sizeof(ObjectJc) + 2* sizeof(int16)) + sizeof(int32)  //sizeof(count) + sizeof(maxNumberOfChars)

    /**Pointer to the referenced buffer.
     * If this is choosen, the size is negative.
    */
    char* buffer;
  }value;
}StringBufferJc;


//#define staticSize_StringBufferJc (sizeof(((StringBufferJc*)(0x1000))->value.direct))



/**Initializes the Stringbuffer with a directly buffer after the StringBuffer management values.
 * The size of the memory determines the size of the buffer. The management needs 0x14 bytes. 
 * The rest is for the buffer itself.
 * @param rawMem memory, uninitialized.
 * @return the initialized instance at that memory placement.
 * @throws RuntimeException if the memory is to less.
 * @deprecated
 */
METHOD_C StringBufferJc* ctorM_StringBufferJc(MemC rawMem);
#define ctor_StringBufferJc ctorM_StringBufferJc


/**Initializes the Stringbuffer with given fix size.
 * If the mthis is sufficient for the buffer, the mthis area will be used for that.
 * Otherwise the buffer is created.
 * @param rawMem memory, uninitialized.
 * @return the initialized instance at that memory placement.
 * @throws RuntimeException if the memory is to less.
 * @deprecated
 */
METHOD_C StringBufferJc* ctorM_i_StringBufferJc(MemC mthis, int size, ThCxt* _thCxt);


/**Initializes the Stringbuffer with a immediate buffer after the StringBuffer management values.
 * The size stored inside ObjectJc determines the size of the buffer. 
 * @param othis The head data, initialized.
 * @return the initialized instance at that memory placement.
 * @throws RuntimeException if the memory is to less.
 */
METHOD_C StringBufferJc* ctorO_StringBufferJc(ObjectJc* othis, ThCxt* _thCxt);


/**Initializes the Stringbuffer to reference the given buffer with content with the given size.
 * This method should be used if an external char[] is to be used.
 * The nrof chars in the StringBuffer is determined by the content of the buffer
 * with the 0-terminated C-like method.
 * The char[] can also be a part of a char[].
 * @param buffer A char[...]-buffer with content. write 0 at start if it should be empty.
 * @param size the nrof bytes for buffer.
 */
METHOD_C StringBufferJc* ctorO_ci_StringBufferJc(ObjectJc* newObj, char* buffer, int size, ThCxt* _thCxt);

/**@deprecated */
METHOD_C StringBufferJc* ctorM_s0i_StringBufferJc(MemC mthis, char* buffer, int size);
/**@deprecated */
METHOD_C StringBufferJc* ctorc_s0i_StringBufferJc(StringBufferJc* ythis, char* buffer, int size);

/**Initializes a StringBuffer with no characters in it and an initial capacity specified by the capacity argument.
 * @param othis The initialized ObjectJc of the instance. 
 *              If the Object conaints enaugh space for size (field ,,ObjectJc.objectIdentSize,,),
 *              than the buffer will be assigned immediately after the StringBufferJc-instance.
 * @param size the nrof bytes for buffer.
 * @deprecated, use ctorO_StringBufferJc with enaugh size in othis.
 */
METHOD_C StringBufferJc* ctorO_i_StringBufferJc(ObjectJc* newObj, int size, ThCxt* _thCxt);

METHOD_C StringBufferJc* ctorO_S_StringBufferJc(ObjectJc* newObj, StringJc src, ThCxt* _thCxt);

/**Initializes a StringBuffer with no characters in it and an initial capacity specified by the capacity argument.
 * @param size the nrof bytes for buffer.
 * @deprecated
 */
METHOD_C StringBufferJc* ctorc_i_StringBufferJc(StringBufferJc* ythis, int size);


//in advance
#define ctor_StringBuilder(MEM, SIZE, THC) ctorM_i_StringBufferJc(MEM, SIZE, THC)


/** java:StringBuffer() */
METHOD_C StringBufferJc* new_StringBufferJc(int size, ThCxt* _thCxt);

/**This macro defines a C-constant (with {..}) for initializing a StringBuffer instance with its static size.
 * @param OBJP Pointer to the instance itself. It is used to store the OWNADDRESS for the ObjectJc-part.
 */
#define CONST_StringBufferJc(OBJP) { CONST_ObjectJc(sizeof(StringBufferJc), &(OBJP)->object, null), 0, staticSize_StringBufferJc, {0,0} }




METHOD_C StringBufferJc* returnThreadBuffer_StringBufferJc();

METHOD_C StringBufferJc* tempThreadBuffer_StringBufferJc();



#define setTemporary_StringBufferJc(YTHIS) ((YTHIS)->_mode |= _mTemporary_StringBufferJc)

/**Gets the buffer and the size of the buffer. For internal use especially. */
METHOD_C char* _getCharsAndSize_StringBufferJc(StringBufferJc* ythis, int* size);


/**Gets the buffer and the actual nr of chars in the buffer. For internal use especially. */
METHOD_C char* _getCharsAndCount_StringBufferJc(StringBufferJc* ythis, int* count);

/**Gets the buffer, the size and the actual nr of chars in the buffer. For internal use especially. */
METHOD_C char* _getCharsSizeCount_StringBufferJc(StringBufferJc* ythis, int* size, int* count);

/**Sets the nr of chars immediately. Only internal use. */
METHOD_C void _setCount_StringBufferJc(StringBufferJc* ythis, int count);


/**Sets the length of the character sequence. It's like java.lang.StringBuffer.setLength().
 * The sequence is changed to a new character sequence whose length is specified by the argument.
 * For every nonnegative index k less than newLength, the character at index k in the new character sequence
 * is the same as the character at index k in the old sequence if k is less than the length of the old character sequence;
 * otherwise, it is the null character '\u0000'.
 * In other words, if the newLength argument is less than the current length, the length is changed to the specified length.
 *
 * If the newLength argument is greater than or equal to the current length, sufficient null characters ('\u0000')
 * are appended so that length becomes the newLength argument.
 *
 * The newLength argument must be greater than or equal to 0.
 *
 * @param newLength The new length
 * @throws IndexOutOfBoundsException if the newLength argument is negative.
 * @see length()
 */
METHOD_C void setLength_StringBufferJc(StringBufferJc* ythis, int newLenhth, ThCxt* _thCxt);


/**This macro defines a C-constant (with {..}) for initializing a StringBuffer instance with its static size.
 * @param OBJP Pointer to the instance itself. It is used to store the OWNADDRESS for the ObjectJc-part.
 */
#define CONST_StringBufferJc(OBJP) { CONST_ObjectJc(sizeof(StringBufferJc), &(OBJP)->object, null), 0, staticSize_StringBufferJc, {0,0} }


/**This macro definesx a C-constant (with {..}) for initializing a StringBuffer instance with an additional size.
 * This macro is usefull if a StringBuffer with a followed char array is defined to increase the size of string in buffer.
 * The definition of such an construct is possible inside a struct definition with followed sample:
 *
 * ,,struct { ... ; StringBufferJc buffer; char bufferIncreasingCharArray[120]; ... } myData =
 *
 * The ,,bufferIncreasingCharArray,, have to be assembled immediately after the ,,buffer,,. The initializer of this part may be written in form of:
 *
 * ,,{ ... , CONST_addSize_StringBufferJc(&myData.buffer, 120), { '\0'}, ... },,
 *
 * The using of a following char array to increase the buffer is opportune to get a larger StringBuffer, because the compiler always assigns the memory
 * of the followed char array immediately after the previous StringBufferJc, and the own char array of the StringBufferJc is assigned as the last
 * element there. The problem of additional bytes because 4-Byte-allignment or such is not present, because a StringBufferJc have a size of multiple 4 bytes.
 * But the user should define the size of the char array as multiple of 4 to prevent alignment problems also.
 *
 * Note: Outside a struct the definition a definition of elements one after another don't may assigned one after another in memory.
 * It differents from compiler versions or other conditions such as segmentation sizes of physical memory!
 *
 * @param OBJP Pointer to the instance itself. It is used to store the OWNADDRESS for the ObjectJc-part.
 * @param ADDSIZE The additional size of the StringBuffer, defined by siezof the followed char array.
 */
#define CONST_addSize_StringBufferJc(OBJP, ADDSIZE) { CONST_ObjectJc(sizeof(StringBufferJc) + (ADDSIZE), &(OBJP)->object, null), 0, staticSize_StringBufferJc + ADDSIZE, {0,0} }
//#define DEF_StringBufferJc(NAME, SIZE) struct{ StringBufferJc buffer; char add[SIZE - staticSize_StringBufferJc]; } NAME = { { CONST_ObjectJc(0, &sBuffer.buffer.object, null), 0, SIZE, {0} } , {0}}


#define length_StringBufferJc(YTHIS) ((YTHIS)->_count & mLength__StringJc)
//METHOD_C int length_StringBufferJc(const StringBufferJc* ythis, ThCxt* _thCxt);

METHOD_C int capacity_StringBufferJc(StringBufferJc* ythis);


METHOD_C int indexOf_i_StringBufferJc(const StringBufferJc* ythis, const StringJc str, int fromIndex, ThCxt* _thCxt);

#define indexOf_StringBufferJc(YTHIS, STR, THC) indexOf_i_StringBufferJc(YTHIS, STR, 0, THC)


METHOD_C char charAt_StringBufferJc(StringBufferJc* ythis, int index, ThCxt* _thCxt);

/** java:StringBuffer.setCharAt(int index, char ch) */
METHOD_C void setCharAt_StringBufferJc(StringBufferJc* ythis, int index, char ch, ThCxt* _thCxt);

METHOD_C StringBufferJc* deleteCharAt_StringBufferJc(StringBufferJc* ythis, int index, ThCxt* _thCxt);
#define deleteCharAt_StringBufferJc(YTHIS, IDX, THC) delete_StringBufferJc(YTHIS,IDX, (IDX)+1, THC)

/** java:StringBuffer.insert(String) */
METHOD_C StringBufferJc* insert_s_StringBufferJc(StringBufferJc* ythis, int offset, StringJc add, ThCxt* _thCxt);

/** java:StringBuffer.insert(int offset, String, int start, int end) */
METHOD_C StringBufferJc* insert_sII_StringBufferJc(StringBufferJc* ythis, int offset, StringJc add, int start, int end, ThCxt* _thCxt);

/** java:StringBuffer.insert(int offset, char c) */
METHOD_C StringBufferJc* insert_C_StringBufferJc(StringBufferJc* ythis, int offset, char add, ThCxt* _thCxt);

#define insert_s0_StringBufferJc(YTHIS, OFFSET, ADD, LEN, _THC) replace_zi_StringBufferJc(YTHIS, OFFSET, OFFSET, ADD, LEN, _THC)

#define insert_s_StringBufferJc(YTHIS, OFFSET, ADD, _THC) replace_StringBufferJc(YTHIS, OFFSET, OFFSET, ADD, _THC)

/**The core method to insert, append, replace, delete. All those methods are implemented with this method.
 * @param
 */
METHOD_C StringBufferJc* replace_zi_StringBufferJc(StringBufferJc* ythis, int start, int end, const char* add, int len, ThCxt* _thCxt);

/** java:StringBuffer.replace(int, int, String)
 *  @param len -1 than calculate strlen(add)
 */
METHOD_C StringBufferJc* replace_s_StringBufferJc(StringBufferJc* ythis, int start, int end, StringJc add, ThCxt* _thCxt);


/**Java: insert(int offset, int i)
 * Inserts the string representation of the second int argument into this sequence.
 * The second argument is converted to a string as if by the method String.valueOf,
 * and the characters of that string are then inserted into this sequence at the indicated offset.
 *
 * The offset argument must be greater than or equal to 0, and less than or equal to the length of this sequence.
 * @param offset - the offset.
 * @param i - an int.
 * @return a reference to this object.
 * throws StringIndexOutOfBoundsException - if the offset is invalid.
 * @see String.valueOf(int), insert(int, java.lang.String), length()
 */
#define insert_I_StringBufferJc(YTHIS, OFFS, VALUE, THCXT) insert_Ir_StringBufferJc(YTHIS, OFFS, VALUE, 10, THCXT)
METHOD_C StringBufferJc* insert_Ir_StringBufferJc(StringBufferJc* ythis, int offset, int32 value, int radix, ThCxt* _thCxt);
#define insert_i_StringBufferJc(YTHIS, OFFS, VALUE, RADIX, THCXT) insert_Ir_StringBufferJc(YTHIS, OFFS, VALUE, RADIX, THCXT)

/** java:StringBuffer.insert(long) */
METHOD_C StringBufferJc* insert_Jr_StringBufferJc(StringBufferJc* ythis, int offset, int64 value, int radix, ThCxt* _thCxt);

/** java:StringBuffer.insert(float) */
METHOD_C StringBufferJc* insert_F_StringBufferJc(StringBufferJc* ythis, int offset, float value, ThCxt* _thCxt);

/** java:StringBuffer.insert(double) */
METHOD_C StringBufferJc* insert_D_StringBufferJc(StringBufferJc* ythis, int offset, double value, ThCxt* _thCxt);


/** java:StringBuffer.replace(int,int,String) */
METHOD_C StringBufferJc* replace_StringBufferJc(StringBufferJc* ythis, int start, int end, StringJc value, ThCxt* _thCxt);


/**Append a zero-terminated char. */
#define append_z_StringBufferJc(YTHIS, SRC, THCXT) replace_zi_StringBufferJc(YTHIS, (YTHIS)->_count & mLength__StringJc, (YTHIS)->_count & mLength__StringJc, SRC, -1, THCXT)

/** java:StringBuffer.append(String) */
METHOD_C StringBufferJc* append_s_StringBufferJc(StringBufferJc* ythis, StringJc add, ThCxt* _thCxt);
//METHOD_C StringBufferJc* append_s_StringBufferJc(StringBufferJc* ythis, StringJc add);

/** java.lang.StringBuffer.append((CharSequence s, int start, int end) 
 * appends a character string with given length. */
METHOD_C StringBufferJc* append_sII_StringBufferJc(StringBufferJc* ythis, StringJc src, int start, int end, ThCxt* _thCxt);


/** java:StringBuffer.append(StringBuffer) */
METHOD_C StringBufferJc* append_u_StringBufferJc(StringBufferJc* ythis, StringBufferJc* add, ThCxt* _thCxt);

/** C-like: appends a character string with given length. */
METHOD_C StringBufferJc* append_zI_StringBufferJc(StringBufferJc* ythis, const char* add, int lengthAdd, ThCxt* _thCxt);


/** java:StringBuffer.append(String) */
//METHOD_C StringBufferJc* append_s0_StringBufferJc(StringBufferJc* ythis, const char* add, ThCxt* _thCxt);
//#define append_s0_StringBufferJc(YTHIS, ADD, THREADC) append_s_StringBufferJc((YTHIS), s0_StringJc(ADD), THREADC);

/** java:StringBuffer.append(char) */
//METHOD_C StringBufferJc* append_c_StringBufferJc(StringBufferJc* ythis, char add, ThCxt* _thCxt);

#define append_C_StringBufferJc(YTHIS, VALUE, THCXT) insert_C_StringBufferJc(YTHIS, (YTHIS)->_count & mLength__StringJc, VALUE, THCXT)


/** java:StringBuffer.append(int) */
#define append_I_StringBufferJc(YTHIS, VALUE, THCXT) insert_Ir_StringBufferJc(YTHIS, (YTHIS)->_count & mLength__StringJc, VALUE, 10, THCXT)
//METHOD_C StringBufferJc* append_i_StringBufferJc(StringBufferJc* ythis, int add, ThCxt* _thCxt);
//METHOD_C StringBufferJc* append_i_StringBufferJc(StringBufferJc* ythis, int add);

/** java:StringBuffer.append(float) */
#define append_F_StringBufferJc(YTHIS, value, THCXT) insert_F_StringBufferJc(YTHIS,(YTHIS)->_count & mLength__StringJc, value, THCXT)
#define append_D_StringBufferJc(YTHIS, value, THCXT) insert_D_StringBufferJc(YTHIS,(YTHIS)->_count & mLength__StringJc, value, THCXT)

#define append_J_StringBufferJc(YTHIS, VALUE, THCXT) insert_Jr_StringBufferJc(YTHIS, (YTHIS)->_count & mLength__StringJc, VALUE, 10, THCXT)

/**Removes the characters in a substring of this sequence. It is original from Java.
 * The substring begins at the specified start and extends to the character
 * at index end - 1 or to the end of the sequence if no such character exists.
 * If start is equal to end, no changes are made.
 *
 * @params start The beginning index, inclusive.
 * @params end The ending index, exclusive.
 * @return This object.
 * @throws StringIndexOutOfBoundsException if start is negative, greater than length(),
 *           or greater than end.
 *
 */
//METHOD_C StringBufferJc* delete_StringBufferJcD(StringBufferJcD* ythis, int start, int end);
METHOD_C StringBufferJc* delete_StringBufferJc(StringBufferJc* ythis, int start, int end, ThCxt* _thCxt);


/** Copies the string-reference into a reference of a StringBuffer.
  * Either, the given reference references a string in a StringBuffer, only the content
  * of the reference will be copied. Or the string is a direct String or substring.
  * Than a new StringBuffer-instance will be created.
*/
METHOD_C StringBufferJc* toStringBuffer_String(StringJc const* src, ThCxt* _thCxt);

/**Returns the String which represents the content of the StringBuffer.
 *
 * Implemenation Note: The StringBuffer will be designated as freezed. 
 * If it is a temporary StringBuffer, this won't be have an effect to the user, 
 * because the StringBufferJc-instance isn't referenced from the user and therefore not used.
 * But if it is an user-defined StringBufferJc, the content should be copied ...TODO. 
 */ 
METHOD_C StringJc toString_StringBufferJc(StringBufferJc* src, ThCxt* _thCxt);

/**cleans the Rest of content in the buffer after the valid chars.
 * This method is enhanced in comparision to Java. It consideres a usage of StringBufferJc for string fields in structures.
 * The content in the structur's fields should be cleanly. The naming of the method is contrastable to the Java method trimToSize().
 */
METHOD_C void cleanToSize_StringBufferJc(StringBufferJc* ythis);




#if defined(__CPLUSPLUSJcpp) && defined(__cplusplus)

/**The C++-Variante. It is comfortable to use C++ because some methods and operators are argument-sensitive (overloaded),
 * and the code is better readable.
 * This class defines only methods, and the methods are not virutal. It followes the java convention: The class java.lang.String is final.
 * Thats why a applicable side action exists: The memory layout of StringJcpp is identical with the memory layout of StringJc.
 * So conversions may be able without any problems. Using static_cast<StringJc*>
 */
class  StringBufferJcpp: public ObjectifcBaseJcpp, public StringBufferJc
{

  public: static StringBufferJcpp& returnThreadBuffer();
  
  public: static StringBufferJcpp& tempThreadBuffer();
  
   

  public: StringBufferJcpp(char* buffer, int len){ ctorc_s0i_StringBufferJc(this, buffer, len); }

  public: StringBufferJcpp(ObjectJc* othis){ ctorO_StringBufferJc(othis, null); }

  public: void setCharAt(int index, char ch){ setCharAt_StringBufferJc(this, index, ch, null); }

  public: StringBufferJcpp* append(StringJc str){ append_s_StringBufferJc(this, str, null); return this; }

  public: StringBufferJcpp& operator=(StringJc str){ setLength_StringBufferJc(this, 0, null); append_s_StringBufferJc(this, str, null); return *this; }

  public: volatile StringBufferJcpp& operator+(StringJc str) volatile{ append_s_StringBufferJc((StringBufferJc*)(static_cast<StringBufferJc volatile*>(this)), str, null); return *this; }
  public: StringBufferJcpp& operator<<(StringJc str){ append_s_StringBufferJc(this, str, null); return *this; }

  public: StringBufferJcpp* append(const char* str){append_s0_StringBufferJc(this, str, null); return this; }

  public: StringBufferJcpp& operator=(const char* str){ setLength_StringBufferJc(this, 0, null); append_s0_StringBufferJc(this, str, null); return *this; }

  public: volatile StringBufferJcpp& operator+(const char* str) volatile{append_s0_StringBufferJc((StringBufferJc*)(static_cast<StringBufferJc volatile*>(this)), str, null); return *this; }
  public: StringBufferJcpp& operator<<(const char* str){append_s0_StringBufferJc(this, str, null); return *this; }

  public: StringBufferJcpp* append(int val){ append_i_StringBufferJc(this, val, null); return this; }

  public: volatile StringBufferJcpp& operator+(int val) volatile{append_i_StringBufferJc((StringBufferJc*)(static_cast<StringBufferJc volatile*>(this)), val, null); return *this; }
  public: StringBufferJcpp& operator<<(int val){append_i_StringBufferJc(this, val, null); return *this; }

  public: void setLength(int newLength){ setLength_StringBufferJc(this, newLength, null); }

  public: StringBufferJcpp* delete_(int start, int end){ delete_StringBufferJc(this, start, end, null); return this; }

  public: void cleanToSize(){ cleanToSize_StringBufferJc(this); }

  public: StringJc toString(){ return toString_StringBufferJc(this); }  

  public: operator StringJc() volatile{ return toString_StringBufferJc((StringBufferJc*)(static_cast<StringBufferJc volatile*>(this))); }

  public: ObjectJc* toObjectJc(){ return &this->base.object; }
  public: ObjectJcpp* toObjectJcpp(){ return null; }
};


//error compiler 'operator +' must have at least one formal parameter of class type:
//  StringBufferJcpp& operator+(char const*, int);


/**Do not use, because it isn't settled, how to allocate without responsibility of the user
 * but also without garbage collection.
 * Therefore this solution isn't able to use.
 */
StringBufferJcpp& operator+(StringJc src, StringJc src2);


/**Do not use, because it isn't settled, how to allocate without responsibility of the user
 * but also without garbage collection.
 * Therefore this solution isn't able to use.
 */
StringBufferJcpp* new_StringBufferJcpp(int size);

#endif //__CPLUSPLUSJcpp

/*@CLASS_C IntegerJc@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/**Parses the string argument as a signed decimal integer. The characters in the string must all be decimal digits,
 * except that the first character may be an ASCII minus sign '-' ('\u002D') to indicate a negative value.
 * The resulting integer value is returned, exactly as if the argument and the radix 10 were given as arguments
 * to the parseInt(java.lang.String, int) method.
 * @param s String containing the int representation to be parsed
 * @return the integer value represented by the argument in decimal.
 * @throws NumberFormatException - if the string does not contain a parsable integer.
 * @javaLike
 */
METHOD_C int parseInt_IntegerJc(StringJc src);


#endif //__StringJc_h__
