/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 * @content Log Message at console. 
 *
 * @author Hartmut Schorrig, Germany, Pinzberg
 * @version sf-0.83
 * list of changes:
 * 2008-04-22: Hartmut creation as poor implementation
 * 2008-12-01: Hartmut implementation of the LogMessage-Concept
 *
 ************************************************************************************************/


//include the own header at first to test its correct dependencies.
#include "Fwc/fw_LogMessage.h"

#include "Fwc/fw_Exception.h"
#include "Jc/ReflectionJc.h"
//#include "fw_basic.h"
#include "Jc/ObjectJc.h"
#include <string.h>
//#include "fw_date.h"



typedef struct MsgOutputConsole_MSG_t
{
  /**The C-like-Interfaces. 
   * NOTE: More as one are a unit, because there content are congruent
   *       and they have to be start at begin of struct because casting.
   */  
  union
  { /**The data based on object, necessary for check. */
    ObjectJc object;
    /**C-Interface-struct for Log-Output. */
    LogMessageFW_i logStateFw;
  } ifc;

  //Mtbl_Message_ifcFW const* mtbl;

  /**Temporary buffer for 1 line, used inside run. It contains the last outputted line. */
  char buffer[1000];


}MsgOutputConsole_MSG_s;

extern int const sizeof_MsgOutputConsole_MSG;
int const sizeof_MsgOutputConsole_MSG = sizeof(MsgOutputConsole_MSG_s);


/*@INTERNAL_PROTOTYPES MsgOutputConsole_MSG ************************************************************/
/**Check wether the instanceof is correct, return the type-correct reference. */

METHOD_C MsgOutputConsole_MSG_s* castO_MsgOutputConsole_MSG(ObjectJc* zthis);


extern ClassJc const reflection_MsgOutputConsole_MSG;



//MsgOutputConsole_MSG_s* 
LogMessageFW_i* ctorO_MsgOutputConsole_MSG(ObjectJc* othis)
{
  extern ClassJc const reflection_MsgOutputConsole_MSG;
  MsgOutputConsole_MSG_s* ythis = (MsgOutputConsole_MSG_s*)othis;
  ctorc_ObjectJc(othis);
  setReflection_ObjectJc(othis, &reflection_MsgOutputConsole_MSG, 0);
  return &ythis->ifc.logStateFw;
}


LogMessageFW_i* create_LogMessageConsole(ThCxt* _thCxt)
{ ObjectJc* othis;
  LogMessageFW_i* out;
  STACKTRC_TENTRY("create_MsgOutputConsole_MSG");
  othis = alloc_ObjectJc(sizeof(MsgOutputConsole_MSG_s), 0, _thCxt);
  out = ctorO_MsgOutputConsole_MSG(othis); 
  STACKTRC_LEAVE; return out;

}




MsgOutputConsole_MSG_s* castO_MsgOutputConsole_MSG(ObjectJc* zthis)
{ MsgOutputConsole_MSG_s* ythis = (MsgOutputConsole_MSG_s*)zthis;
  //TODO check type via reflection, fast algorithm because used in any interface method.
  ASSERT(instanceof_ObjectJc(zthis, &reflection_MsgOutputConsole_MSG));
  //ASSERT(ythis->mtbl == &mtbl_MsgDispatcher_MSG);  //an derivated class isnot accepted! 
  return ythis;
}




/**Write to console. This routine should only be called in the MsgDispatcher_MSG.run()-Method.
 */ 
bool sendMsgVaList_MsgOutputConsole_MSG
( struct LogMessageFW_t* ithis
, int32 identNumber
, OS_TimeStamp timestamp
, StringJc text
, Va_listFW args
//, struct VaArgBuffer_t const* const args
, ThCxt* _thCxt
)
{ 
  MsgOutputConsole_MSG_s* ythis = castO_MsgOutputConsole_MSG(&ithis->base.object); //(MsgDispatcher_MSG_s*)zthis;
  int posBuffer;
  //os_setCurrentThreadContext(_threadContext);
  //int lengthBuffer;
  char* buffer2;
  int maxBuffer = sizeof(ythis->buffer);
  STACKTRC_TENTRY("writeFile_MsgDispatcher_MSG");
  
  posBuffer = toStringFormat_Fw(ythis->buffer, maxBuffer, &timestamp, "\nYY-MM-dd-hh:mm:ss.SSS: ", 2);
  buffer2 = ythis->buffer + posBuffer;
  maxBuffer -= posBuffer;

  posBuffer = sprintf(buffer2, "%5.5i :", identNumber);
  buffer2 += posBuffer; 
  maxBuffer -= posBuffer;
  { //va_list argList;
    //argList = (char*)args;  //TODO preliminary for Msc6
    posBuffer = _prepareOutputText_LogMessageFW(buffer2, maxBuffer, identNumber, timestamp, text, args, _thCxt); 
  }
  printf(ythis->buffer);


  STACKTRC_LEAVE; return true;
}



bool isOnline_MsgOutputConsole_MSG_F(LogMessageFW_i* ythis, ThCxt* _thCxt)
{ //do noting.
  return true; //everytime online.
}


void flush_MsgOutputConsole_MSG_F(LogMessageFW_i* ythis, ThCxt* _thCxt)
{ //do noting.
}


void close_MsgOutputConsole_MSG_F(LogMessageFW_i* ythis, ThCxt* _thCxt)
{ //do noting.
}




/*@REFLECTION MsgOutputConsole_MSG ************************************************************/

char const sign_Mtbl_MsgOutputConsole_MSG[] = "sign_Mtbl_MsgOutputConsole_MSG";


/**To organize dynamic link method call the jump table of virtual methods is neccessary. */
struct Mtbl_MsgOutputConsole_MSG_t
{ MtblHeadJc head;
  Mtbl_LogMessageFW LogMessage_ifcFW;
  MtblHeadJc end;
} const mtbl_MsgOutputConsole
=
{ { sign_Mtbl_MsgOutputConsole_MSG
  , (struct Size_Mtbl_t*)( (0+2) * sizeof(void*)) 
  }
, { { sign_Mtbl_LogMessageFW
    , (struct Size_Mtbl_t*)( (4+2)  * sizeof(void*))
    }
  , sendMsgVaList_MsgOutputConsole_MSG
  , flush_MsgOutputConsole_MSG_F
  , close_MsgOutputConsole_MSG_F
  , isOnline_MsgOutputConsole_MSG_F
  , { { sign_Mtbl_ObjectJc, (struct Size_Mtbl_t*)( (5+2)  * sizeof(void*))}
    , clone_ObjectJc_F
    , equals_ObjectJc_F
    , finalize_ObjectJc_F
    , hashCode_ObjectJc_F
    , toString_ObjectJc_F
    }
  }
  , { signEnd_Mtbl_ObjectJc, null}
};


extern ClassJc const reflection_LogMessageFW_i;

static struct interfaces_MsgOutputConsole_MSG_t
{ /** Base data of every array */
  ObjectArrayJc head;
  ClassOffset_idxMtblJc data[1];
}interfaces_MsgOutputConsole_MSG =
{ CONST_ObjectArrayJc(ClassOffset_idxMtblJc, 1, OBJTYPE_ClassOffset_idxMtblJc, null, null)
, { {&reflection_LogMessageFW_i, OFFSET_Mtbl(struct Mtbl_MsgOutputConsole_MSG_t, LogMessage_ifcFW) }
  }
};


   
ClassJc const reflection_MsgOutputConsole_MSG =
{ CONST_ObjectJc(OBJTYPE_ClassJc + sizeof(ClassJc), &reflection_MsgOutputConsole_MSG, null)
, "MsgOutputConsole_MSG"
, 0     //posObjectJc
, sizeof(MsgOutputConsole_MSG_s)
, null  //attributes and associations
, null  //method
, null  //superclass
, (ClassOffset_idxMtblJcARRAY*)&interfaces_MsgOutputConsole_MSG  //interfaces
, 0x4000000  //modifiers
, &mtbl_MsgOutputConsole.head
};


