/************************************************************************************************
 * Copyright/Copyleft:
 *
 * For this source the LGPL Lesser General Public License,
 * published by the Free Software Foundation is valid.
 * It means:
 * 1) You can use this source without any restriction for any desired purpose.
 * 2) You can redistribute copies of this source to everybody.
 * 3) Every user of this source, also the user of redistribute copies
 *    with or without payment, must accept this license for further using.
 * 4) But the LPGL ist not appropriate for a whole software product,
 *    if this source is only a part of them. It means, the user
 *    must publish this part of source,
 *    but don't need to publish the whole source of the own product.
 * 5) You can study and modify (improve) this source
 *    for own using or for redistribution, but you have to license the
 *    modified sources likewise under this LGPL Lesser General Public License.
 *    You mustn't delete this Copyright/Copyleft inscription in this source file.
 *
 * This source may be used also with another licence, if the author 
 * and all other here named co-authors have agreed to this contract.
 * Especially a company can use a copy of this sources in its products without publishing.
 * The user should have a underwritten contract therefore.
 *
 * @author Hartmut Schorrig, Germany, Pinzberg, www.vishia.org
 *
 **copyright***************************************************************************************
 *
 */
#ifndef __OSAL_os_file_h__
#define __OSAL_os_file_h__

#include "OSAL/inc/os_time.h"


/**The ANSI-C doesn't define the sys/stat.h, only POSIX-Standard. Therefore not all platforms implement this features.
 * This file supplies a commonly useable interface.
 */ 


/**Structure is filled on call of os_getFileDescription(...).
 */ 
typedef struct OS_FileDescription_t
{
  /**Files which are longer as 2 GByte aren't supported. */
  int32 fileLength;

  /**Some flag bits. Use macros to request it. */
  int32 flags;

  #define mExist_OS_FileDescription 1
  #define mCanRead_OS_FileDescription 2
  #define mCanWrite_OS_FileDescription 4

  OS_TimeStamp timeChanged;

  char absPath[224];   //absolute path

} OS_FileDescription;


/**Fills the given dst with description informations of the file.
 * @param addPathLength number of additional chars of the file path. It may be 0 normally. 
 *                      The struct OS_FileDescription contains 224 chars for the absolute file path.
 *                      If this parameter is >0, an additional memory space after the dst spaces should be given.
 */
int os_getFileDescription(OS_FileDescription* dst, int addPathLength, const char* filepath); 

#define os_exists_FileDescription(OS_FileDescription) (OS_FileDescription.flags & 0x1) 
#define os_canRead_FileDescription(OS_FileDescription) (OS_FileDescription.flags & 0x2) 
#define os_canWrite_FileDescription(OS_FileDescription) (OS_FileDescription.flags & 0x4) 


#endif //__OSAL_os_file_h__
