package org.vishia.exampleJava2C.java4c;

import org.vishia.exampleJava2C.emulationEnv.MainEmulation;
import org.vishia.exampleJava2C.emulationEnv.WayActuator;
import org.vishia.exampleJava2C.emulationEnv.WaySensor;
import org.vishia.exampleJava2C.emulationEnv.iRequireMainController;
import org.vishia.mainCmd.LogMessageConsole;
import org.vishia.util.LogMessage;
import org.vishia.util.LogMessageFile;
import org.vishia.util.MsgDispatcher;

/**This class is the example main class to control a movement.
 * 
 * @author JcHartmut
 * @java2c=staticInstance. CC:Instances of this class will be created 
 *                     only in the construction path of the system.
 */ 
public class MainController
{
  /**Aggregation The message interface. It is public to enable static access via MainController.singleton. */
  public final LogMessage msg;
  
  /**Composition The message dispatcher. 
   * It is not embedded in C because the amount of memory depends from arguments (Queue size etc.
   */
  public final MsgDispatcher msgDispatcher = new MsgDispatcher(/*maxNrofDispatchEntries*/ 100, /*sizeQueue*/ 200, 28, 28);

  /**Embedded Composition a log message output to file. */
  public final LogMessageFile logMsgFileCtrlValues = new LogMessageFile("testLog/ctrl$MMdd_HHmmss$.log", 10, 1, msgDispatcher.getSharedFreeEntries());
  
  /**Index to identify the LogMsg output to file.*/
  public final static int kMsgOutFile = 4, kMsgOutFileGC = 6; 
  
  public final LogMessageFile logMsgFileGC = new LogMessageFile("testLog/GC_$MMdd_HHmmss$.log", 10, 1, msgDispatcher.getSharedFreeEntries());
  
  /**Aggregation to a way sensor for input for control. */
  private final WaySensor way1Sensor, way2Sensor;
  
  /**Aggregation to a way actuator, servomotor or such, output for control. */
  private final WayActuator way1Actuator, way2Actuator;
  
  /** The set value for the way in 0.1mm-units from above -3 m to 3 m. */
  private short wWay = 10000;  //1 m as initial.
  
  /**controller offset (deviation). */
  private int dWay;
  
  private short[] dWay1 = new short[1];
  
  /**Simple set value generator as first example: add 0.5 mm per step in cycle time. */
  private short addWay = 5;
  
  /**A Set value generator used hence second example using new. */
  private final SetValueGenerator mySetValueGenerator;
  
  private SetValueGenerator.Target.OnlyTest.InnerTest testEnvJ2c;
  
  /**The actuating variable for the movement. It is a voltage for a motor in range -10000 to 10000, unit milliVolt. */
  private short yOut;

  private short[] yOut2;
  
  /**The PID controller. 
   * CC: In C it is an embedded struct because the reference is final (given from construction, fix)
   *     and created here.
   */
  private final PID_controller pidCtrl1 = new PID_controller();
  
  /**The PID controller as universal control module. */
  private final CtrlBase pidCtrl2 = pidCtrl1;
  
  /**This class helps to read target positions and its velocity from a text file. */
  private final ReadTargetFromText targetReader = new ReadTargetFromText();
  
  /**A interface reference to test.
   * CC: This reference uses a reference struct with 2 values, the reference to the Object_Jc
   *     and a value which contains the index of jump table of the instance for interface method call.
   */
  private Testifc testifc;
  
  /**A reference to test the garbage collection.
   * CC: This reference uses a reference struct with 2 values, the reference to the instance
   *     and a value which contains beside the index of jump table of the instance for dynamic method call
   *     the index of back reference from the instance block in BlockHeap.
   */
  private TestClass testDynamicReference;
  
  
  /**A reference to test which references need not 2 values.
   * final, it means it is given from startup and it is never destroyed if this instance itself is not destroyed.
   * TestClass, it is a final class so no virtual methods are called.
   */
  final TestClassFinal testStaticReference;
  
  final TestArrayExamples testArrayExamples = new TestArrayExamples(123);
  
  /**The MainController is a singelton, the constructor is private. */
  public static MainController singleton;
  
  
  /*
  public void create(iRequireMainController broker, LogMessageJc msg, TestClassFinal testStaticReference)
  { singleton = new MainController(broker, msg, testStaticReference);
    
  }
  */
  
  /**The constructor of this Controller assumes, that the depended instances all built always. 
   * @param broker Interface contains all require methods to depended instances.
   */
  public MainController(iRequireMainController broker, LogMessage msg1, TestClassFinal testStaticReference)
  { singleton = this;
    //int maxNrofDispatchEntries = 100; //max number of different ranges in identifier numbers of messages.
    //int sizeQueue = 200; //max nrof messages where are queued temporary to process in the dispatcher thread.
    //some tests, debug it.
    msgDispatcher.setOutputRange(0, 1000, 0x200, MsgDispatcher.mAdd, 4);
    msgDispatcher.setOutputRange(12, 15, 0x1, MsgDispatcher.mSet, 4);
    msgDispatcher.setOutputRange(5, 7, 0x2, MsgDispatcher.mSet, 4);
    msgDispatcher.setOutputRange(20, 22, 0x4, MsgDispatcher.mSet, 4);
    msgDispatcher.setOutputRange(13, 13, 0x8, MsgDispatcher.mSet, 4);
    msgDispatcher.setOutputRange(6, 20, 0x10, MsgDispatcher.mSet, 4);
    msgDispatcher.setOutputRange(0, 4, 0x1, MsgDispatcher.mSet, 4);
    msgDispatcher.setOutputRoutine(kMsgOutFile, "FileCtrl", true, logMsgFileCtrlValues);
    msgDispatcher.setOutputRoutine(kMsgOutFileGC, "FileGC", false, logMsgFileGC);
    msgDispatcher.setOutputRange(0, 1000, 1<<kMsgOutFile, MsgDispatcher.mAdd, 4);
    //this.msg = LogMessage.convertFromMsgDispatcher(msgDispatcher);
    this.msg = msgDispatcher;
    /**Example of a input for dispatching control. */
    String msgCtrl = "1..999: 0x1; 5: 0x1; 0..1002:FileGC + FileCtrl; 2000..2003:FileGC + FileCtrl; ";
    /**If an error occurs, the buffer contains the error message plus input at fault position.
     * Here no allocated String should be used, because this routine should used in a system
     * without garbage-collection. 
     * @java2c=stackInstance, fixStringBuffer.
     * The StringBuffer is created in the stackin C!
     */
    final StringBuffer errorBuffer = new StringBuffer(100);
    String sError = msgDispatcher.setOutputFromString(msgCtrl, errorBuffer);
    if(sError != null)
    { msg.sendMsg(1, "Error parsing msg dispatcher control string: %s.", errorBuffer.toString());
    }
    /**Sends test messages, to test the message dispatcher.*/
    msg.sendMsg(1, "Test to console %s", "test-string");
    msg.sendMsg(1000, "Test to dispatched files. numbers= %d, %d", 1,2);
    
    //this.msg = msg;
    mySetValueGenerator = new SetValueGenerator(msg);
    this.testStaticReference = testStaticReference;
    way1Sensor = broker.requireWay1Sensor();
    way2Sensor = broker.requireWay2Sensor();
    way1Actuator = broker.requireWay1Actuator();
    way2Actuator = broker.requireWay2Actuator();

    testArrayExamples.test();
  }
  
 
  /**Gets the set value in units mm*/
  public short getWaySetvalue(){ return wWay; }
  
  /**Gets the offset of the controller on input*/
  public int getWayOffset(){ return dWay; }
  
  
  
  /**prepares the working of the controller, called on startup from outside.
   * 
   */
  public void startup()
  { wireFunction();
    targetReader.test();
  }
  
  
  public void wireFunction()
  { pidCtrl1.connectInput(dWay1);
    yOut2 = pidCtrl1.provideOutput();
  }
  
  /**This is the cyclical called method to execute the controlling. It is called from outside. */
  public void step()
  {
    //reads the inputs from Hardware, it is in units 1 micrometer.
    int xWay = way1Sensor.getWay();

    //first example: way moves up and down between 1.000 m and 2.000 m in 5 mm per step (1m/s if sample time = 5 millisec)
    boolean bFirstExample = false;
    if(bFirstExample)
    { if(wWay >= 20000)
      { /*The way is greater the maximum. */
        addWay = -50; 
        pidCtrl1.setIntg((short)-yOut);
  
      }
      else if(wWay <= 10000)
      { addWay = 50; 
        pidCtrl1.setIntg((short)-yOut);
      }
      wWay += addWay; 
      if(wWay >=20000) wWay =1;  //test Java2C only!
    }
    else
    { //for good example:
      wWay = mySetValueGenerator.step();
    }
    /**difference of way in units 1 Micrometer, range should be -32..32 mm. */
    dWay = xWay - 100 * wWay;

    /* If the difference is outside a capturing range, the controller is off 
     * and the output value is set immediately. 
     */
    if(dWay > 30000)
    { //The way is out of range, move down.
      yOut = -20000;
      pidCtrl1.setIntg(yOut);
    }
    else if(dWay < -30000)
    { //The way is out of range, move up.
      yOut = 20000;
      pidCtrl1.setIntg(yOut);
    }
    else
    { /*call of PID-control. */ 
      dWay1[0] = (short)(-dWay);  //Input for PID
      pidCtrl2.step(999);
      yOut = yOut2[0];
      //yOut = pidCtrl1.step((short)(-dWay));
      /*The yOut have an output range from -16768 to 16767, limit the output value: */ 
      if(yOut > 19999)
      { yOut = 19999; 
        /* the next statement is commented, because it may be "forgotten",
         * therefore the integral value runs away and has an overflow than.
         * the overflow is recognized and an exception is thrown.
         * If the statement is not commented, the controller is stable.
         */
        //pidCtrl1.setIntg(yOut);
      }
      else if(yOut < -19999)
      { yOut = -19999; 
        //next statement commented, see above.
        //pidCtrl1.setIntg(yOut);
      }
    }  
    
    way1Actuator.setMotorVoltage((short)(yOut>>1));
      
  }

  /**Get the internal value of the integrator of the first way controller. */
  public float getWay11Intg(){ return pidCtrl1.getIntg(); }

  /**stores a next target point. This method may be called in a slower thread.
   * If there is no space for storing the demand, the value isn't be stored.
   * The method should be called later once more.
   * 
   * @param targetValue the target point value.
   * @return true if it is stored, false if there is no space to storing. 
   */ 
  public boolean setTarget(short targetValue)
  { //it is a delagation pattern. The outside user don't know the SetValueGenerator.
    return mySetValueGenerator.setTarget(targetValue);
  }
  

}


class TestSuper
{ int b;
  TestSuper(int a)
  { b=a;
  }
}
