package org.vishia.java2C;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;

/**This class contains all standard types. It is instanciated as singleton. 
 * Most of its members are static. */
public final class CRuntimeJavalikeClassData
{
  /**The singleton association will be set as first operation in the constructor.
   * It means, the instance is available, if it is firstly created. 
   * A null-check ensures, that the constructor is called only one time.
   */
  public static CRuntimeJavalikeClassData singleton;
  
  /**Base of all types contained in {@link LocalIdents} and therefore {@link ClassData},
   * which are defined in {@link Java2C_Main.CRuntimeJavalikeClassData}. This types and identifier
   * are accessible in all scopes. See also {@link #userTypes}.
   */
  public final LocalIdents stdTypes;

  /**This type identifies a place-holder for variable arguments. */
  public final static ClassData clazz_va_argRaw = new ClassData("", "va_argRaw","va_argRaw", "v", '.', null, '%', "+final+primitive+");
  /**This type identifies the va_list type defined in stdarg.h of standard-C. */
  public final static ClassData clazz_va_list = new ClassData("", "Va_list","Va_listFW", "v", '.', "Fwc/fw_LogMessage", '$', "+final+embedded+");
  
  /**This class is used for unknown types for auto-declaration of methods, 
   * which are called but not found in translated sources or *.stc-files. */
  public final static ClassData clazz_unknown = new ClassData("", "void","void", "V", '.', "stdarg", '?', "+final+unknown+");
  
  public final static ClassData clazz_void = new ClassData("", "void","void", "V", '.', "stdarg", '%', "+final+primitive+");
  public final static ClassData clazz_voidPtr = new ClassData("", "void*","void*", "V", '.', "stdarg", '*', "+final+primitive+");
  public final static ClassData clazz_voidVolatile = new ClassData("", "void volatile*","void volatile*", "V", '.', "stdarg", '*', "+final+primitive+");
  public final static ClassData clazz_bool = new ClassData("", "boolean", "bool", "b", 'Z', null, '%', "+final+primitive+");
  public final static ClassData clazz_int8 = new ClassData("", "byte", "int8", "i", 'B', null, '%', "+final+primitive+");
  public final static ClassData clazz_int16 = new ClassData("", "short", "int16", "i", 'S', null, '%', "+final+primitive+");
  public final static ClassData clazz_int32 = new ClassData("", "int", "int32", "i", 'I', null, '%', "+final+primitive+");
  public final static ClassData clazz_int64 = new ClassData("", "long", "int64", "l", 'J', null, '%', "+final+primitive+");
  public final static ClassData clazz_int = new ClassData("", "int", "int", "i", 'I', null, '%', "+final+primitive+");
  public final static ClassData clazz_s0 = new ClassData("", "s0", "char*", "z", 'z', null, '%', "+final+primitive+");   //String-Literal "xyz"
  public final static ClassData clazz_char = new ClassData("", "char", "char", "c", 'C', null, '%', "+final+primitive+");
  public final static ClassData clazz_float = new ClassData("", "float", "float", "f", 'F', null, '%', "+final+primitive+");
  public final static ClassData clazz_double = new ClassData("", "double", "double", "f", 'D', null, '%', "+final+primitive+");
  public final static ClassData clazzObjectJc = new ClassData("java/lang/", "Object", "ObjectJc", "O", 'L', "Jc/ObjectJc", '*', "");
  public final static ClassData clazzStringJc = new ClassData("java/lang/", "String", "StringJc", "S", 's', "Jc/StringJc", 't', "+final+embedded+primitive+");
  public final static ClassData clazzAllocInBlockJc = new ClassData("org/vishia/bridgeC/", "AllocInBlock", "BlockHeapJc", "X", '*', "BlockHeap/BlockHeapJc", '*', "+final+embedded+primitive+");
  
  { clazzObjectJc.addMethod("clone", "clone", Method.modeOverrideable, clazzObjectJc.classTypeInfo);
    clazzObjectJc.addMethod("equals", "equals", Method.modeOverrideable, clazz_bool.classTypeInfo, new ClassData[]{clazzObjectJc} );
    clazzObjectJc.addMethod("finalize", "finalize", Method.modeOverrideable, clazz_void.classTypeInfo);
    clazzObjectJc.addMethod("hashCode", "hashCode", Method.modeOverrideable, clazz_int32.classTypeInfo);
    clazzObjectJc.addMethod("toString", "toString", Method.modeOverrideable + Method.modeNoThCxt, clazzStringJc.classTypeInfo);
    clazzObjectJc.completeInheritanceWithOwnMethods();
  }
  
  public final ClassData clazzExceptionJc = new ClassData("java/lang/", "Exception", "ExceptionJc", "Exc", 'x', "Fwc/fw_Exception", '*', "");
  public final ClassData clazzLogMessageFW = new ClassData("org/vishia/util/", "LogMessage", "LogMessageFW_i", "LogM", 'L', "Fwc/fw_LogMessage", '*', "+final+ObjectJc+");
  public final ClassData clazzLogMessageConsole = new ClassData("org/vishia/MainCmd/", "LogMessageConsole", "LogMessageConsole", "Logc", 'L', "MsgDisp/LogMessageConsole", '*', "+final++ObjectJc+");
  public final ClassData clazzVaArgBuffer = new ClassData("org/vishia/bridgeC/", "VaArgBuffer", "VaArgBuffer", "Vargb", 'L', "MsgDisp/VaArgBuffer", '*', "+final+");
  public final ClassData clazzInteger = new ClassData("java/lang/", "Integer", "IntegerJc", "Intg", 'L', "Jc/ObjectJc", '*', "");
  public final ClassData clazzDouble = new ClassData("java/lang/", "Double", "DoubleJc", "Double", 'L', "Jc/ObjectJc", '*', "");
  public final ClassData clazzMath = new ClassData("java/lang/", "Math", "MathJc", "Math", 'L', "Jc/ObjectJc", '*', "");
  public final ClassData clazzArrays = new ClassData("java/util/", "Arrays", "ArraysJc", "Arrays", 'L', "Jc/ArraysJc", '*', "");
  public final ClassData clazzDate = new ClassData("java/util/", "Date", "DateJc_s", "Date", 'L', "Jc/DateJc", '*', "+ObjectJc+");
  public final ClassData clazzSimpleDateFormat = new ClassData("java/text/", "SimpleDateFormat", "SimpleDateFormatJc_s", "Dateformat", 'L', "Jc/DateJc", '*', "+ObjectJc+");
  public final ClassData clazzTextFieldPositionJc = new ClassData("java/text/", "FieldPosition", "TextFieldPositionJc_s", "FieldPos", 'L', "Jc/DateJc", '*', "+ObjectJc+");
  public final ClassData clazzFormatterJc = new ClassData("java/util/", "Formatter", "FormatterJc_s", "Fm", 'L', "Jc/FormatterJc", '*', "+ObjectJc+");
  public final ClassData clazzListJc = new ClassData("java/util/", "List", "ListJc", "Li", 'L', "Jc/ListJc", '*', "");
  public final ClassData clazzIteratorJc = new ClassData("java/util/", "Iterator", "IteratorJc", "Iter", 'L', "Jc/ListJc", '*', "");
  public final ClassData clazzCollectionJc = new ClassData("java/util/", "Collection", "CollectionJc", "Collect", 'L', "Jc/ListJc", '*', "");
  public final ClassData clazzConcurrentLinkedQueue = new ClassData("java/util/", "ConcurrentLinkedQueue", "ConcurrentLinkedQueueJc", "Clq", 'L', "Jc/ConcurrentLinkedQueueJc", '*', "");

  public final ClassData clazzThreadJc = new ClassData("java/lang/", "Thread", "ThreadJc", "Th", 'L', "Jc/ThreadJc", '*', "");

  
  public final ClassData clazzStringBufferJc; // = new ClassData("java/lang/", "StringBuilder", "StringBufferJc", "Sb", 'b', "Jc/StringBufferJc", '*', "+thx++ObjectJc+");
  public final ClassData clazzMemC = new ClassData("org/vishia/bridgeC/", "MemC", "MemC", "M", 'm', "Fwc/fw_MemC", '$', "+final+embedded+xxxprimitive+");
  public final ClassData clazzOS_TimeStamp = new ClassData("org/vishia/bridgeC/", "OS_TimeStamp", "OS_TimeStamp", "t", 't', "OSAL/inc/os_time", '$', "+final+embedded+xxxprimitive+");
  
  public final ClassData clazzSystemJc = new ClassData("java/lang/", "System", "SystemJc", "System", 'L', "Jc/SystemJc", '*', "+final+");
    
  public final ClassData clazzPrintStreamJc = new ClassData("java/io/", "PrintStream", "PrintStreamJc", "PrintSt", 'L', "Jc/SystemJc", '*', "+final+");
  public final ClassData clazzFileWriterJc = new ClassData("java/io/", "FileWriter", "FileWriterJc_s", "fWr", 'L', "Jc/FileIoJc", '*', "+ObjectJc+");

  public static final FieldData fieldObjectJc = new FieldData("ObjectJc", clazzObjectJc, null, null, null, '.', '*', 0, null, '.', null);
  
  
  
  /**Typical 1-param arrays. */
  private final ClassData[] param_s0 = { clazz_s0};
  private final ClassData[] param_int8 = { clazz_int8};
  private final ClassData[] param_int16 = { clazz_int16};
  private final ClassData[] param_int = { clazz_int};
  private final ClassData[] param_int32 = { clazz_int32};
  private final ClassData[] param_char = { clazz_char};
  private final ClassData[] paramStringJc = { clazzStringJc};
  private final ClassData[] param_float = { clazz_float};
  private final ClassData[] param_double = { clazz_double};
  
  //private final FieldData field_voidP = new FieldData("void", clazz_void, '.', '*', 0, null, '.', null);
  private final FieldData field_raw = new FieldData("raw", clazz_void, null, null, null, '.', '%', 0, null, '.', null);
  private final FieldData field_boolean = new FieldData("boolean", clazz_bool, null, null, null, '.', '%', 0, null, '.', null);
  private final FieldData field_int = new FieldData("int", clazz_int, null, null, null, '.', '%', 0, null, '.', null);
  private final FieldData field_int16 = new FieldData("int", clazz_int16, null, null, null, '.', '%', 0, null, '.', null);
  private final FieldData field_int32 = new FieldData("int", clazz_int32, null, null, null, '.', '%', 0, null, '.', null);
  private final FieldData field_int64 = new FieldData("int", clazz_int64, null, null, null, '.', '%', 0, null, '.', null);

  //public final FieldData ident_byte_AY = new FieldData("byte_AY", clazz_int8, '.', '%', 1, null, '%', clazz_int8); 
  private final FieldData field_byteY = new FieldData("int", clazz_int8, null, null, null, '.', 'X', 1, null, '%', null);
  private final FieldData field_intY  = new FieldData("int", clazz_int32, null, null, null,  '.', 'X', 1, null, '%', null);
  private final FieldData field_longY  = new FieldData("long", clazz_int64, null, null, null,  '.', 'X', 1, null, '%', null);
  private final FieldData field_charY = new FieldData("char", clazz_char, null, null, null, '.', 'X', 1, null, '%', null);
  private final FieldData fieldStringJc = new FieldData("StringJc", clazzStringJc, null, null, null, '.', 't', 0, null, '.', null);
  private final FieldData fieldObjectY = new FieldData("ObjectJc", clazzObjectJc, null, null, null, '.', 'X', 1, null, '*', null);
  
  /**This field type identifies a place-holder for variable arguments. */
  public final FieldData field_va_argRaw = new FieldData("va_argRaw", clazz_va_argRaw, null, null, null, '.', '%', 1, null, '%', null);
  
  /**The type MemC is used in C only as embedded type. */
  private final FieldData field_MemC = new FieldData("MemC", clazzMemC, null, null, null, '.', '%', 0, null, '.', null);
  private final FieldData fieldOS_TimeStamp = new FieldData("OS_TimeStamp", clazzOS_TimeStamp, null, null, null, '.', '%', 0, null, '.', null);

  public final Method methodASSERT = new Method(null, null, "assert", "assert", "ASSERT", Method.modeNoThCxt + Method.modeStatic, field_boolean, null, null, "");
  
  public final FieldData field_int_va_arg = new FieldData("int_va_arg", clazz_int, null, null, null, '.', '%', 1, null/*new String[]{"11"}*/, '%', null);
  
  CRuntimeJavalikeClassData(RunRequiredFirstPass_ifc runRequiredFirstPass) 
  throws FileNotFoundException, IllegalArgumentException, IOException, IllegalAccessException, InstantiationException, ParseException
  { 
    
    /**Create the package for java.lang and java.util and add JavaSrc in the package tree,
     * which have its representation with stc-files. The stc-Files will be translated if the types are needed.
     */
    JavaSrcTreePkg javaSrcPkgJavaStd = new JavaSrcTreePkg("java/", "java");
    JavaSrcTreePkg javaSrcPkgLang = javaSrcPkgJavaStd.getOrAddPkg("java/lang/", "lang");
    JavaSrcTreePkg javaSrcPkgUtil = javaSrcPkgJavaStd.getOrAddPkg("java/util/", "util");
    stdTypes = javaSrcPkgLang.getLocalIdents();
    assert(CRuntimeJavalikeClassData.singleton == null);
    CRuntimeJavalikeClassData.singleton = this;  //now CRuntimeJavalike.singleton.stdTypes are able to access.
    //
    JavaSrcTreeFile javaSrcStringBuffer = javaSrcPkgLang.setFileJava(null, null, "StringBuffer.java", "Jc/", "Jc", "","Jc", null, false);
    javaSrcPkgLang.setFileJava(null, null, "StringBuilder.java", "Jc/", "Jc", "","Jc", null, false);

    javaSrcPkgLang.setFileJava(null, null, "RuntimeException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    javaSrcPkgLang.setFileJava(null, null, "ClassCastException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    javaSrcPkgLang.setFileJava(null, null, "NullPointerException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    javaSrcPkgLang.setFileJava(null, null, "IndexOutOfBoundsException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    javaSrcPkgLang.setFileJava(null, null, "ArrayIndexOutOfBoundsException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    javaSrcPkgLang.setFileJava(null, null, "StringIndexOutOfBoundsException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    javaSrcPkgLang.setFileJava(null, null, "ArrayStoreException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    javaSrcPkgLang.setFileJava(null, null, "IllegalArgumentException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    javaSrcPkgLang.setFileJava(null, null, "NoSuchFieldException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    javaSrcPkgLang.setFileJava(null, null, "NoSuchMethodException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    javaSrcPkgLang.setFileJava(null, null, "IllegalStateException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    
    JavaSrcTreePkg javaSrcPkgIO = javaSrcPkgJavaStd.getOrAddPkg("java/io/", "io");
    javaSrcPkgIO.setFileJava(null, null, "IOException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    javaSrcPkgIO.setFileJava(null, null, "UnsupportedEncodingException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    //
    JavaSrcTreePkg javaSrcPkgText = javaSrcPkgJavaStd.getOrAddPkg("java/text/", "text");
    javaSrcPkgText.setFileJava(null, null, "ParseException.java", "Fwc/fw_Exception", null, "","Jc", null, false);
    //
    
    stdTypes.putClassType("java", javaSrcPkgJavaStd);
    /**First add all defined files resp public classes to the standard types: */
    //stdTypes.putClassTypesAll(javaSrcPkgLang.getPkgLevelIdents());
    /**Second add all standard types to the package. Necessary for first pass, also for stc-File: */
    //javaSrcPkgLang.addStdTypes(stdTypes);  //contains classes.
    
    stdTypes.putClassType(clazz_void);
    stdTypes.putClassType(clazz_bool);
    stdTypes.putClassType(clazz_int32);
    stdTypes.putClassType(clazz_s0);
    
    clazz_int32.addCastFromType(clazz_int8, "?", "%%");
    clazz_int32.addCastFromType(clazz_int16, "?", "%%");
    clazz_int32.addCastFromType(clazz_int, "?", "%%");
    clazz_int.addCastFromType(clazz_int8, "?", "%%");
    clazz_int.addCastFromType(clazz_int16, "?", "%%");
    clazz_int.addCastFromType(clazz_int32, "?", "%%");
    
    clazz_int64.addCastFromType(clazz_int8, "?", "%%");
    clazz_int64.addCastFromType(clazz_int16, "?", "%%");
    clazz_int64.addCastFromType(clazz_int32, "?", "%%");
    clazz_int64.addCastFromType(clazz_int32, "?", "%%");
    stdTypes.putClassType("long", "int64", clazz_int64, "%", null);
    
    stdTypes.putClassType("short", "int16", clazz_int16, "%", null);
    stdTypes.putClassType("byte", "int8", clazz_int8, "%", null);
    stdTypes.putClassType("float", "float", clazz_float, "%", null);
    stdTypes.putClassType("double", "double", clazz_double, "%", null);
    stdTypes.putClassType("char", "char", clazz_char, "%", null);
    stdTypes.putClassTypeStandard(clazzObjectJc);
   
    
    { clazzAllocInBlockJc.addMethod("setRunModeAll", "setRunModeAll", Method.modeStatic, clazz_void.classTypeInfo);
      stdTypes.putClassType(clazzAllocInBlockJc);
    }
    
    { //clazz_va_list.addMethod("get", "get_Va_list", 0, clazz_va_list.classTypeInfo);
      /**NOTE: get() isn't a method, it is the access immediately. In Java a method is necessary: */
      clazz_va_list.addMethod("get", "!", Method.modeNoThCxt, clazz_va_list.classTypeInfo);
      stdTypes.putClassType(clazz_va_list);
    }
    
    { clazzExceptionJc.addMethod("getMessage", "getMessage", 0, clazzStringJc.classTypeInfo);
      stdTypes.putClassType("Exception", clazzExceptionJc);
    }
    
    stdTypes.putClassTypeStandard(clazzStringJc);
    
    
    
    //clazzStringBufferJc = stdTypes.getTypeContingentlyRunFirstPass("StringBuffer", runRequiredFirstPass);
    clazzStringBufferJc = stdTypes.getType("StringBuffer", null);
    stdTypes.putClassType(clazzStringBufferJc);
    
    stdTypes.putClassType("LinkedList", "LinkedListJc", null, "@", null);
    
    stdTypes.putClassType("List", "LinkedListJc", clazzListJc, "@", null);
    clazzListJc.addMethod("add", "add_Collection", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32, clazzObjectJc});
    clazzListJc.addMethod("add", "add", clazz_int32.classTypeInfo, new ClassData[]{});
    clazzListJc.addMethod("addAll", "addAll", clazz_int32.classTypeInfo, new ClassData[]{clazzCollectionJc});
    clazzListJc.addMethod("addAll", "addAll_i", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32, clazzCollectionJc});
    clazzListJc.addMethod("clear", "clear", 0, clazz_int32.classTypeInfo);
    clazzListJc.addMethod("contains", "contains", clazz_int32.classTypeInfo, new ClassData[]{clazzObjectJc});
    clazzListJc.addMethod("containsAll", "containsAll", clazz_int32.classTypeInfo, new ClassData[]{clazzCollectionJc});
    clazzListJc.addMethod("equals", "equals", clazz_int32.classTypeInfo, new ClassData[]{clazzObjectJc});
    clazzListJc.addMethod("get", "get", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32});
    clazzListJc.addMethod("hashCode", "hashCode", 0, clazz_int32.classTypeInfo);
    clazzListJc.addMethod("indexOf", "indexOf", clazz_int32.classTypeInfo, new ClassData[]{clazzObjectJc});
    clazzListJc.addMethod("isEmpty", "isEmpty", 0, clazz_int32.classTypeInfo);
    clazzListJc.addMethod("iterator", "iterator", 0, clazz_int32.classTypeInfo);
    clazzListJc.addMethod("lastIndexOf", "lastIndexOf", clazz_int32.classTypeInfo, new ClassData[]{clazzObjectJc});
    clazzListJc.addMethod("listIterator", "listIterator", 0, clazz_int32.classTypeInfo);
    clazzListJc.addMethod("listIterator", "listIterator_i", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32});
    clazzListJc.addMethod("remove", "remove_i", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32});
    clazzListJc.addMethod("remove", "remove_O", clazz_int32.classTypeInfo, new ClassData[]{clazzObjectJc});
    clazzListJc.addMethod("removeAll", "removeAll", clazz_int32.classTypeInfo, new ClassData[]{clazzCollectionJc});
    clazzListJc.addMethod("retainAll", "retainAll", clazz_int32.classTypeInfo, new ClassData[]{clazzCollectionJc});
    clazzListJc.addMethod("set", "set", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32, clazzObjectJc});
    clazzListJc.addMethod("size", "size", 0, clazz_int32.classTypeInfo);
    clazzListJc.addMethod("subList", "subList", clazz_int32.classTypeInfo, new ClassData[]{clazz_int32, clazz_int32});
    clazzListJc.addMethod("toArray", "toArray", 0, clazz_int32.classTypeInfo);
    clazzListJc.addMethod("toArray", "toArray_Array", clazz_int32.classTypeInfo, new ClassData[]{});
    
    
    stdTypes.putClassType("Iterator", "IteratorJc", clazzIteratorJc, "@", null);
    clazzIteratorJc.addMethod("hasNext", "hasNext", 0, clazz_bool.classTypeInfo);
    clazzIteratorJc.addMethod("next", "next", 0, clazzObjectJc.classTypeInfo);
    clazzIteratorJc.addMethod("remove", "remove", 0, clazz_void.classTypeInfo);
    
    
    stdTypes.putClassType("ArrayList", "ListJc", null, "@", null);
    stdTypes.putClassType("LinkedList", "ArrayListJc", null, "@", null);
    stdTypes.putClassType("ConcurrentLinkedQueue", "ConcurrentLinkedQueueJc", null, "@", null);

    { ClassData[] params = { clazz_int32};
      //clazzStringJc.addMethod("append", "append_i_thcStringJc", clazzStringBufferJc.classTypeInfo, params );
      clazzStringJc.addMethod("new", "new_bYiiCharset", 0, clazzStringJc.classTypeInfo, new FieldData[]{ field_byteY, clazz_int32.classTypeInfo, clazz_int32.classTypeInfo, clazzStringJc.classTypeInfo} );
      clazzStringJc.addMethod("new", "new_cY", 0, clazzStringJc.classTypeInfo, new FieldData[]{ field_charY} );
      clazzStringJc.addMethod("new", "new_cYi", 0, clazzStringJc.classTypeInfo, new FieldData[]{ field_charY, field_int, field_int} );
      clazzStringJc.addMethod("charAt", "charAt", Method.modeNoThCxt, clazz_char.classTypeInfo, param_int32 );
      clazzStringJc.addMethod("equals", "equals", Method.modeNoThCxt, clazz_int32.classTypeInfo, new ClassData[]{ clazzStringJc} );
      clazzStringJc.addMethod("indexOf", "indexOf", Method.modeNoThCxt, clazz_int32.classTypeInfo, new ClassData[]{ clazz_char} );
      clazzStringJc.addMethod("indexOf", "indexOf_i", Method.modeNoThCxt, clazz_int32.classTypeInfo, new ClassData[]{ clazz_char, clazz_int32} );
      clazzStringJc.addMethod("indexOf", "indexOf_S", Method.modeNoThCxt, clazz_int32.classTypeInfo, new ClassData[]{ clazzStringJc} );
      clazzStringJc.addMethod("indexOf", "indexOf_Si", Method.modeNoThCxt, clazz_int32.classTypeInfo, new ClassData[]{ clazzStringJc, clazz_int32} );
      clazzStringJc.addMethod("startsWith", "startsWith", Method.modeNoThCxt, clazz_bool.classTypeInfo, new ClassData[]{ clazzStringJc} );
      clazzStringJc.addMethod("lastIndexOf", "lastIndexOf", Method.modeNoThCxt, clazz_int32.classTypeInfo, new ClassData[]{ clazz_char} );
      clazzStringJc.addMethod("lastIndexOf", "lastIndexOf_i", Method.modeNoThCxt, clazz_int32.classTypeInfo, new ClassData[]{ clazz_char, clazz_int32} );
      clazzStringJc.addMethod("lastIndexOf", "lastIndexOf_S", Method.modeNoThCxt, clazz_int32.classTypeInfo, new ClassData[]{ clazzStringJc} );
      clazzStringJc.addMethod("lastIndexOf", "lastIndexOf_Si", Method.modeNoThCxt, clazz_int32.classTypeInfo, new ClassData[]{ clazzStringJc, clazz_int32} );
      clazzStringJc.addMethod("length", "length", Method.modeNoThCxt, clazz_int32.classTypeInfo);
      clazzStringJc.addMethod("size", "size", Method.modeNoThCxt, clazz_int32.classTypeInfo);
      clazzStringJc.addMethod("substring", "substring", clazzStringJc.classTypeInfo, new ClassData[]{ clazz_int32, clazz_int32} );
      clazzStringJc.addMethod("substring", "substring_i", clazzStringJc.classTypeInfo, param_int32 );
      clazzStringJc.addMethod("replace", "replace", clazzStringJc.classTypeInfo, new ClassData[]{ clazz_char, clazz_char} );
      clazzStringJc.addMethod("getChars", "getChars", 0, clazz_void.classTypeInfo, new FieldData[]{ field_int, field_int, field_charY, field_int} );
      clazzStringJc.addCastFromType(clazz_s0, "s0_StringJc(?)", "t%");  //NOTE: clazz_s0 is designated as skalar, because no struct type_t should be built.
      clazzStringJc.addCastInitializerFromType(clazz_s0, "CONST_s0_StringJc(?)", "t%");
      //done above: stdTypes.putClassType("String", clazzStringJc);
    }
    {
      clazzConcurrentLinkedQueue.addMethod("ctorO", "ctorO_Clq", clazzConcurrentLinkedQueue.classTypeInfo, new ClassData[]{clazzConcurrentLinkedQueue});
      clazzConcurrentLinkedQueue.addMethod("ctorO", "ctorO_MemC", 0, clazzConcurrentLinkedQueue.classTypeInfo, new FieldData[]{field_MemC});
      clazzConcurrentLinkedQueue.addMethod("add", "add", clazz_bool.classTypeInfo, new ClassData[]{clazz_voidVolatile});
      clazzConcurrentLinkedQueue.addMethod("poll", "poll", 0, clazz_voidVolatile.classTypeInfo);
      stdTypes.putClassType("ConcurrentLinkedQueue", clazzConcurrentLinkedQueue);
    }
    {
      clazzThreadJc.addMethod("sleep", "!os_delayThread", clazz_void.classTypeInfo, param_int);
      stdTypes.putClassType(clazzThreadJc);
    }
    
    
    
    {
      clazzMemC.addMethod("alloc", "!alloc_MemC", Method.modeNoThCxt|Method.modeStatic, field_MemC, param_int);
      stdTypes.putClassType("MemC", clazzMemC);
    }
    
    {
      clazzOS_TimeStamp.addMethod("ctorM", "!ctor_OS_TimeStamp", Method.modeNoThCxt, clazzOS_TimeStamp.classTypeInfo);
      clazzOS_TimeStamp.addMethod("os_getDateTime", "!os_getDateTime", Method.modeNoThCxt|Method.modeStatic, fieldOS_TimeStamp);
      clazzOS_TimeStamp.addField("time_sec", field_int32);
      clazzOS_TimeStamp.addField("time_nsec", field_int32);
      clazzOS_TimeStamp.addMethod("os_delayThread", "!os_delayThread", Method.modeNoThCxt, field_boolean, param_int32);
      stdTypes.putClassType("OS_TimeStamp", clazzOS_TimeStamp);
    }
    
    { clazzPrintStreamJc.addMethod("println", "println_s0", clazz_void.classTypeInfo, new ClassData[]{ clazz_s0});
      clazzPrintStreamJc.addMethod("println", "println_s", clazz_void.classTypeInfo, new ClassData[]{ clazzStringJc});
      clazzPrintStreamJc.addMethod("println", "println_sb", clazz_void.classTypeInfo, new ClassData[]{ clazzStringBufferJc});
      stdTypes.putClassType("PrintStream", "PrintStreamJc", clazzPrintStreamJc, "@", null);
    }  
    { clazzFileWriterJc.addMethod("ctorO", "ctorO", 0, clazzFileWriterJc.classTypeInfo);
      clazzFileWriterJc.addMethod("open", "open", 0, field_int32, new ClassData[]{ clazzStringJc, clazz_bool });
      clazzFileWriterJc.addMethod("isOpen", "isOpen", Method.modeNoThCxt, clazzFileWriterJc.classTypeInfo);
      clazzFileWriterJc.addMethod("write", "write", 0, clazz_void.classTypeInfo, paramStringJc);
      clazzFileWriterJc.addMethod("flush", "flush", 0, clazz_void.classTypeInfo);
      clazzFileWriterJc.addMethod("close", "close", 0, clazz_void.classTypeInfo);
      clazzFileWriterJc.addField("kFileNotFound", new FieldData("kFileNotFound", clazz_int, null, null, null, 'd', '%', 0, null, '.', clazzFileWriterJc));
      clazzFileWriterJc.addField("kFileOpenError", new FieldData("kFileOpenError", clazz_int, null, null, null, 'd', '%', 0, null, '.', clazzFileWriterJc));
      stdTypes.putClassType(clazzFileWriterJc);
    }  
    { clazzSystemJc.classLevelIdents.putClassElement("out", "PrintStreamJc", clazzPrintStreamJc, 's', '@', "@", 0, null, '.', clazzSystemJc);
      clazzSystemJc.addMethod("arraycopy", "arraycopy", Method.modeStatic, clazz_void.classTypeInfo, new ClassData[]{clazzObjectJc, clazz_int, clazzObjectJc, clazz_int, clazz_int});
      stdTypes.putClassType("System", "SystemJc", clazzSystemJc, "%", null);
    }
    { clazzLogMessageFW.addMethod("sendMsgVaList", "sendMsgVaList", Method.modeOverrideable, clazz_bool.classTypeInfo, new ClassData[]{clazz_int32, clazzOS_TimeStamp, clazzStringJc, clazz_va_list});
      clazzLogMessageFW.addMethod("close", "close", Method.modeOverrideable, clazz_void.classTypeInfo);
      clazzLogMessageFW.addMethod("flush", "flush", Method.modeOverrideable, clazz_void.classTypeInfo);
      clazzLogMessageFW.addMethod("isOnline", "isOnline", Method.modeOverrideable, clazz_bool.classTypeInfo);
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_S", clazz_void.classTypeInfo, new ClassData[]{clazz_int32, clazzStringJc});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_z", clazz_void.classTypeInfo, new ClassData[]{clazz_int32, clazz_s0});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_Szv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_Szv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_Szv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_Szv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_Szv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_Szv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_Szv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_Szv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_tzzv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_tzzv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_tzzv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_tzzv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_tzzv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("sendMsg", "sendMsg_tzzv", 0, clazz_void.classTypeInfo, new FieldData[]{field_int32, clazzOS_TimeStamp.classTypeInfo, fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});
      clazzLogMessageFW.addMethod("report", "write_S", clazz_void.classTypeInfo, new ClassData[]{clazz_int32, clazzStringJc});
      clazzLogMessageFW.addMethod("reportln", "write_iS", clazz_void.classTypeInfo, new ClassData[]{clazz_int32, clazz_int32, clazzStringJc});
      clazzLogMessageFW.addMethod("reportln", "write_S", clazz_void.classTypeInfo, new ClassData[]{clazz_int32, clazzStringJc});
      clazzLogMessageFW.completeInheritanceWithOwnMethods();
      //clazzLogMessageFW.addMethod("convertFromMsgDispatcher", "convertFromMsgDispatcher", clazzLogMessageFW.classTypeInfo, new ClassData[]{clazzMsgDispatcher});
      stdTypes.putClassType("LogMessage", "LogMessageJc", clazzLogMessageFW, "$", null);
      stdTypes.putClassType("Report", "LogMessageJc", clazzLogMessageFW, "$", null);
    }
    { clazzLogMessageConsole.addMethod("create", "create", Method.modeStatic, clazzLogMessageFW.classTypeInfo);
      stdTypes.putClassType("LogMessageConsole", "LogMessageConsole", clazzLogMessageConsole, "$", null);
    }
    { 
      clazzVaArgBuffer.addMethod("get_va_list", "get_va_list", 0, clazz_va_list.classTypeInfo);
      stdTypes.putClassType(clazzVaArgBuffer);
    }
    { stdTypes.putClassType("Integer", "IntegerJc", clazzInteger, "S", null);
      clazzInteger.classLevelIdents.putClassElement("MAX_VALUE", "int32", clazz_int32, 's', '%', "%", 0, null, '.', clazzInteger);
      clazzInteger.classLevelIdents.putClassElement("MIN_VALUE", "int32", clazz_int32, 's', '%', "%", 0, null, '.', clazzInteger);
      //clazzInteger.addMethod("")
      
    }
    { stdTypes.putClassType("Double", "DoubleJc", clazzDouble, "S", null);
      clazzDouble.addMethod("toString", "toString", Method.modeStatic, clazzStringJc.classTypeInfo, new ClassData[]{clazz_double});
      //clazzInteger.addMethod("")
      
    }
    { stdTypes.putClassType("Math", "MathJc", clazzMath, "S", null);
      clazzMath.addMethod("pow", "pow", Method.modeStatic, clazz_double.classTypeInfo, new ClassData[]{clazz_double, clazz_double});  
      //clazzInteger.addMethod("")
      
    }
    { clazzArrays.addMethod("binarySearch", "binarySearch_int_ii", Method.modeStatic, field_int, new FieldData[]{field_intY, field_int32, field_int32, field_int32});        
      clazzArrays.addMethod("binarySearch", "binarySearch_int", Method.modeStatic, field_int, new FieldData[]{field_intY, field_int32});        
      clazzArrays.addMethod("binarySearch", "binarySearch_int", Method.modeStatic, field_int, new FieldData[]{field_longY, field_int64});        
      stdTypes.putClassType("Arrays", "ArraysJc", clazzArrays, "S", null);
    }
    { clazzDate.addMethod("ctorO", "ctorO", 0, clazzDate.classTypeInfo);
      clazzDate.addMethod("UTC", "UTC", Method.modeStatic, field_int64, new FieldData[]{field_int32, field_int32, field_int32, field_int32, field_int32, field_int32});
      clazzDate.addCastToType(clazzOS_TimeStamp, "?->val", "$*");  //OS_TimeStamp is member inside DateJc and contains all relevant informations.  
      stdTypes.putClassType("Date", clazzDate);
    }
    { clazzSimpleDateFormat.addField("DATE_FIELD", field_int);        
      clazzSimpleDateFormat.addMethod("ctorO", "ctorO", 0, clazzSimpleDateFormat.classTypeInfo);        
      clazzSimpleDateFormat.addMethod("ctorO", "ctorO_S", 0, clazzSimpleDateFormat.classTypeInfo, paramStringJc);        
      clazzSimpleDateFormat.addMethod("applyPattern", "applyPattern", 0, clazz_void.classTypeInfo, paramStringJc);        
      clazzSimpleDateFormat.addMethod("format", "format", 0, clazzStringJc.classTypeInfo, new FieldData[]{fieldOS_TimeStamp});        
      clazzSimpleDateFormat.addMethod("format", "format_tSb", 0, clazzStringBufferJc.classTypeInfo, new FieldData[]{fieldOS_TimeStamp, clazzStringBufferJc.classTypeInfo, clazzTextFieldPositionJc.classTypeInfo });        
      stdTypes.putClassType("SimpleDateFormat", clazzSimpleDateFormat);
    }
    { clazzTextFieldPositionJc.addMethod("ctorO", "ctorO", 0, clazzTextFieldPositionJc.classTypeInfo, param_int);        
      stdTypes.putClassType(clazzTextFieldPositionJc);
    }
    { clazzFormatterJc.addMethod("ctorO", "ctorO", 0, clazzFormatterJc.classTypeInfo);        
      clazzFormatterJc.addMethod("ctorO", "ctorO_Sb", 0, clazzFormatterJc.classTypeInfo, new ClassData[]{ clazzStringBufferJc });        
      clazzFormatterJc.addMethod("format", "format", 0, clazzFormatterJc.classTypeInfo, new ClassData[]{ clazzStringJc, clazz_va_list });        
      clazzFormatterJc.addMethod("format", "format_a", Method.modeNoThCxt, clazzFormatterJc.classTypeInfo, new FieldData[]{fieldStringJc, field_va_argRaw });        
      clazzFormatterJc.addMethod("format", "format_a", Method.modeNoThCxt, clazzFormatterJc.classTypeInfo, new FieldData[]{fieldStringJc, field_va_argRaw, field_va_argRaw});        
      clazzFormatterJc.addMethod("format", "format_a", Method.modeNoThCxt, clazzFormatterJc.classTypeInfo, new FieldData[]{fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw});        
      clazzFormatterJc.addMethod("format", "format_a", Method.modeNoThCxt, clazzFormatterJc.classTypeInfo, new FieldData[]{fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});        
      clazzFormatterJc.addMethod("format", "format_a", Method.modeNoThCxt, clazzFormatterJc.classTypeInfo, new FieldData[]{fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});        
      clazzFormatterJc.addMethod("format", "format_a", Method.modeNoThCxt, clazzFormatterJc.classTypeInfo, new FieldData[]{fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});        
      clazzFormatterJc.addMethod("format", "format_a", Method.modeNoThCxt, clazzFormatterJc.classTypeInfo, new FieldData[]{fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});        
      clazzFormatterJc.addMethod("format", "format_a", Method.modeNoThCxt, clazzFormatterJc.classTypeInfo, new FieldData[]{fieldStringJc, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw, field_va_argRaw});        
      stdTypes.putClassType(clazzFormatterJc);
    }
  
  }

  //because there is no parent, the basicly types have to be defined:
  
} 

