package org.vishia.java2C;

import java.util.List;

/**This class holds the data of one variable in a local context respectively a field of a class, 
 * see {@link LocalIdents} 
 * or describe the access to a instance, see {@link CCodeData}.
 */
public class FieldData
{
  /**The name of the identifier. <code>null</code> if it is the description of an access to an unnamed instance. */
  private final String sName;
  
  /**The type respectively reference type of the field. */
  public final ClassData typeClazz;
  
  /**The type of a key if it is a generic container type of keys and values. Elsewhere null. */
  public ClassData keyClazz;
  
  /**The type of a elements of a generic container type. Elsewhere null. */
  public ClassData elementClazz;
  
  /**If this field is set, the instance referenced in the described field is guaranteed an instance
   * of that type, nor from a derived type. It means, no dynamic linked call is necessary,
   * methods can called static using this instance type. This property saves some calculation times,
   * while calling override-able methods of the referenced class.
   * <br>
   * If references are used as interface references for Java test and software design (including
   * the documentation aspect), but the implementation uses the named instance in any case, 
   * this feature should be used.
   * <br>
   * For a reference variable this behavior is possible to control with @ java2c=instanceType:TYPE.
   * This behavior is also set at embedded instance automatically. 
   * This field is null, if the instance isn't predefined.
   */
  public final ClassData instanceClazz;
  
  /**The class where the identifier is member of, null if it is a local identifier. */
  public final ClassData declaringClazz;
  
  /**Char to indicate the location and kind of definition.
   * <ul><li>'S' static constant
   *     <li>'s' static non constant
   *     <li>'d' immediate constant with define
   *     <li>'.' non static
   * </ul>     
   */
  public final char modeStatic;
  
  /**Describes which kind of association is it.
   * <ul>
   * <li>'%' primitive type. The instance contains the value. 
   * <li>'f' final instance. After construction the value is fix.
   * <li>'F' final instance of the same type as 
   * 
   */
  public char xxxmodeInstance;
  
  /**If the identifier is an array with fix size, programmed in java in form of
   * <pre>
   * Type[][] myArray = new Type[expr][expr];
   * </pre>
   * The <code>expr</code> translated in C-Form are stored here. This expr should be a simple value
   * in C. There are used to generate the initialization of an array.
   * <br>
   * <code>null</code> if it isn't an array or it is a dynamic array.
   */
  public final String[] fixArraySizes;
  
  /**0 if it isn't an array. 1.. if it is an array. 
   * if modeArrayElement=='B' it is the fix size of a StringBuffer with direct char*.
   */
  private final int dimensionArrayOrFixSize;
  
  
  /**Kind of the access to an element, if it is an array.
   * The adequate chars are used see {@link #modeAccess}.
   * Special case: If it is 'B', the field describes a <code>StringBufferJc</code> with a fix size.
   * In that case the {@link #dimensionArrayOrFixSize} contains the number of chars inclusive the 4 chars
   * storing in the <code>StringBufferJc</code>-instance with immediate buffer.
   */
  public final char modeArrayElement;
  
  /**Kind of the instance. For non-arrays:
   * <table>
   * <tr><td>$</td><td>an embedded struct</td></tr>
   * <tr><td>%</td><td>a variable of basic primitive type such as int32, float, char,
   *                   but also an embedded indivisible struct like StringJc, MemC, OS_TimeStamp,
   *                   whose elements aren't accessed in Java. </td></tr>
   * <tr><td>*</td><td>a simple reference, pointer in C, used for non-GC-controlled associations.</td></tr>
   * <tr><td>@</td><td>an enhanced reference, for GC (garbage collector) -controlled associations. An index of mtbl is possible.</td></tr>
   * <tr><td>t</td><td>a StringJc, it is an enhanced reference ~REF also, but with special accesses</td></tr>
   * <tr><td>&</td><td>an method-table reference ~MTB, containing C-jump-table with index, GC is not possible</td></tr>
   * <tr><td>~</td><td>ythis as reference (call own method</td></tr>
   * </table>
   * if {@link #dimensionArrayOrFixSize} >0, this element describes the access to the array head. 
   * The kind of elements respectively the access to the data of the elements are described in {@link #modeArrayElement}.
   * with the same key chars as shown above. Independent of the kind of referencing the array
   * the elements may be embedded struct ('$'), references ('*') etc., see {@link #modeArrayElement}.
   * <table>
   * <tr><td>Q</td><td>an embedded array without head. All elements are embedded. The {@link #fixArraySize} is set.</td></tr>
   * <tr><td>Y</td><td>an embedded array with <code>ObjectArrayJc</code> head informations. All elements are embedded. The {@link #fixArraySize} is set.</td></tr>
   * <tr><td>P</td><td>a simple reference to the array elements without head, pointer in C, used for non-GC-controlled associations.</td></tr>
   * <tr><td>X</td><td>a simple reference to <code>ObjectArrayJc</code> head informations, pointer in C, to an array type, used for non-GC-controlled associations.</td></tr>
   * <tr><td>@</td><td>an enhanced reference to an array type, starting with ObjectArrayJc, for GC (garbage collector) -controlled associations.</td></tr>
   * </table>
   */
  public final char modeAccess;
  
  /**see {@link getClassLevel()}*/
  int nClassLevel;

  /**see {@link getOuterLevel()}. Note: this private member is set in the outer class, 
   * see {@link putClassElement(String, String, ClassData, String, ClassData)}
   * or  {@link putLocalElement(String, String, ClassData, String)} 
   */
  int nOuterLevel;
  
  /**initializes the instance node. */
  FieldData
  ( String sName
  , ClassData typeClazz
  , ClassData instanceClass
  , ClassData elementClass
  , ClassData keyClass
  , char staticMode
  , char accessMode
  , int dimensionArray
  , String[] fixArraySizes
  , char modeArrayElement
  , ClassData declaringClazz
  )
  { this.sName = sName;
    this.modeAccess = accessMode;
    //this.sModifier = sModifier; 
    this.fixArraySizes = fixArraySizes; 
    this.declaringClazz = declaringClazz; 
    this.typeClazz = typeClazz;
    this.instanceClazz = instanceClass;
    this.elementClazz = elementClass;
    this.keyClazz = keyClass;
    this.dimensionArrayOrFixSize = dimensionArray;  //NOTE: if modeArrayElement=='B' it is the fix size of a StringBuffer
    //assert(dimensionArray == 0 || dimensionArray == 1);
    if(modeArrayElement == '$')
      stop();
    this.modeArrayElement = modeArrayElement;
    this.modeStatic = staticMode;
  }
  
  
  FieldData
  ( String sName
  , ClassData typeClazz
  , ClassData instanceClass
  , ClassData elementClass
  , ClassData keyClass
  , char staticMode
  , char accessMode
  , char modeArrayElement
  , int dimensionArray
  , List<String> listFixArraySizes
  , ClassData declaringClazz
  )
  { this.sName = sName;
    this.modeAccess = accessMode;
    //this.sModifier = sModifier; 
    this.declaringClazz = declaringClazz; 
    this.typeClazz = typeClazz;
    this.instanceClazz = instanceClass;
    this.elementClazz = elementClass;
    this.keyClazz = keyClass;
    this.dimensionArrayOrFixSize = dimensionArray;  //NOTE: if modeArrayElement=='B' it is the fix size of a StringBuffer
    //assert(dimensionArray == 0 || dimensionArray == 1);
    if(modeArrayElement == '$')
      stop();
    this.modeArrayElement = modeArrayElement;
    this.modeStatic = staticMode;

    if(listFixArraySizes != null){
      int zFixArraySizes = listFixArraySizes.size();
      this.fixArraySizes = zFixArraySizes == 0 ? null : new String[zFixArraySizes];
      int ixFixArraySizes = 0;
      for(String fixArraySize: listFixArraySizes) {
        this.fixArraySizes[ixFixArraySizes++] = fixArraySize;
      }
    }else{
      this.fixArraySizes = null;
    }
          
  }
  
  

  /**Copies given FieldData, but modify with given parameters
   * @param parent //the parent
   * @param classLevelAdd Adds the class level, 0: keep from parent. To copy instances for super classes
   * @param outerLevelAdd Adds the outer level, 0: keep from parent. To copy instances for inner classes
   * @param modeAccessP if '.' than keep parent.modeAccess, else take that.
   */
  FieldData(FieldData parent, int classLevelAdd, int outerLevelAdd, char modeAccessP)
  { this.sName = parent.sName;
    this.modeAccess = modeAccessP == '.' ? parent.modeAccess : modeAccessP;
    this.declaringClazz = parent.declaringClazz; 
    this.typeClazz = parent.typeClazz;
    this.instanceClazz = parent.instanceClazz;
    this.keyClazz = parent.keyClazz;
    this.elementClazz = parent.elementClazz;
    this.dimensionArrayOrFixSize = parent.dimensionArrayOrFixSize; 
    this.fixArraySizes = parent.fixArraySizes;
    this.modeArrayElement = parent.modeArrayElement;
    this.modeStatic = parent.modeStatic;
    this.nClassLevel = parent.nClassLevel + classLevelAdd;
    this.nOuterLevel = parent.nOuterLevel + outerLevelAdd;
  }
  
  
  /*
  FieldData(FieldData parent, int dimensionArrayOrFixSize)
  { //this.sTypeIdent = parent.sTypeIdent; 
    this.sName = parent.sName;
    this.modeAccess = parent.modeAccess;
    //this.sModifier = sModifier; 
    this.fixArraySizes = parent.fixArraySizes; 
    this.clazz = parent.clazz; 
    this.typeClazz = parent.typeClazz;
    this.dimensionArray = (byte)dimensionArrayOrFixSize; 
    this.modeArrayElement = parent.modeArrayElement;
    this.modeStatic = parent.modeStatic;
  }
  */
  
  /**level of the affiliation to a class in the inheritance level.
   * 0= it is a local visible variable. 1=element of this class, 2=element of super class etc.
   */
  public int getClassLevel(){ return nClassLevel; }
  
  
  /**Returns 0 if it isn't an array, 1.. for number of array dimension.*/
  public int getDimensionArray(){ return ".B".indexOf(modeArrayElement) <0 ? dimensionArrayOrFixSize : 0; }

  
  /**Returns the size >0 if the type is a StringBuffer with a immediate String with fix size, else 0.
   */
  public int getFixSizeStringBuffer(){return modeArrayElement=='B'? dimensionArrayOrFixSize : 0; }
  
  /**level of the affiliation to a class in the outer class relation.
   * 0= it is a local visible variable. 1=element of this class, 2=element of outer class etc.
   */
  public int getOuterLevel(){ return nOuterLevel; }
  
  /**returns the name of the element. */
  public String getName(){ return sName; }

  /**returns the name of the type of the element. */
  public String getTypeName(){ return typeClazz.getClassCtype_s(); }
  
  public String getTypeChar()
  { return "" + typeClazz.cVaArgIdent;
  }
  
 
  
  
  /**Generates the C type declaration appropriate to the type info in this FieldData.
   * It is the type code used for definition of the variable.
   * <ul>
   * <li>Simple embedded Arrays: At example type int. It returns only <code>int</code> because
   *     the complete definition is <code>int name[123]</code> or <code>int name[] = { 1, 2, 3}</code> 
   * </ul>
   * <br>
   * If it is an simple array, the type code designating the pointer to the elements requires an additional '*'
   * @return String with C type declaration.
   */
  public String gen_Declaration()
  { final String sPreModifier;
    if(sName != null && sName.equals("int32")) 
      stop();
    switch(modeStatic)
    { case '.': sPreModifier = ""; break;
      case 'd': sPreModifier = ""; break;
      case 's': sPreModifier = ""; break;
      case 'S': sPreModifier = "const "; break;
      default: { sPreModifier = null; assert(false); }
    }
    final String sTypeReturn; //  .sTypeIdent;
    switch(modeAccess)
    { case '$':
      case 't':
      case '%': sTypeReturn = typeClazz.getClassCtype_s(); break;
      case 'C':
      case '*': sTypeReturn = "struct " + typeClazz.getClassIdentName() + "_t*"; break;
      case '@': sTypeReturn = typeClazz.getClassIdentName() + "REF"; break;
      case '&': sTypeReturn = typeClazz.getClassIdentName() + "MTB"; break;
      case 'X': sTypeReturn = typeClazz.getClassIdentName() + "_Y*"; break;
      case 'Y': sTypeReturn = typeClazz.getClassIdentName() + "_Y"; break;
      case 'P': 
      { switch(modeArrayElement)
        { case '%': sTypeReturn = typeClazz.getClassCtype_s() + "*"; ; break;
          case '*': sTypeReturn = "struct " + typeClazz.getClassIdentName() + "_t**"; break;
          case '@': sTypeReturn = typeClazz.getClassIdentName() + "REF*"; break;
          default: sTypeReturn = null;
        }  
      } break;
      case 'Q': 
      { /**embedded array. to use in declaration, at ex. int name[] is produced, here only int.*/
        switch(modeArrayElement)
        { case '%': sTypeReturn = typeClazz.getClassCtype_s() + ""; ; break;
          case '*': sTypeReturn = "struct " + typeClazz.getClassIdentName() + "_t*"; break;
          case '@': sTypeReturn = typeClazz.getClassIdentName() + "REF"; break;
          default: sTypeReturn = null;
        }  
      } break;
      default: sTypeReturn = null;
    }
    assert(sTypeReturn != null);
    return sPreModifier + sTypeReturn;
    
  }

  /**generates the code for a variable definition of the given FieldInfo.
   * @return code for definition of the variable, without ";" on end.
   * This method is only called inside {@link GenerateClass#gen_variableDefinition(org.vishia.zbnf.ZbnfParseResultItem, org.vishia.zbnf.ZbnfParseResultItem, LocalIdents, java.util.List, char)}
   * It supplies all informations which are able to gotten in this class.
   */
  String gen_VariableDefinition()
  {
    if(sName.equals("fixArray"))
      stop();
    
    final String sNameC = modeStatic != '.' ? sName + "_" + declaringClazz.getClassIdentName()
                                            : sName;  //static variable with _Classname
    String typeCode = gen_Declaration();
    if(sName.equals("singleton"))
      stop();
    String ret = "???";
    if(dimensionArrayOrFixSize >0)
    { final String elementType;
      switch(modeArrayElement)
      { case '%': case 't': case '$': elementType = typeClazz.getClassCtype_s(); break;
        case '*': elementType = "struct " + typeClazz.getClassIdentName() + "_t*"; break;
        case '@': elementType = typeClazz.getClassIdentName() + "REF"; break;
        case '&': elementType = typeClazz.getClassIdentName() + "MTB"; break;
        case 'B': elementType =null; assert(modeAccess == '$'); break;
        default: elementType =null; assert(false);
      }
      switch(modeAccess)
      { case 'Y':
        { /**A fix size embedded array, but with head: */
          ret = "struct " + sNameC + "_Y {"
                 + " ObjectArrayJc head; " + elementType + " data[" + fixArraySizes[0] + "]; }"
                 + sNameC;  //NOTE: the ";" after definition is set outside.
        }break;
        case 'Q':
        { /**An simple fix embedded array: */
          if(fixArraySizes != null)
          { ret = typeCode + " " + sNameC;
            for(String fixArraySize: fixArraySizes)
            { ret += "[" + fixArraySize + "]";
            }
          }
          else assert(false);  //it have to be fixArraySizes!    
        }break;
        case 'X':
        { ret = typeCode + " " + sNameC;
          //TODO arrays with more as one dimension.
        } break;
        case '$':
        {
          assert(modeArrayElement == 'B');
          { /**Its a fix String buffer. */
            ret = "struct SbY_" + sNameC + "_t {"
                 + " StringBufferJc sb; char _b[" + (dimensionArrayOrFixSize-4) + "]; }"
                 + sNameC;  //NOTE: the ";" after definition is set outside.
            
          }
          
        }break;
        default: assert(false);  
      }
    }
    else {
      if(modeStatic == 'd'){
        ret = "#define " + sNameC + " " + fixArraySizes[0];    
      }
      else {
        ret = typeCode + " " + sNameC;
      }
    }
    return ret;  
  }

  /**Writes the structure of this instance to a <code>*.stc</code>-file. 
   * It is an class level identifier, see {@link ClassData#classLevelIdents}, 
   * or a return type or argument of a method,
   * because only for such FieldData this method is called. 
   * See {@link LocalIdents#writeStruct(int)} and {@link ClassData.Method#writeStruct()}
   * @return The content to write to <code>*.stc</code>-file. The form of string is the same
   * described in the syntax <code>Java2Cstc.zbnf</code>.
   */
  public String writeStruct() 
  { 
    if(typeClazz == null)
      assert(false);
    if(typeClazz.sClassNameJava.equals("InnerTest"))
      stop();
    
    
    
    String out = "field:{ ";

    //out += typeClazz.sPackage;
    ClassData outerClazz = typeClazz; //start with it.
    while( (outerClazz = outerClazz.outerClazz) != null)
    { //insert left:
      //out += outerClazz.sClassNameJava + "." ;
      stop();
    }
    //out += typeClazz.sClassNameJava;
    out += typeClazz.sClassNameJavaFullqualified;
    
    if(keyClazz != null){
      out += "<" + keyClazz.sClassNameJavaFullqualified + "," + elementClazz.sClassNameJavaFullqualified + ">";
    }
    else if(elementClazz != null){
      out += "<" + elementClazz.sClassNameJavaFullqualified + ">";
    }
    
    if(instanceClazz != null){
      out += " instance:" + instanceClazz.sClassNameJavaFullqualified;
    }
    out += " "
          + modeAccess 
          + modeArrayElement 
          + modeStatic;
    assert(typeClazz.sPackage.length() == 0 || typeClazz.sPackage.endsWith("/"));
    if(modeArrayElement == 'B'){
      out += "-fixBufferSize[" + dimensionArrayOrFixSize + "]"; 
    }
    else {
      /**Array may be with fix size.Either show [??] per size or the size. */
      for(int ii=0; ii<dimensionArrayOrFixSize; ii++)
      { final String sFixArraySize = 
          fixArraySizes == null ? "??"
          : fixArraySizes[ii] == null ? "?"
          : fixArraySizes[ii];
        out += "[" + sFixArraySize + "]";
      }
    }  
    out += " "+ sName + "; ";
    out += "/*info"
        + dimensionArrayOrFixSize
        + nClassLevel + nOuterLevel
        + "*/ }";
    //if()
    return out;
  }    
          
  
  /**Tests whether a cast to the given FieldData type is necessary and executes it.
   * If srcType is the same as this, no cast occurs, the value is returned direct.
   * If srcType is a derive-able type of this, the cast is set with {@link #addCast(ClassData, String)}.
   * That is used to produce the casted return value. The access mode is considered too. 
   * @param srcType The type of the value
   * @param value The value string.
   * @return The value itself if no cast is necessary
   * @param src The Expression with its type and access modes
   * @param modeAccessDst A deviating mode of access for the destination or '.'. If '.', the {@link #modeAcess} 
   *        of this is valid.
   * @return String, either src.cCode if no cast is necessary, or the casted expression.
   */
  public String testAndcast(CCodeData src, char modeAccessDst)
  { if(src.cCode.equals("REFJc(ythis->implifc2)"))
      stop();
    
    final String sRet;
    if(modeAccessDst == '.'){ modeAccessDst = this.modeAccess; }
    if( src.cCode.equals("null"))
    { //a null-pointer should be compatible with all expect StringJc.
      if(typeClazz.sClassNameC.equals("StringJc")
        && dimensionArrayOrFixSize == 0
        )
      { 
        sRet = "null_StringJc"; }
      else {
        sRet = "null"; } 
    }
    else {
      
      if(src.cCode.startsWith("requireWay1Sensor_iRequireMainController(broker, _thCxt)"))
        stop();
      
      
      ClassData.CastInfo castInfo;
      if(  this.typeClazz == src.identInfo.typeClazz
        || src.identInfo.typeClazz == Java2C_Main.singleton.standardClassData.clazz_unknown  //unknown type, don't cast, it is okay:
        || src.modeAccess == '?'                       //unknown type, don't cast, it is okay:
        || this.typeClazz == Java2C_Main.singleton.standardClassData.clazz_unknown  //unknown type, don't cast, it is okay:
        || this.typeClazz == Java2C_Main.singleton.standardClassData.clazz_void  //void as argument is a void*
        || this.typeClazz == Java2C_Main.singleton.standardClassData.clazz_va_argRaw  //any type is valid
        ) {
        sRet = testAndChangeAccess(modeAccessDst, src.cCode, src.modeAccess);
      }
      else if( (castInfo = src.identInfo.typeClazz.getCastInfoToType(typeClazz.sClassNameC)) != null
             ||(castInfo = typeClazz.getCastInfoFromType(src.identInfo.typeClazz.sClassNameC)) != null
             ){
        /**cast src to this type: */
        String sSrcCasted = testAndChangeAccess(castInfo.modeAccessSrc, src.cCode, src.modeAccess);
        String sSrc1 = castInfo.pre + sSrcCasted + castInfo.post;
        sRet = testAndChangeAccess(modeAccessDst, sSrc1, castInfo.modeAccessDst);
      }
      else if(src.modeAccess =='X' && this.typeClazz==CRuntimeJavalikeClassData.clazzObjectJc){
        /**Cast of a array to Object reference: */
        sRet = testAndChangeAccess(modeAccessDst, "(" + src.cCode + ")->head.object", '$');
      }
      else if(src.modeAccess =='Y' && this.typeClazz==CRuntimeJavalikeClassData.clazzObjectJc){
        /**Cast of a array to Object reference: */
        sRet = testAndChangeAccess(modeAccessDst, "(" + src.cCode + ").head.object", '$');
      }
      else {
        /**No castInfo, but the types are different. It is a derivation. */
        String sClassNameDst = typeClazz.sClassNameC;
        if(typeClazz.isPrimitiveType()){
          String sSrcCasted = testAndChangeAccess('%', src.cCode, src.modeAccess);
          sRet = "((/*J2C:cast%*/" + sClassNameDst + ")(" + sSrcCasted + "))";
        } 
        else if("t$".indexOf(typeClazz.classTypeInfo.modeAccess) >=0){
          /**The dst should be a value (embedded). */
          String sSrcCasted = testAndChangeAccess('$', src.cCode, src.modeAccess);
          sRet = "((/*J2C:cast$*/" + sClassNameDst + ")(" + sSrcCasted + "))";
        } 
        else {
          /**Normal pointer cast. */
          String sSrcCasted = testAndChangeAccess('*', src.cCode, src.modeAccess);
          sRet = "((/*J2C:cast*/" + sClassNameDst + "*)(" + sSrcCasted + "))";
        }
      }
    }                  
    return sRet;
    
  }

  
  /**Changes the access.
   * The '@' is handled like '*', because the building of enhanced reference is done outside.
   * @param modeAccessDst Necessary access, ones of chars <code>&*@~%?</code>, see {@link #modeAccess}
   * @param sSrc1 The cCode of src
   * @param modeAccessSrc Existing access of sSrc1
   * @return
   */
  public static String testAndChangeAccess(char modeAccessDst, String sSrc1, char modeAccessSrc)
  { String sAccessParam;
    if(modeAccessSrc == modeAccessDst || modeAccessSrc == '?' || modeAccessDst == '?' ) {
      sAccessParam = sSrc1;
    } else if("&".indexOf(modeAccessSrc)>=0 && "*~".indexOf(modeAccessDst)>=0) {
      sAccessParam = " (" + sSrc1 + ".ref)";
    } else if("@".indexOf(modeAccessSrc)>=0 && "*~".indexOf(modeAccessDst)>=0) {
      sAccessParam = "REFJc(" + sSrc1 + ")";
    } else if("@*~&".indexOf(modeAccessSrc)>=0 && "%$".indexOf(modeAccessDst)>=0) {
      sAccessParam = "* (" + sSrc1 + ")";
    } else if("%$".indexOf(modeAccessSrc)>=0 && "@*~&".indexOf(modeAccessDst) >=0){
      sAccessParam = "& (" + sSrc1 + ")";
    } else if("@*~&".indexOf(modeAccessSrc)>=0 && "@*~&".indexOf(modeAccessDst) >=0){
      /**Reference and this-reference are compatible. */
      sAccessParam = sSrc1;
    } else if("%$t".indexOf(modeAccessSrc)>=0 && "%$t".indexOf(modeAccessDst) >=0){
      /**embedded or primitive type are not different. */
      sAccessParam = sSrc1;
    } else {
      assert(false);
      sAccessParam = null;
    } 
    return sAccessParam;
  }
  
  
  
  private String xxxtestAndcast(ClassData srcType, String value)
  { String sRet;
    if( value.equals("null"))
    { //a null-pointer should be compatible with all expect StringJc.
      if(typeClazz.sClassNameC.equals("StringJc")
        && dimensionArrayOrFixSize == 0
        )
      { 
        sRet = "null_StringJc"; }
      else {
        sRet = value; } 
    }
    else
    { //TODO may be some array tests
      
      sRet = typeClazz.xxxtestAndcast(srcType, value, '.');
    }
    return sRet;    
  }


  /**builds a String representation especially to test under eclipse. */
  public String toString()
  { String sRet = typeClazz.getClassIdentName() + " " + sName + " " + nClassLevel + nOuterLevel + modeAccess + "[" + modeArrayElement + "]" + modeStatic + nClassLevel + nOuterLevel
           + (declaringClazz != null ? declaringClazz.sClassNameC : " -stacklocal");
    return sRet;
  }


  
  
  /**It's a debug helper. The method is empty, but it is a mark to set a breakpoint. */
  void stop()
  { //debug
  }



  

}
