package org.vishia.java2C;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**This class contains all information of available java sources while translation process.
 */
public class JavaSources
{
  
  public interface ClassDataOrJavaSrcFile
  {
    /**Returns the ClassData of the instance, or <code>null</code>. If the instance is of type ClassData,
     * it returns the instance itself. If the instance is of type {@link JavaSrcTreeFile}, it returns
     * the associated ClassData, if the file is translated yet or the stc-file is read already.
     * If the file isn't translated, it returns null.*/
    ClassData getClassData();
    
    /**Returns the instance if it is a JavaSrcTreeFile, else <code>null</code>. */
    JavaSrcTreeFile getJavaSrc();
  
    /**Returns the instance if it is a JavaSrcTreePkg, else <code>null</code>. */
    JavaSrcTreePkg getJavaPkg();
  
    /**Returns the name of the class or package. It is the public class in a file.
     */
    String getTypeName();
    
    
    /**Returns the known Identifier of the Type. */
    LocalIdents getLocalIdents();
  }
  
  
  /**Tree of all found java-files in the source path, independent of their usage as source for translation.
   * The files at the leafs of the tree {@link JavaSrcTreeFile} contains the information, 
   * whether they should translate.
   * <br>
   * This JavaFolder is the root folder, without name and package designation. 
   * It contains the first level package instances. 
   */
  public final JavaSrcTreePkg javaSrcTree = new JavaSrcTreePkg("", "");

   /**All files to translate.
   */
  final List<JavaSrcTreeFile> listJavaSrcFilesToTranslate = new LinkedList<JavaSrcTreeFile>();

  /**Index (Map) of all known packages to fast search via its name. 
   * The key contains the package path separated and ending with <code>/</code>.
   */
  final Map<String, JavaSrcTreePkg> indexJavaSrcPkgs = new TreeMap<String, JavaSrcTreePkg>();
  
  
}
