package org.vishia.java2C;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

/**Representation of an existing Java file.
 */
public class JavaSrcTreeFile implements JavaSources.ClassDataOrJavaSrcFile
{
  
  private final String sFileName;
  
  /**The name of the public class of this file. In Java only one class can be public in a file.
   */
  private final String sPublicClassName;
  
  private final String sJavaSrcPath;

  /**Pre- and Suffixes to build the class name in C additional to the java-name of classes. */
  private final String sPrefixClassNameC, sSuffixClassNameC;
  
  private final String sStcPath;
  
  private final String sFilePathC;

  private final JavaSrcTreePkg itsPkg;
  
  private final File fileJava;
  
  /**True if this file is to be translate. */
  private final boolean bToTranslateToC;
  
  /**If set, the file is translated, the classData presents its content for foreign use.
   */
  ClassData classData = null;
  
  /**Initializes the instance.
   * @param itsPkg The package where the file is member of.
   * @param javaSrcPath The source path where the file is found first time.
   * @param fileJava The File instance ready to open source file.
   * @param sFileNameJava The file name without path but with extension <code>.java</code> without directories.
   * @param prefixCFile The prefix to build the C class name and the C file name. 
   *        The part after the last slash <code>/</code> is the prefix for class names.
   *        The prefix should ended with slash if the file name has no prefix, 
   *        but a path to store the to create *.c and *.h files are given.
   *        The prefix may contain <code>""</code>.
   *        <br>
   *        If the prefix is null, the param suffixCFile contains the complete C-File-name. 
   *        This is only possible for Files, which are not translated from Java2C, 
   *        but it is parsed their stc-File. In this case the stc-File contains the valid 
   *        <code>struct</code> names for C. No {@link#sPrefixClassNameC} or suffix is necessary than.
   * @param suffixCFile The suffix to build the C class name and the C file name.
   * @param prefixNames
   * @param suffixNames
   * @param stcPath null or a special path where the stc-file is located.
   * @param translateToC false than the file should not be translate to C,
   *        instead the stc-file should be used any time. This is if it is a standard class 
   *        or a users class, which is located in a library.
   */
  public JavaSrcTreeFile(JavaSrcTreePkg itsPkg
  , String javaSrcPath, File fileJava, String sFileNameJava
  , String prefixCFile, String suffixCFile, String prefixNames, String suffixNames, String stcPath
  , boolean translateToC
  )
  {
    super();
    assert(sFileNameJava.endsWith(".java"));
    this.sPublicClassName = sFileNameJava.substring(0, sFileNameJava.length()-5); // without ".java"
    this.sJavaSrcPath = javaSrcPath;
    this.fileJava = fileJava;
    this.sFileName = sFileNameJava;
    int posFolderPrefix1 = prefixCFile.lastIndexOf('/');
    int posFolderPrefix2 = prefixCFile.lastIndexOf('\\');
    int posFolderPrefix = posFolderPrefix1 >=0 && posFolderPrefix1 > posFolderPrefix2 ? posFolderPrefix1 : posFolderPrefix2;
    //sPrefixClassNameC = posFolderPrefix >=0 ? prefixCFile.substring(posFolderPrefix+1): prefixCFile; //may be empty, but not null.
    sPrefixClassNameC = prefixNames; //may be empty, but not null.
    sSuffixClassNameC = suffixNames;
    sFilePathC = prefixCFile + (suffixCFile == null ? "" : sPublicClassName + suffixCFile);
    sStcPath = stcPath;
    this.bToTranslateToC = translateToC;
    this.itsPkg = itsPkg;
  }
  
  /**returns <code>null</code>if the file is not translated, returns the ClassData if it is translated.
   * @see org.vishia.java2C.JavaSources.ClassDataOrJavaSrcFile#getClassData()
   */
  public ClassData getClassData()
  {
    return classData;  //null if not translated
  }


  /**returns the instance because it is.
   * @see org.vishia.java2C.JavaSources.ClassDataOrJavaSrcFile#getJavaSrc()
   */
  public JavaSrcTreeFile getJavaSrc()
  {
    return this;
  }
  
  
  
  /**
   * @param fileJava
   */
  void infoSecondFile(File fileJava)
  { //TODO
    
  }
  
  
  public String getPublicClassName(){ return sPublicClassName; }
  
  public LocalIdents getPkgLevelTypes(){ return itsPkg.getPkgLevelIdents(); }
  
  public String getFileNameC(){ return sFilePathC; } 
  
  /**Returns the source path where the file is found. 
   * It is only a info, where the file comes from, not to process.
   */
  public String getJavaSrcPathInfo(){ return sJavaSrcPath; }
  
  public String toString()
  { String info= this.sJavaSrcPath + this.itsPkg.getPkgPath() + this.sFileName 
    + (bToTranslateToC ? "=:" + this.getFileNameC() : " : " + this.sStcPath );
    return info;
  }  

  public String reportInfo()
  { String info= (bToTranslateToC ? " *--" : " +--") + sFileName
          + " : " + sFilePathC + " (Location: " + sJavaSrcPath + ")";
    return info;
  }  

  public String getClassCNamePrefix(){ return sPrefixClassNameC; }

  public String getClassCNameSuffix(){ return sSuffixClassNameC; }
  
  public void setClassData(ClassData data)
  { assert(classData == null);  //set only one time.
    assert(sPublicClassName.equals(data.sClassNameJava));
    classData = data;
  }
  
  public boolean isToTranslate(){ return bToTranslateToC; }

  public boolean isTranslated(){ return classData != null; }
  
  
  public final File getFileJava()
  {
    return fileJava;
  }

  public LocalIdents getLocalIdents()
  {
    if(classData != null){
      return classData.classLevelIdents;
    } else {
      return null;  //no local idents available.
    }
  }

  
  /**Implements {@link JavaSources.ClassDataOrJavaSrcFile#getJavaPkg()}
   */
  public JavaSrcTreePkg getJavaPkg()
  {
    return null; // it isn't a JavaSrcTreePkg
  }

  /**Implements {@link JavaSources.ClassDataOrJavaSrcFile#getTypeName()}
   */
  public String getTypeName()
  {
    return sPublicClassName;
  }
  
  

}


