package org.vishia.java2C;

import java.io.File;
import java.io.FileFilter;
import java.text.ParseException;
import java.util.Iterator;
import java.util.List;

import org.vishia.mainCmd.Report;

/**This class is a helper to get the java source tree from given class path.
 * The class may be used only temporary, 
 * it is wrapped around its method {@link JavaSrcTreeGetter#gatherAllJavaSrcFiles(JavaFolder, List)}.
 * The usage is (new JavaSrcTreeGetter(report)).captureAllJavaSrcFiles(dst, src).
 *
 */
public class JavaSrcTreeGetter
{

  private final Report console;
  
  private final ConfigSrcPathPkg_ifc inputCfg;
  
  /**Aggregation to the Instance, which stores the produced data 
   * while call of {@link #captureAllJavaSrcFiles(List, JavaSrcTreePkg, List)}.
   */
  private final JavaSources dstJavaSrcData;
  
  private final List<String> listInputToTranslate;
  
  private FileFilter javaFileFilter = new FileFilter(){
    public boolean accept(File file)
    { return file.isFile() && file.getName().endsWith(".java");
      }
    };
    
  private FileFilter dirFilter = new FileFilter(){
      public boolean accept(File file)
      { return file.isDirectory();
      }
    };
    
 
  /**Constructs an instance, which should be used only temporary to capture the tree.
   * @param inputCfg Association to the instance, which knows the assignment between Java-Packages and C-Pre- and Suffixes.
   * @param console
   */
  public JavaSrcTreeGetter(JavaSources dstJavaSrcData, ConfigSrcPathPkg_ifc inputCfg, List<String> listInputToTranslate, Report console)
  {
    this.console = console;
    this.inputCfg = inputCfg;
    this.dstJavaSrcData = dstJavaSrcData;
    this.listInputToTranslate = listInputToTranslate;
  }

  /**Captures all Java Files, which are found at file system in all given Java Source pathes
   * evaluates the content of {@link ConfigSrcPathPkg_ifc} (association given on constructor)
   * and sets respectively completes the input files.
   * Uses the {@link #inputCfg} aggregation to get the source path and the package replacement
   * 
   * @param javaSrcTree The destination, it should be an empty List before.
   * @param listJavaSrcpath List of all source pathes. The list won't be changed here.
   * @param listInputToTranslate List of all sources to translate. The list won't be changed here.
   * @throws ParseException 
   */
  public void gatherAllJavaSrcFiles(LocalIdents identsToAssignFirstPkgIdents, List<String> listJavaSrcpath) throws ParseException
  {
    for(String srcPath: listJavaSrcpath){
      File javaFolder = new File(srcPath);
      if(!javaFolder.isDirectory()) throw new IllegalArgumentException("Java-Srcpath should be a directory: " + srcPath);
      File[] firstPkgs = javaFolder.listFiles(dirFilter);
      for(File firstPkg: firstPkgs){
        captureAllJavaSrcFilesRecursive(identsToAssignFirstPkgIdents, firstPkg, dstJavaSrcData.javaSrcTree, "", srcPath, false);
      }
    }
    if(console.getReportLevel()>=Report.fineInfo){
      console.reportln(Report.fineInfo, "===All found Java source files==="); 
      reportAllJavaSrcFiles(dstJavaSrcData.javaSrcTree);
    }
  }
  
  
  /**Captures the package folders and java files of the given file directory level.
   * @param dirPkg The directory in file tree for package.
   * @param dstParent write packages and files into it.
   * @param sPkgPath package path with / as separator and at end for the given level
   * @param javaSrcPath The source path which is captured yet. Path before package tree.
   * @throws ParseException 
   */
  private void captureAllJavaSrcFilesRecursive(LocalIdents identsToAssignPkgIdents, File dirPkg, JavaSrcTreePkg dstParent, String sPkgPathParent, String javaSrcPath, boolean bTranslatePkg) throws ParseException
  { String sNamePkg = dirPkg.getName();
    String sPkgPath = sPkgPathParent + sNamePkg + "/";    
    /**Register the package. */
    JavaSrcTreePkg dstJavaPkg = dstParent.getOrAddPkg(sPkgPath, sNamePkg); //, identsToAssignPkgIdents);
    dstJavaSrcData.indexJavaSrcPkgs.put(sPkgPath, dstJavaPkg);
    identsToAssignPkgIdents.putClassType(sNamePkg, dstJavaPkg);  //add in LocalIdents
    /**Check whether the whole package is to translate to C: */
    { Iterator<String> iter = listInputToTranslate.iterator();
      while(!bTranslatePkg && iter.hasNext()){
        String sPkgCheckTranslate = iter.next();
        if(sPkgCheckTranslate.equals(sPkgPath)){
          bTranslatePkg = true;
        }
      }
    }  
    /**check packages (directories) */
    File[] javaPkgs = dirPkg.listFiles(dirFilter);
    for(File subFolder: javaPkgs){
      captureAllJavaSrcFilesRecursive(dstJavaPkg.getLocalIdents(), subFolder, dstJavaPkg, sPkgPath, javaSrcPath, bTranslatePkg);
    }
    /**check files */
    File[] javaFiles = dirPkg.listFiles(javaFileFilter);
    if(javaFiles.length>0){
      /**Get prefix and suffix for C-name: */
      //dstJavaPkg.addStdTypes(Java2C_Main.singleton.standardClassData.stdTypes);
      if(sPkgPath.equals("org/vishia/java2C/test/"))
        stop();
      /**Prefix and suffix from packageReplacement from config file. */
      final String sFilePrefixPkg, sFileSuffixPkg, sNamePrefixPkg, sNameSuffixPkg;
      { ConfigSrcPathPkg_ifc.Set preSuffix = inputCfg.getCPathPrePostfixForPackage(sPkgPath);
        if(preSuffix == null){ /**The package is not found in package replacement: */
          sFilePrefixPkg = "";
          sFileSuffixPkg = "";
          sNamePrefixPkg = "";
          sNameSuffixPkg = "";
        }
        else{ /**Use the package replacement: */
          sFilePrefixPkg = preSuffix.getFilePrefix();
          sFileSuffixPkg = preSuffix.getFileSuffix();
          sNamePrefixPkg = preSuffix.getNamePrefix();
          sNameSuffixPkg = preSuffix.getNameSuffix();
        }
      }  
      /**Iterate all files: */
      for(File fileJava: javaFiles){
        String nameFileJava = fileJava.getName();
        String nameJava = nameFileJava.substring(0, nameFileJava.length()-5);  //always extension ".java", without it!
        /**Prefix and suffix from packageReplacement from config file. */
        final String sFilePrefix, sFileSuffix, sNamePrefix, sNameSuffix;
        { ConfigSrcPathPkg_ifc.Set preSuffix = inputCfg.getCPathPrePostfixForPackage(sPkgPath+nameJava);
          if(preSuffix == null){ /**The file is not found in file replacement, use package replacement: */
            sFilePrefix = sFilePrefixPkg;
            sFileSuffix = sFileSuffixPkg;
            sNamePrefix = sNamePrefixPkg;
            sNameSuffix = sNameSuffixPkg;
          }
          else{ /**Use the file replacement: */
            sFilePrefix = preSuffix.getFilePrefix();
            sFileSuffix = preSuffix.getFileSuffix();
            sNamePrefix = preSuffix.getNamePrefix();
            sNameSuffix = preSuffix.getNameSuffix();
          }
        }  
        //String pathC = sFilePrefix + nameJava + sFileSuffix;
        /**Check whether it is to translate to C:*/
        assert(nameFileJava.endsWith(".java"));
        String sPublicClassName = nameFileJava.substring(0, nameFileJava.length()-5); // without ".java"
    
        ClassData existingStdType = Java2C_Main.singleton.standardClassData.stdTypes.getType(sPublicClassName, null);
        if(existingStdType == null){
          /**If the type is registered already as standard type (at ex from org/vishia/bridgeC), than do not handle.*/
          
          boolean bTranslate= bTranslatePkg;
          if(!bTranslate)
          { /**Search whether the java-file is to translate: look in the listInputToTranslate. */
            Iterator<String> iter = listInputToTranslate.iterator();
            String sFilePathName = sPkgPath + nameFileJava;
            while(!bTranslate && iter.hasNext()){
              String sCheckTranslate = iter.next();
              if(sCheckTranslate.equals(sFilePathName)){
                bTranslate = true;
              }
            }
          }  

          JavaSrcTreeFile javaSrc = dstJavaPkg.setFileJava(
              javaSrcPath, fileJava, nameFileJava
            , sFilePrefix, sFileSuffix, sNamePrefix, sNameSuffix, null, bTranslate);
          if(bTranslate){
            dstJavaSrcData.listJavaSrcFilesToTranslate.add(javaSrc);
          }
        } else
          stop();
      }
    }
    
    
  }


  private void reportAllJavaSrcFiles(JavaSrcTreePkg javaFolderParent)
  {
    for(JavaSrcTreePkg javaFolder: javaFolderParent.listChildren()){
      console.reportln(Report.fineInfo, javaFolder.getPkgPath());
      javaFolder.reportClasses(console);
      reportAllJavaSrcFiles(javaFolder);
    }
  }
  
  void stop(){}

}  

