package org.vishia.java2C;

import java.io.File;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;
import java.util.Map;

import org.vishia.mainCmd.Report;
import org.vishia.util.SortedTree;

/**Representation of an existing Folder in the Java pkg tree.
 */
public class JavaSrcTreePkg implements SortedTree<JavaSrcTreePkg>, JavaSources.ClassDataOrJavaSrcFile
{
  private final String sPkgPath;
  
  private final String sPkgName;
  
  private TreeMap<String, JavaSrcTreePkg> subPkgs;
  
  /**All file.java containing in this package. */
  private final TreeMap<String, JavaSrcTreeFile> javaFiles = new TreeMap<String, JavaSrcTreeFile>();

  
  /**Same content as {@link #javaFiles}, but the key is the class name. */
  //private final TreeMap<String, JavaSrcTreeFile> javaPublicClasses = new TreeMap<String, JavaSrcTreeFile>();
  private final List<JavaSources.ClassDataOrJavaSrcFile> javaPublicClasses = new LinkedList<JavaSources.ClassDataOrJavaSrcFile>();
  
  /**Set to true if {@link #setFileJava(String, File, String, String, String, boolean)}
   * is called the first time. */
  private boolean pkgHasFiles = false;
  
  /**Index of all classes available in the package, package privates too.  */
  private final LocalIdents pkgIdents;

  
  /**
   * @param pkgIdent
   * @param javaSrcPath
   */
  public JavaSrcTreePkg(String pkgPath, String pkgName) //, LocalIdents identsToAssignPkgIdents)
  { sPkgPath = pkgPath;
    sPkgName = pkgName;
    //pkgIdents = identsToAssignPkgIdents != null ? identsToAssignPkgIdents : new LocalIdents(); 
    pkgIdents = new LocalIdents(); 
  }
  
  

  /**returns the Folder to the given package name. If the Folder doesn't exist, it creates it.
   * @param pkgPath
   * @param pkgName
   * @param identsToAssignPkgIdents if not <code>null</code> than this LocalIdents are used
   *        as pkgIdents. It is for the standard packages java.lang, java.util.
   * @return
   */
  public JavaSrcTreePkg getOrAddPkg(String pkgPath, String pkgName) //, LocalIdents identsToAssignPkgIdents)
  {
    if(subPkgs == null){ subPkgs = new TreeMap<String, JavaSrcTreePkg>(); }
    JavaSrcTreePkg ret = subPkgs.get(pkgName);
    if(ret == null){
      ret = new JavaSrcTreePkg(pkgPath, pkgName); //, identsToAssignPkgIdents);
      subPkgs.put(pkgName,ret);
      //javaPublicClasses.putClassType(pkgName, ret);
      this.pkgIdents.putClassType(pkgName, ret);   //to find as identifier while translating.
    }
    return ret;
  }
  
  
  public String getPkgPath(){ return sPkgPath; }
  
  public final TreeMap<String, JavaSrcTreeFile> getJavaFiles()
  {
    return javaFiles;
  }



  public final List<JavaSources.ClassDataOrJavaSrcFile> getPublicClasses()
  {
    return javaPublicClasses;
  }



  /**Adds all standard types to the package. This method will be called if any file is containing
   * in the package. It means, the package is the immediate package of any class. 
   * This method will be called too while creating a standard package (java.lang).
   * @param types
   */
  public void xxxaddStdTypes(LocalIdents types)
  {
    //javaPublicClasses.putClassTypesAll(types);
    pkgIdents.putClassTypesAll(types);
  }
  
  
  /**Registers the first occurrence of a found Java file.
   * @param itsPkg The package where the file is member of.
   * @param javaSrcPath The source path where the file is found first time.
   * @param fileJava The File instance ready to open source file.
   * @param sFileNameJava The file name without path but with extension <code>.java</code> without directories.
   * @param prefixCFile The prefix to build the C class name and the C file name. 
   *        The part after the last slash <code>/</code> is the prefix for class names.
   *        The prefix should ended with slash if the file name has no prefix, 
   *        but a path to store the to create *.c and *.h files are given.
   *        The prefix may contain <code>""</code>.
   * @param suffixCFile The suffix to build the C class name and the C file name.
   * @param stcPath null or a special path where the stc-file is located.
   * @param translateToC false than the file should not be translate to C,
   *        instead the stc-file should be used any time. This is if it is a standard class 
   *        or a users class, which is located in a library.
   * @return
   */
  public JavaSrcTreeFile setFileJava(String javaSrcPath, File fileJava, String sFileNameJava
  , String prefixCFile, String suffixCFile, String prefixNames, String suffixNames, String stcPath
  , boolean translateToC
  )//(String javaSrcPath, File fileJava, String nameFileJava, String pkgPath, String pathC, boolean bToTranslateToC)
  { JavaSrcTreeFile javaSrc;
    if(!pkgHasFiles){
      /** First call of this method, add all standard types at first. */
      pkgHasFiles =true;
    }
        
    javaSrc = javaFiles.get(sFileNameJava);  //check whether the file is known yet.
    if( javaSrc== null){
      /**register only the first occurrence of a Java file. If the file is located more than one time,
       * the first occurrence should be used.
       */
      javaSrc = new JavaSrcTreeFile( this, javaSrcPath, fileJava, sFileNameJava
                                   , prefixCFile, suffixCFile, prefixNames, suffixNames
                                   , stcPath, translateToC);
      javaFiles.put(sFileNameJava, javaSrc);
      /**assume that the file contains a public class and put the name in type list, without yet ClassData. */
      javaPublicClasses.add(javaSrc);
      //TODO: a first first pass is necessary, with parsing file, to get all classes.
      //up to now: only one class per file.
      pkgIdents.putClassType(javaSrc.getPublicClassName(), javaSrc);
    }
    else{
      javaSrc.infoSecondFile(fileJava);
    }
    return javaSrc;
  }


  public void reportClasses(Report report)
  {
    if(javaFiles != null){
      Set<Map.Entry<String, JavaSrcTreeFile>> entries = javaFiles.entrySet();
      for(Map.Entry<String, JavaSrcTreeFile> entry: entries){
        JavaSrcTreeFile javaFile= entry.getValue();
        report.reportln(Report.fineInfo, javaFile.reportInfo());
      }
    }
  }
  

  public JavaSrcTreePkg getChild(String key)
  { return subPkgs.get(key);
  }

  LocalIdents getPkgLevelIdents(){ return pkgIdents; }

  public Iterator<JavaSrcTreePkg> iterChildren()
  {
    // TODO Auto-generated method stub
    return null;
  }



  public Iterator<JavaSrcTreePkg> iterChildren(String key)
  {
    // TODO Auto-generated method stub
    return null;
  }



  public List<JavaSrcTreePkg> listChildren()
  {
    List<JavaSrcTreePkg> ret = new LinkedList<JavaSrcTreePkg>();
    if(subPkgs != null){
      Set<Map.Entry<String, JavaSrcTreePkg>> entries = subPkgs.entrySet();
      for(Map.Entry<String, JavaSrcTreePkg> entry: entries){
        JavaSrcTreePkg folder= entry.getValue();
        ret.add(folder);
      }
    }  
    return ret;
  }



  public List<JavaSrcTreePkg> listChildren(String key)
  {
    // TODO Auto-generated method stub
    return null;
  }



  public ClassData getClassData()
  {
    return null;
  }



  /**Implements {@link JavaSources.ClassDataOrJavaSrcFile#getJavaSrc()}
   */
  public JavaSrcTreeFile getJavaSrc()
  {
    return null; // it isn't a JavaSrcTreePkg
  }

  /**Implements {@link JavaSources.ClassDataOrJavaSrcFile#getJavaPkg()}
   */
  public JavaSrcTreePkg getJavaPkg()
  {
    return this; 
  }
  



  /* (non-Javadoc)
   * @see org.vishia.java2C.JavaSources.ClassDataOrJavaSrcFile#getLocalIdents()
   */
  public LocalIdents getLocalIdents()
  {
    return pkgIdents;
  }
  
  /**Implements {@link JavaSources.ClassDataOrJavaSrcFile#getTypeName()}
   */
  public String getTypeName()
  {
    return sPkgName;
  }
  
  
  
}
