package org.vishia.util;

import java.io.BufferedReader;
import java.io.IOException;

/**This class helps to convert a special ASCII format in a byte[]. The input represents a 16-bit-word in memory
 * with 3 chars. 
 * @author e09srrh0
 *
 */
public class ReadChars3Stream
{
  
  /**
   * @param data Output array to write data.
   * @param idxStart Start index in data to write
   * @param input Input stream starting with a line containing 3-char-words.
   *              The terminated line should start with <code>&&&</code>.
   * @throws IOException if any io-operation failed.
   */
  public static void read(byte[] data, int idxStart, BufferedReader input) throws IOException
  { boolean bCont = true;
    int ixData = idxStart;
    while(bCont && input.ready()){
      String sLine = input.readLine();
      if(sLine.startsWith("&&&")){
        bCont = false;
      }
      else {
        int zLine = sLine.length();
        int ixLine = 0;
        while(ixLine < zLine){
          short value = convert3Chars(sLine.substring(ixLine, ixLine+3));
          data[ixData++] = (byte)(value);
          data[ixData++] = (byte)(value>>8);
          ixLine += 3;
        }
      }  
    }
      
    if(ixData != data.length){
      throw new IllegalArgumentException("to less data: " + data.length + "expected, " + ixData + " got. ");
    }
  }
  
  
  private static short convert3Chars(String s3)
  {
    char cc;
    int charVal;
    int val = 0;
    short nCheckSoll;
    
    cc = s3.charAt(0);
    charVal = cc - '0';
    if(charVal >= 0x40) throw new IllegalArgumentException("fault first char " + s3);
    val = charVal;
    
    cc = s3.charAt(1);
    charVal = cc - '0';
    if(charVal >= 0x40) throw new IllegalArgumentException("fault second char " + s3);
    val = val | (charVal<<6);
    
    cc = s3.charAt(2);
    charVal = cc - '0';
    if(charVal >= 0x40) throw new IllegalArgumentException("fault third char " + s3);
    nCheckSoll = (short)(charVal >>4); 
    val = val | (charVal<<12);
    
    { /**calculate check code:*/
      short nCheck=0;
      short nBits=(short)val;
      { int ii; for(ii=0; ii<7; ii++){ nCheck+=nBits; nBits>>=2;} //checksumm 2 bit
      }
      nCheck&=3;   //2 bit Checksumm
      if(nCheckSoll != nCheck) throw new IllegalArgumentException("checksum error");
    }
    return (short)val;
    
  }
  

}
